// AppObjectCtl.cpp : Implementation of CAppObject
#include "stdafx.h"
#include "AppObject.h"
#include "AppObjectCtl.h"

/////////////////////////////////////////////////////////////////////////////
// CAppObject

STDMETHODIMP CAppObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IAppObject
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CAppObject::get_Document(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Copy in our document's IDispatch pointer
			VARIANT var;
			var.vt = VT_DISPATCH;
			var.pdispVal = m_pDocument.p;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CAppObject::get_Height(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				VARIANT var;
				var.vt = VT_I4;
				var.lVal = rcWnd.bottom - rcWnd.top;
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::put_Height(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				// Pull the new value
				int iHeight = var.lVal;
				if ( !::MoveWindow(m_hWnd,rcWnd.left,rcWnd.top,rcWnd.right-rcWnd.left,iHeight,TRUE) ) {
					// Unable to move window
					hr = E_FAIL;
				} // if
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::get_Width(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				VARIANT var;
				var.vt = VT_I4;
				var.lVal = rcWnd.right - rcWnd.left;
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::put_Width(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				// Pull the new value
				int iWidth = var.lVal;
				if ( !::MoveWindow(m_hWnd,rcWnd.left,rcWnd.top,iWidth,rcWnd.bottom-rcWnd.top,TRUE) ) {
					// Unable to move window
					hr = E_FAIL;
				} // if
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::get_Top(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				VARIANT var;
				var.vt = VT_I4;
				var.lVal = rcWnd.top;
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::put_Top(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				// Pull the new value
				int iTop = var.lVal;
				if ( !::MoveWindow(m_hWnd,rcWnd.left,iTop,rcWnd.right-rcWnd.left,rcWnd.bottom-iTop,TRUE) ) {
					// Unable to move window
					hr = E_FAIL;
				} // if
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::get_Left(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				VARIANT var;
				var.vt = VT_I4;
				var.lVal = rcWnd.left;
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::put_Left(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current dimensions
				RECT rcWnd;
				::GetWindowRect(m_hWnd,&rcWnd);

				// Pull the new value
				int iLeft = var.lVal;
				if ( !::MoveWindow(m_hWnd,iLeft,rcWnd.top,rcWnd.right-iLeft,rcWnd.bottom-rcWnd.top,TRUE) ) {
					// Unable to move window
					hr = E_FAIL;
				} // if
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::get_WindowState(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			::VariantClear(pVal);

			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine current window state
				WINDOWPLACEMENT wp;
				::ZeroMemory(&wp,sizeof(wp));

				::GetWindowPlacement(m_hWnd,&wp);
				VARIANT var;
				var.vt = VT_I4;
				var.lVal = wp.showCmd;
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::put_WindowState(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Pull the new state and change the window
				// state
				::ShowWindow(m_hWnd,var.lVal);
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::get_Caption(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Retrieve the caption
				CComBSTR bstrCaption;
#ifndef _UNICODE
				TCHAR szCaption[_MAX_PATH+1] = {0};
				::GetWindowText(m_hWnd,szCaption,_MAX_PATH);
#else
				::GetWindowText(m_hWnd,bstrCaption,_MAX_PATH);
#endif
				::VariantClear(pVal);
				VARIANT var;
				var.vt = VT_BSTR;
				var.bstrVal = ::SysAllocString(bstrCaption); // make new copy...
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::put_Caption(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_BSTR,&newVal)) ) {
			if ( !::IsWindow(m_hWnd) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Pull the new caption
				CComBSTR bstrCaption(newVal.bstrVal);
#ifndef _UNICODE
				// For non-Unicode versions, we must convert the
				// BSTR value to TCHAR.
				USES_CONVERSION;
				LPTSTR szCaption = OLE2T(newVal.bstrVal);
				::SetWindowText(m_hWnd,szCaption);
#else
				// For Unicode versions, we can simply use the
				// BSTR.
				::SetWindowText(m_hWnd,bstrCaption);
#endif
			} // else
		} // if
		else {
			// Wasn't some sort of variant we could
			// coerce
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::SetDocument(LPDISPATCH newVal)
{
	HRESULT hr = S_OK;
	try {
		// Accept new value
		if ( newVal == NULL ) {
			// Invalid pointer
			ATLASSERT(newVal != NULL);
			hr = E_POINTER;
		} // if
		else {
			// Set the pointer
			m_pDocument = newVal;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::Quit()
{
	// This one's easy...we simply send our HWND a
	// WM_CLOSE message...
	HRESULT hr = S_OK;
	try {
		if ( !::IsWindow(m_hWnd) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Send WM_CLOSE
			::SendMessage(m_hWnd,WM_CLOSE,(WPARAM)0,(LPARAM)0);
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CAppObject::InitHwnd(OLE_HANDLE hWnd)
{
	HRESULT hr = S_OK;
	try {
		if ( hWnd ) {
			// Initialize the window handle attribute
			if ( !::IsWindow(reinterpret_cast<HWND>(hWnd)) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Assign our attribute
				m_hWnd = reinterpret_cast<HWND>(hWnd);

				// Subclass the window
				if ( !m_CMFCView.SubclassWindow(m_hWnd) ) {
					// We failed to subclass the window!
					ATLASSERT(FALSE);
					hr = E_UNEXPECTED;
				} // if
			} // else
		} // if
		else {
			// Invalid HWND
			hr = E_INVALIDARG;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}
