// DIBHolder.cpp : implementation of the CDIBHolder class
//

#include "stdafx.h"
#include "DIBHolder.h"
#include "windowsx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView construction/destruction

CDIBHolder::CDIBHolder() :
	m_pDIB(NULL),
	m_pbmFileHeader(NULL),
	m_pbmInfoHeader(NULL),
	m_pbmInfo(NULL),
	m_pRGBTable(NULL),
	m_pDIBBits(NULL),
	m_iNumColors(0)
{
	// TODO: More?
}

CDIBHolder::CDIBHolder(const char* szFileame) :
	m_pDIB(NULL),
	m_pbmFileHeader(NULL),
	m_pbmInfoHeader(NULL),
	m_pbmInfo(NULL),
	m_pRGBTable(NULL),
	m_pDIBBits(NULL),
	m_iNumColors(0)
{
    // Load the DIB
    LoadDIBFile(szFileame);
}

CDIBHolder::~CDIBHolder()
{
    // Free the memory assigned to the bitmap.
    delete [] m_pDIB;
}

/////////////////////////////////////////////////////////////////////////////
// CDIBHolder member functions

void CDIBHolder::LoadDIBFile(const char* szFilename)
{
	// If there is an existing bitmap in memory, delete
	// it.
	if ( m_pDIB != NULL ) {
		// Delete the bitmap
		delete [] m_pDIB;
	} // if

	// Reset our class attributes
	m_pbmFileHeader = NULL;
	m_pbmInfoHeader = NULL;
	m_pbmInfo = NULL;
	m_pRGBTable = NULL;
	m_pDIBBits = NULL;
	m_iNumColors = 0;

    // Open the specified DIB file
    CFile fileDIB(szFilename,CFile::modeRead);
    
    // Read the bitmap's header
    BITMAPFILEHEADER bmFileHeader;
    fileDIB.Read((void*)&bmFileHeader,sizeof(bmFileHeader));

    // Is it really a bitmap?
	if ( bmFileHeader.bfType != 0x4d42 ) {
		// Nope...
		AfxMessageBox("The specified file is not a bitmap.",
			MB_OK | MB_ICONERROR);
    } // if
	else {
		// It is a bitmap file, so calculate the size
		// of the actual DIB, which is the file size
		// less the size of the bitmap file header.
		DWORD dwLength = fileDIB.GetLength();    
		DWORD dwDIBSize = dwLength - sizeof(bmFileHeader);

        // Allocate the memory for the bitmap color information
        m_pDIB = new BYTE[dwDIBSize];
        
        // Read the bitmap into memory and close the file.
        fileDIB.Read((void*)m_pDIB,dwDIBSize);
        fileDIB.Close();

        // Initialize the bitmap's BITMAPINFO and BITMAPINFOHEADER
		// structure pointers.
        m_pbmInfo = (LPBITMAPINFO)m_pDIB;
		ASSERT(m_pbmInfo != NULL); // should not be NULL!
        m_pbmInfoHeader = (LPBITMAPINFOHEADER)m_pDIB;
		ASSERT(m_pbmInfoHeader != NULL); // should not be NULL!
        
        // Determine the bitmap's color table location.
        m_pRGBTable = (RGBQUAD*)(m_pDIB + m_pbmInfoHeader->biSize);

        // How many colors?
        int m_iNumColors = GetNumColors();

		// Calculate the bitmap image's size.
		m_pbmInfoHeader->biSizeImage = GetSize();
        
		// Complete the biClrUsed field
		if ( m_pbmInfoHeader->biClrUsed == 0 ) {
			// It's currently invalid, so fill it
			m_pbmInfoHeader->biClrUsed = m_iNumColors;
		} // if

		// Calculate the location of the bitmap's color data
		DWORD dwClrTableSize = m_iNumColors * sizeof(RGBQUAD);
		m_pDIBBits =
            m_pDIB + m_pbmInfoHeader->biSize + dwClrTableSize;
	} // else
}

DWORD CDIBHolder::GetSize()
{
	DWORD dwSize = 0;

	// If the bitmap's biSizeImage field is invalid,
	// we need to figure out how big it is ourselves.
	if ( !m_pbmInfoHeader->biSizeImage ) {
		// How wide?
		DWORD dwWidth = (DWORD)Width();
        
        // How tall?
		DWORD dwHeight = (DWORD)Height();
        
		// Multiply width times height to yield
		// total number of bytes required to store
		// this image.
		dwSize = dwWidth * dwHeight;
	} // if
	else {
		// Return the size stored in the BITMAPINFOHEADER
		dwSize = m_pbmInfoHeader->biSizeImage;
	} // else

	return dwSize;
}

UINT CDIBHolder::Width()
{
	// Simply return the width
    return (UINT)m_pbmInfoHeader->biWidth;
}
        
UINT CDIBHolder::Height()
{
	// Simply return the height
    return (UINT)m_pbmInfoHeader->biHeight;
}
        
UINT CDIBHolder::GetNumColors()
{
	int iNumColors = 0;

    if ((m_pbmInfoHeader->biClrUsed == 0) &&
		(m_pbmInfoHeader->biBitCount < 16)) {
		// 256 (or less) colors
        iNumColors = 1 << m_pbmInfoHeader->biBitCount;
	} // if
    else {
		// Pull the color information directly from
		// the bitmap.
        iNumColors = m_pbmInfoHeader->biClrUsed;
	} // if

	return iNumColors;
}
