// DocObjectCtl.cpp : Implementation of CDocObject
#include "stdafx.h"
#include "DocObject.h"
#include "DocObjectCtl.h"
#include <afxres.h> // ID_FILE_SAVE/PRINT, etc...

/////////////////////////////////////////////////////////////////////////////
// CDocObject

STDMETHODIMP CDocObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDocObject
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CDocObject::get_Application(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Copy in our application's IDispatch pointer
			VARIANT var;
			var.vt = VT_DISPATCH;
			var.pdispVal = m_pAppObject.p;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CDocObject::get_TextSelection(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Copy in our text selection's IDispatch pointer
			VARIANT var;
			var.vt = VT_DISPATCH;
			var.pdispVal = m_pTextSelObject.p;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CDocObject::get_Dirty(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		ATLASSERT(pVal != NULL);
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our modified status
				BOOL bIsDirty = (BOOL)::SendMessage(m_hWndEdit,EM_GETMODIFY,(WPARAM)0,(LPARAM)0);

				VARIANT var;
				var.vt = VT_BOOL;
				var.boolVal = bIsDirty;
				::VariantCopy(pVal,&var);
			} // else
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::put_Dirty(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_BOOL,&newVal)) ) {
			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Pull the new value
				BOOL bIsDirty = var.boolVal;
				::SendMessage(m_hWndEdit,EM_SETMODIFY,(WPARAM)bIsDirty,(LPARAM)0);
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::SetApplication(LPDISPATCH newVal)
{
	HRESULT hr = S_OK;
	try {
		// Accept new value
		if ( newVal == NULL ) {
			// Invalid pointer
			ATLASSERT(newVal != NULL);
			hr = E_POINTER;
		} // if
		else {
			// Set the pointer
			m_pAppObject = newVal;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::SetTextSelection(LPDISPATCH newVal)
{
	HRESULT hr = S_OK;
	try {
		// Accept new value
		if ( newVal == NULL ) {
			// Invalid pointer
			ATLASSERT(newVal != NULL);
			hr = E_POINTER;
		} // if
		else {
			// Set the pointer
			m_pTextSelObject = newVal;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::New()
{
	HRESULT hr = S_OK;
	try {
		// Tell the view to save the file...
		if ( !::IsWindow(m_hWndView) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndView,WM_COMMAND,(WPARAM)ID_FILE_NEW,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::Open()
{
	HRESULT hr = S_OK;
	try {
		// Tell the view to save the file...
		if ( !::IsWindow(m_hWndView) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndView,WM_COMMAND,(WPARAM)ID_FILE_OPEN,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::SaveAs()
{
	HRESULT hr = S_OK;
	try {
		// Tell the view to save the file...
		if ( !::IsWindow(m_hWndView) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndView,WM_COMMAND,(WPARAM)ID_FILE_SAVE_AS,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::Save()
{
	HRESULT hr = S_OK;
	try {
		// Tell the view to save the file...
		if ( !::IsWindow(m_hWndView) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndView,WM_COMMAND,(WPARAM)ID_FILE_SAVE,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::Print()
{
	HRESULT hr = S_OK;
	try {
		// Tell the view to save the file...
		if ( !::IsWindow(m_hWndView) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndView,WM_COMMAND,(WPARAM)ID_FILE_PRINT,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::Clear()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndEdit,EM_SETSEL,(WPARAM)0,(LPARAM)-1);
			TCHAR szReplace[] = {_T("")};
			::SendMessage(m_hWndEdit,EM_REPLACESEL,(WPARAM)FALSE,(LPARAM)szReplace);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::InitViewHwnd(OLE_HANDLE hWnd)
{
	HRESULT hr = S_OK;
	try {
		if ( hWnd ) {
			// Initialize the window handle attribute
			if ( !::IsWindow(reinterpret_cast<HWND>(hWnd)) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Assign our attribute
				m_hWndView = reinterpret_cast<HWND>(hWnd);
			} // else
		} // if
		else {
			// Invalid HWND
			hr = E_INVALIDARG;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CDocObject::InitEditHwnd(OLE_HANDLE hWnd)
{
	HRESULT hr = S_OK;
	try {
		if ( hWnd ) {
			// Initialize the window handle attribute
			if ( !::IsWindow(reinterpret_cast<HWND>(hWnd)) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Assign our attribute
				m_hWndEdit = reinterpret_cast<HWND>(hWnd);

				// Subclass the window
				if ( !m_CEditCtrl.SubclassWindow(m_hWndEdit) ) {
					// We failed to subclass the window!
					ATLASSERT(FALSE);
					hr = E_UNEXPECTED;
				} // if
			} // else
		} // if
		else {
			// Invalid HWND
			hr = E_INVALIDARG;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}
