// DocObjectCtl.h : Declaration of the CDocObject

#ifndef __DOCOBJECT_H_
#define __DOCOBJECT_H_

#include "resource.h"       // main symbols
#include "DocObjectCP.h"

/////////////////////////////////////////////////////////////////////////////
// CDocObject
class ATL_NO_VTABLE CDocObject : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CDocObject, &CLSID_DocObject>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CDocObject>,
	public IDispatchImpl<IDocObject, &IID_IDocObject, &LIBID_DOCOBJECTLib>,
	public IObjectSafetyImpl<CDocObject, INTERFACESAFE_FOR_UNTRUSTED_CALLER|INTERFACESAFE_FOR_UNTRUSTED_DATA>,
	public CProxy_IDocObjectEvents< CDocObject >,
	public CComControl<CDocObject>,
	public IProvideClassInfo2Impl<&CLSID_DocObject, &DIID__IDocObjectEvents>
{
public:
	CDocObject() :
		m_bIsDirty(FALSE),
		m_hWndView(NULL),
		m_hWndEdit(NULL),
		m_CEditCtrl(NULL,this,1)
	{
	}

	void FinalRelease()
	{
		// Unsubclass the edit control
		m_CEditCtrl.UnsubclassWindow();
	}

DECLARE_REGISTRY_RESOURCEID(IDR_DOCOBJECT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CDocObject)
	COM_INTERFACE_ENTRY(IDocObject)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IObjectSafety)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CDocObject)
	CONNECTION_POINT_ENTRY(DIID__IDocObjectEvents)
END_CONNECTION_POINT_MAP()

BEGIN_MSG_MAP(CDocObject)
	// Message map
ALT_MSG_MAP(1)
	// Alternate message map
	MESSAGE_HANDLER(EM_REPLACESEL, OnReplaceSel)
	MESSAGE_HANDLER(EM_SETMODIFY, OnSetModify)
	MESSAGE_HANDLER(WM_CHAR, OnChar)
	MESSAGE_HANDLER(WM_SETTEXT, OnSetText)
	MESSAGE_HANDLER(WM_PASTE, OnPaste)

END_MSG_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IDocObject
public:
	STDMETHOD(InitEditHwnd)(/*[in]*/ OLE_HANDLE hWnd);
	STDMETHOD(InitViewHwnd)(/*[in]*/ OLE_HANDLE hWnd);
	STDMETHOD(Clear)();
	STDMETHOD(Print)();
	STDMETHOD(Save)();
	STDMETHOD(SaveAs)();
	STDMETHOD(Open)();
	STDMETHOD(New)();
	STDMETHOD(SetTextSelection)(/*[in]*/ LPDISPATCH newVal);
	STDMETHOD(SetApplication)(/*[in]*/ LPDISPATCH newVal);
	STDMETHOD(get_Dirty)(/*[out, retval]*/ VARIANT *pVal);
	STDMETHOD(put_Dirty)(/*[in]*/ VARIANT newVal);
	STDMETHOD(get_TextSelection)(/*[out, retval]*/ VARIANT *pVal);
	STDMETHOD(get_Application)(/*[out, retval]*/ VARIANT *pVal);

// Subclassed edit control message handlers
protected:
	LRESULT FireDirty(UINT iMsg, WPARAM wParam, LPARAM lParam)
	{
		if ( !m_bIsDirty ) {
			// Set our new state and fire the event...
			m_bIsDirty = TRUE;
			Fire_IsDirty();
		} // if
		
		// Allow the control to handle the character
		return m_CEditCtrl.DefWindowProc(iMsg,wParam,lParam);
	}
	LRESULT OnSetModify(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		// Caller is forcing dirty bit
		m_bIsDirty = !wParam ? FALSE : TRUE;
		return FireDirty(EM_SETMODIFY,wParam,lParam);
	}
	LRESULT OnReplaceSel(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return FireDirty(EM_REPLACESEL,wParam,lParam);
	}
	LRESULT OnChar(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return FireDirty(WM_CHAR,wParam,lParam);
	}
	LRESULT OnSetText(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return FireDirty(WM_SETTEXT,wParam,lParam);
	}
	LRESULT OnPaste(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return FireDirty(WM_PASTE,wParam,lParam);
	}

// Attributes
protected:
	CComPtr<IDispatch> m_pAppObject;
	CComPtr<IDispatch> m_pTextSelObject;
	HWND m_hWndView;
	HWND m_hWndEdit;
	CContainedWindow m_CEditCtrl;
	BOOL m_bIsDirty;
};

#endif //__DOCOBJECT_H_
