// GLObjectView.cpp : implementation of the CGLObjectView class
//

#include "stdafx.h"
#include "GLObject.h"
#include <math.h>

#include "GLObjectDoc.h"
#include "GLObjectView.h"

#include "ObjectPage.h"
#include "LightPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGLObjectView

IMPLEMENT_DYNCREATE(CGLObjectView, CGLView)

BEGIN_MESSAGE_MAP(CGLObjectView, CGLView)
	//{{AFX_MSG_MAP(CGLObjectView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CGLView::OnFilePrintPreview)
	// Custom OpenGL notifications
	ON_NOTIFY(OBJECT_NOTIFY_OBJECTCHANGE, IDD_OBJECT, OnChangeObject)
	ON_NOTIFY(OBJECT_NOTIFY_RENDERCHANGE, IDD_OBJECT, OnChangeRender)
	ON_NOTIFY(OBJECT_NOTIFY_SCALECHANGE, IDD_OBJECT, OnChangeScale)
	ON_NOTIFY(OBJECT_NOTIFY_AXESCHANGE, IDD_OBJECT, OnChangeAxes)
	ON_NOTIFY(LIGHT_NOTIFY_PROPSCHANGE, IDD_LIGHT, OnChangeProps)
	ON_NOTIFY(LIGHT_NOTIFY_SPOTCHANGE, IDD_LIGHT, OnChangeSpot)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGLObjectView construction/destruction

CGLObjectView::CGLObjectView() :
	m_iObject(0),
	m_bDrawAxes(FALSE),
	m_bDoSpotlight(FALSE),
	m_iRenderStyle(0),
	m_fScaleX(1.0f),
	m_fScaleY(1.0f),
	m_fScaleZ(1.0f),
	m_fXRotate(0.0f),
	m_fYRotate(0.0f),
	m_fZRotate(0.0f),
	m_fZTranslate(0.0f),
	m_fMaterialShininess(100.0f),
	m_bMouseActive(FALSE)
{
	// Establish our initial light parameters, starting with
	// the ambient light conditions.
	m_fLightAmbient0[0] = 0.2f; // red component
	m_fLightAmbient0[1] = 0.2f; // green component
	m_fLightAmbient0[2] = 0.2f; // blue component
	m_fLightAmbient0[3] = 1.0f; // alpha

	// Next the diffuse light parameter
	m_fLightDiffuse0[0] = 0.7f; // red component
	m_fLightDiffuse0[1] = 0.7f; // green component
	m_fLightDiffuse0[2] = 0.7f; // blue component
	m_fLightDiffuse0[3] = 1.0f; // alpha

	// Then, the specular light parameter
	m_fLightSpecular0[0] = 0.0f; // red component
	m_fLightSpecular0[1] = 0.0f; // green component
	m_fLightSpecular0[2] = 0.0f; // blue component
	m_fLightSpecular0[3] = 1.0f; // alpha

	// And finally, its position
	m_fLightPosition0[0] = 0.0f; // X
	m_fLightPosition0[1] = 0.0f; // Y
	m_fLightPosition0[2] = 30.0f; // Z
	m_fLightPosition0[3] = 1.0f; // W (positional light)

	// The floodlight
	//
	// Establish our initial light parameters, starting with
	// the ambient light conditions.
	m_fLightAmbient1[0] = 0.2f; // red component
	m_fLightAmbient1[1] = 0.2f; // green component
	m_fLightAmbient1[2] = 0.2f; // blue component
	m_fLightAmbient1[3] = 1.0f; // alpha

	// Next the diffuse light parameter
	m_fLightDiffuse1[0] = 0.7f; // red component
	m_fLightDiffuse1[1] = 0.7f; // green component
	m_fLightDiffuse1[2] = 0.7f; // blue component
	m_fLightDiffuse1[3] = 1.0f; // alpha

	// Then, the specular light parameter
	m_fLightSpecular1[0] = 1.0f; // red component
	m_fLightSpecular1[1] = 1.0f; // green component
	m_fLightSpecular1[2] = 1.0f; // blue component
	m_fLightSpecular1[3] = 1.0f; // alpha

	// And finally, its position.  This position places the
	// light such that it reflects off of the surface when 
	//the object is initially shown (assuming no rotation).
	m_fLightPosition1[0] = 0.0f; // X
	m_fLightPosition1[1] = 15.0f; // Y
	m_fLightPosition1[2] = 30.0f; // Z
	m_fLightPosition1[3] = 1.0f; // W (positional light)

	// Establish our material parameters
	m_fMaterialAmbient[0] = 0.7f; // initially a gray color
	m_fMaterialAmbient[1] = 0.7f;
	m_fMaterialAmbient[2] = 0.7f;
	m_fMaterialAmbient[3] = 1.0f;
	m_fMaterialEmitted[0] = 0.1f; // all specular content
	m_fMaterialEmitted[1] = 0.1f;
	m_fMaterialEmitted[2] = 0.1f;
	m_fMaterialEmitted[3] = 1.0f;
	m_fMaterialSpecular[0] = 1.0f; // all specular content
	m_fMaterialSpecular[1] = 1.0f;
	m_fMaterialSpecular[2] = 1.0f;
	m_fMaterialSpecular[3] = 1.0f;
}

CGLObjectView::~CGLObjectView()
{
}

BOOL CGLObjectView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CGLView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGLObjectView drawing

void CGLObjectView::OnDraw(CDC* pDC)
{
	// Call our base class
	CGLObjectDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Set our palette
	if ( m_hPalette != NULL ) {
	    SelectPalette(pDC->m_hDC, m_hPalette, FALSE);
	    RealizePalette(pDC->m_hDC);
	} // if

	// Draw the OpenGL scene
    wglMakeCurrent(pDC->m_hDC, m_hRC);
    RenderView();
	SwapBuffers(pDC->m_hDC);
    wglMakeCurrent(pDC->m_hDC, NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CGLObjectView diagnostics

#ifdef _DEBUG
void CGLObjectView::AssertValid() const
{
	CGLView::AssertValid();
}

void CGLObjectView::Dump(CDumpContext& dc) const
{
	CGLView::Dump(dc);
}

CGLObjectDoc* CGLObjectView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGLObjectDoc)));
	return (CGLObjectDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGLObjectView helpers

void CGLObjectView::RenderView()
{
	// Clear the viewport
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Perform an Z translation
	glTranslatef(0.0f, 0.0f, m_fZTranslate);

	glPushMatrix();

		// Rotate and scale
		glScalef(m_fScaleX,m_fScaleY,m_fScaleZ);
		glRotatef(m_fXRotate, 1.0f, 0.0f, 0.0f);
		glRotatef(m_fYRotate, 0.0f, 1.0f, 0.0f);
		glRotatef(m_fZRotate, 0.0f, 0.0f, 1.0f);

		// Determine which object to draw
		switch ( m_iObject ) {
			case 0: // cube
				// Determine which rendering style to apply
				switch ( m_iRenderStyle ) {
					case 0: // wireframe
						RenderWireCube();
						break;

					case 1: // solid
						RenderSolidCube();
						break;

					case 2: // shaded
						RenderShadedCube();
						break;

					default:
						ASSERT(FALSE);
						break;

				} // switch
				break;

			case 1: // sphere
				// Determine which rendering style to apply
				switch ( m_iRenderStyle ) {
					case 0: // wireframe
						RenderWireSphere();
						break;

					case 1: // solid
						RenderSolidSphere();
						break;

					case 2: // shaded
						RenderShadedSphere();
						break;

					default:
						ASSERT(FALSE);
						break;

				} // switch
				break;

			case 2: // torus
				// Determine which rendering style to apply
				switch ( m_iRenderStyle ) {
					case 0: // wireframe
						RenderWireTorus();
						break;

					case 1: // solid
						RenderSolidTorus();
						break;

					case 2: // shaded
						RenderShadedTorus();
						break;

					default:
						ASSERT(FALSE);
						break;

				} // switch
				break;

			case 3: // dodecahedron
				// Determine which rendering style to apply
				switch ( m_iRenderStyle ) {
					case 0: // wireframe
						RenderWireDodec();
						break;

					case 1: // solid
						RenderSolidDodec();
						break;

					case 2: // shaded
						RenderShadedDodec();
						break;

					default:
						ASSERT(FALSE);
						break;

				} // switch
				break;

			default:
				ASSERT(FALSE);
				break;

		} // switch

		// Draw coordinate axes
		if ( m_bDrawAxes ) {
			glDisable(GL_LIGHTING);
			glDisable(GL_LIGHT0);
			glDisable(GL_LIGHT1);

			glColor3f(0.0f, 0.0f, 1.0f); // blue
			glBegin(GL_LINES); // X axis
			   glVertex3f(15.0f, 0.0f, 0.0f);
			   glVertex3f(-15.0f, 0.0f, 0.0f);
			glEnd();

			glColor3f(1.0f, 1.0f, 0.0f); // yellow
			glBegin(GL_LINES); // Y axis
			   glVertex3f(0.0f, 15.0f, 0.0f);
			   glVertex3f(0.0f, -15.0f, 0.0f);
			glEnd();

			glColor3f(1.0f, 0.0f, 0.0f); // red
			glBegin(GL_LINES); // Z axis
			   glVertex3f(0.0f, 0.0f, 15.0f);
			   glVertex3f(0.0f, 0.0f, -15.0f);
			glEnd();
		} // if

	glPopMatrix();

	glFlush();
}

void CGLObjectView::RenderWireSphere()
{
	// Draw as a wireframe (use white always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(1.0f, 1.0f, 1.0f);

	// Use the auxiliary drawing library
	auxWireSphere(7.0f);
}

void CGLObjectView::RenderSolidSphere()
{
	// Draw as a solid (use gray always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(0.8f, 0.8f, 0.8f);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    glEnable(GL_DEPTH_TEST);
	glEnable(GL_POLYGON_SMOOTH);

	// Use the auxiliary drawing library
	auxSolidSphere(7.0f);

	RenderWireSphere();
}

void CGLObjectView::RenderShadedSphere()
{
	// Draw as shaded
	SetupLight();

	// Use the auxiliary drawing library
	auxSolidSphere(7.0f);
}

void CGLObjectView::RenderWireTorus()
{
	// Draw as a wireframe (use white always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(1.0f, 1.0f, 1.0f);

	// Use the auxiliary drawing library
	auxWireTorus(3.0f,7.0f);
}

void CGLObjectView::RenderSolidTorus()
{
	// Draw as a solid (use gray always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(0.8f, 0.8f, 0.8f);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    glEnable(GL_DEPTH_TEST);
	glEnable(GL_POLYGON_SMOOTH);

	// Use the auxiliary drawing library
	auxSolidTorus(3.0f,7.0f);

	RenderWireTorus();
}

void CGLObjectView::RenderShadedTorus()
{
	// Draw as shaded
	SetupLight();

	// Use the auxiliary drawing library
	auxSolidTorus(3.0f,7.0f);
}

void CGLObjectView::RenderWireDodec()
{
	// Draw as a wireframe (use white always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(1.0f, 1.0f, 1.0f);

	// Use the auxiliary drawing library
	auxWireDodecahedron(7.0f);
}

void CGLObjectView::RenderSolidDodec()
{
	// Draw as a solid (use gray always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(0.8f, 0.8f, 0.8f);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    glEnable(GL_DEPTH_TEST);
	glEnable(GL_POLYGON_SMOOTH);

	// Use the auxiliary drawing library
	auxSolidDodecahedron(7.0f);
}

void CGLObjectView::RenderShadedDodec()
{
	// Draw as shaded
	SetupLight();

	// Use the auxiliary drawing library
	auxSolidDodecahedron(7.0f);
}

void CGLObjectView::RenderWireCube()
{
	// Draw as a wireframe (use white always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(1.0f, 1.0f, 1.0f);

	// Draw our cube by hand...
	glBegin(GL_LINES); // front face of cube (4 sides)
		glVertex3f(-7.0f, 7.0f, 7.0f);
		glVertex3f(-7.0f, -7.0f, 7.0f);

		glVertex3f(-7.0f, -7.0f, 7.0f);
		glVertex3f(7.0f, -7.0f, 7.0f);

		glVertex3f(7.0f, -7.0f, 7.0f);
		glVertex3f(7.0f, 7.0f, 7.0f);

		glVertex3f(7.0f, 7.0f, 7.0f);
		glVertex3f(-7.0f, 7.0f, 7.0f);
	glEnd();

	glBegin(GL_LINES); // "back"
		glVertex3f(7.0f, 7.0f, -7.0f);
		glVertex3f(7.0f, -7.0f, -7.0f);

		glVertex3f(7.0f, -7.0f, -7.0f);
		glVertex3f(-7.0f, -7.0f, -7.0f);

		glVertex3f(-7.0f, -7.0f, -7.0f);
		glVertex3f(-7.0f, 7.0f, -7.0f);

		glVertex3f(-7.0f, 7.0f, -7.0f);
		glVertex3f(7.0f, 7.0f, -7.0f);
	glEnd();

	glBegin(GL_LINES); // "right top"
		glVertex3f(7.0f, 7.0f, -7.0f);
		glVertex3f(7.0f, 7.0f, 7.0f);
	glEnd();

	glBegin(GL_LINES); // "left top"
		glVertex3f(-7.0f, 7.0f, -7.0f);
		glVertex3f(-7.0f, 7.0f, 7.0f);
	glEnd();

	glBegin(GL_LINES); // "right bottom"
		glVertex3f(7.0f, -7.0f, -7.0f);
		glVertex3f(7.0f, -7.0f, 7.0f);
	glEnd();

	glBegin(GL_LINES); // "left bottom"
		glVertex3f(-7.0f, -7.0f, -7.0f);
		glVertex3f(-7.0f, -7.0f, 7.0f);
	glEnd();
}

void CGLObjectView::RenderSolidCube()
{
	// Draw as a wireframe (use white always)
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glColor3f(0.8f, 0.8f, 0.8f);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    glEnable(GL_DEPTH_TEST);
	glEnable(GL_POLYGON_SMOOTH);

	glBegin(GL_POLYGON);
		glVertex3f(7.0f,7.0f,7.0f);
		glVertex3f(-7.0f,7.0f,7.0f);            
		glVertex3f(-7.0f,-7.0f,7.0f);           
		glVertex3f(7.0f,-7.0f,7.0f);            
	glEnd();

	glBegin(GL_POLYGON);
		glVertex3f(7.0f,7.0f,-7.0f);
		glVertex3f(7.0f,-7.0f,-7.0f);           
		glVertex3f(-7.0f,-7.0f,-7.0f);          
		glVertex3f(-7.0f,7.0f,-7.0f);           
	glEnd();

	glBegin(GL_POLYGON);
		glVertex3f(-7.0f,7.0f,7.0f);
		glVertex3f(-7.0f,7.0f,-7.0f);           
		glVertex3f(-7.0f,-7.0f,-7.0f);          
		glVertex3f(-7.0f,-7.0f,7.0f);           
	glEnd();

	glBegin(GL_POLYGON);
		glVertex3f(7.0f,7.0f,7.0f);
		glVertex3f(7.0f,-7.0f,7.0f);            
		glVertex3f(7.0f,-7.0f,-7.0f);           
		glVertex3f(7.0f,7.0f,-7.0f);            
	glEnd();

	glBegin(GL_POLYGON);
		glVertex3f(-7.0f,7.0f,-7.0f);
		glVertex3f(-7.0f,7.0f,7.0f);            
		glVertex3f(7.0f,7.0f,7.0f);             
		glVertex3f(7.0f,7.0f,-7.0f);            
	glEnd();

	glBegin(GL_POLYGON);
		glVertex3f(-7.0f,-7.0f,-7.0f);
		glVertex3f(7.0f,-7.0f,-7.0f);           
		glVertex3f(7.0f,-7.0f,7.0f);            
		glVertex3f(-7.0f,-7.0f,7.0f);           
	glEnd();

	// Since we're not using light, it's hard to distinguish
	// the sides, so we'll overlay the wireframe model to
	// make it easier to see...
	RenderWireCube();
}

void CGLObjectView::Normal( GLfloat* pt1, GLfloat* pt2, GLfloat* pt3,  double* nml )
{
	// Form two vectors from the given input points
	double v1[3], v2[3];
	v1[0] = pt2[0] - pt1[0];
	v1[1] = pt2[1] - pt1[1];
	v1[2] = pt2[2] - pt1[2];
	v2[0] = pt3[0] - pt1[0];
	v2[1] = pt3[1] - pt1[1];
	v2[2] = pt3[2] - pt1[2];

	// Take their cross product
	nml[0] = v1[1] * v2[2] - v1[2] * v2[1];
	nml[1] = v1[2] * v2[0] - v1[0] * v2[2];
	nml[2] = v1[0] * v2[1] - v1[1] * v2[0];

	// Normalize the resultant
	double dLen = sqrt(nml[0]*nml[0] + nml[1]*nml[1] + nml[2]*nml[2]);
	if ( dLen != 0.0 ) {
		nml[0] = nml[0] / dLen;
		nml[1] = nml[1] / dLen;
		nml[2] = nml[2] / dLen;
	} // if
}

void CGLObjectView::RenderShadedCube()
{
	// Set up our light parameters
	SetupLight();

	// Draw our cube by hand...
	GLfloat pt1[3];
	GLfloat pt2[3];
	GLfloat pt3[3];
	double dNormal[3];
	glBegin(GL_POLYGON);
		pt1[0] = 7.0f;
		pt1[1] = 7.0f;
		pt1[2] = 7.0f;
		pt2[0] = -7.0f;
		pt2[1] = 7.0f;
		pt2[2] = 7.0f;
		pt3[0] = -7.0f;
		pt3[1] = -7.0f;
		pt3[2] = 7.0f;
		Normal(pt1,pt2,pt3,dNormal);
		glNormal3f((GLfloat)dNormal[0],(GLfloat)dNormal[1],(GLfloat)dNormal[2]);
		glVertex3f(pt1[0],pt1[1],pt1[2]);
		glVertex3f(pt2[0],pt2[1],pt2[2]);           
		glVertex3f(pt3[0],pt3[1],pt3[2]);            
		glVertex3f(7.0f,-7.0f,7.0f);            
	glEnd();

	glBegin(GL_POLYGON);
		pt1[0] = 7.0f;
		pt1[1] = 7.0f;
		pt1[2] = -7.0f;
		pt2[0] = 7.0f;
		pt2[1] = -7.0f;
		pt2[2] = -7.0f;
		pt3[0] = -7.0f;
		pt3[1] = -7.0f;
		pt3[2] = -7.0f;
		Normal(pt1,pt2,pt3,dNormal);
		glNormal3f((GLfloat)dNormal[0],(GLfloat)dNormal[1],(GLfloat)dNormal[2]);
		glVertex3f(pt1[0],pt1[1],pt1[2]);
		glVertex3f(pt2[0],pt2[1],pt2[2]);           
		glVertex3f(pt3[0],pt3[1],pt3[2]);            
		glVertex3f(-7.0f,7.0f,-7.0f);           
	glEnd();

	glBegin(GL_POLYGON);
		pt1[0] = -7.0f;
		pt1[1] = 7.0f;
		pt1[2] = 7.0f;
		pt2[0] = -7.0f;
		pt2[1] = 7.0f;
		pt2[2] = -7.0f;
		pt3[0] = -7.0f;
		pt3[1] = -7.0f;
		pt3[2] = -7.0f;
		Normal(pt1,pt2,pt3,dNormal);
		glNormal3f((GLfloat)dNormal[0],(GLfloat)dNormal[1],(GLfloat)dNormal[2]);
		glVertex3f(pt1[0],pt1[1],pt1[2]);
		glVertex3f(pt2[0],pt2[1],pt2[2]);           
		glVertex3f(pt3[0],pt3[1],pt3[2]);            
		glVertex3f(-7.0f,-7.0f,7.0f);           
	glEnd();

	glBegin(GL_POLYGON);
		pt1[0] = 7.0f;
		pt1[1] = 7.0f;
		pt1[2] = 7.0f;
		pt2[0] = 7.0f;
		pt2[1] = -7.0f;
		pt2[2] = 7.0f;
		pt3[0] = 7.0f;
		pt3[1] = -7.0f;
		pt3[2] = -7.0f;
		Normal(pt1,pt2,pt3,dNormal);
		glNormal3f((GLfloat)dNormal[0],(GLfloat)dNormal[1],(GLfloat)dNormal[2]);
		glVertex3f(pt1[0],pt1[1],pt1[2]);
		glVertex3f(pt2[0],pt2[1],pt2[2]);           
		glVertex3f(pt3[0],pt3[1],pt3[2]);            
		glVertex3f(7.0f,7.0f,-7.0f);            
	glEnd();

	glBegin(GL_POLYGON);
		pt1[0] = -7.0f;
		pt1[1] = 7.0f;
		pt1[2] = -7.0f;
		pt2[0] = -7.0f;
		pt2[1] = 7.0f;
		pt2[2] = 7.0f;
		pt3[0] = 7.0f;
		pt3[1] = 7.0f;
		pt3[2] = 7.0f;
		Normal(pt1,pt2,pt3,dNormal);
		glNormal3f((GLfloat)dNormal[0],(GLfloat)dNormal[1],(GLfloat)dNormal[2]);
		glVertex3f(pt1[0],pt1[1],pt1[2]);
		glVertex3f(pt2[0],pt2[1],pt2[2]);           
		glVertex3f(pt3[0],pt3[1],pt3[2]);            
		glVertex3f(7.0f,7.0f,-7.0f);            
	glEnd();

	glBegin(GL_POLYGON);
		pt1[0] = -7.0f;
		pt1[1] = -7.0f;
		pt1[2] = -7.0f;
		pt2[0] = 7.0f;
		pt2[1] = -7.0f;
		pt2[2] = -7.0f;
		pt3[0] = 7.0f;
		pt3[1] = -7.0f;
		pt3[2] = 7.0f;
		Normal(pt1,pt2,pt3,dNormal);
		glNormal3f((GLfloat)dNormal[0],(GLfloat)dNormal[1],(GLfloat)dNormal[2]);
		glVertex3f(pt1[0],pt1[1],pt1[2]);
		glVertex3f(pt2[0],pt2[1],pt2[2]);           
		glVertex3f(pt3[0],pt3[1],pt3[2]);            
		glVertex3f(-7.0f,-7.0f,7.0f);           
	glEnd();
}

void CGLObjectView::SetupLight()
{
	// Enable the basic light
	glLightfv(GL_LIGHT0,GL_AMBIENT,m_fLightAmbient0);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,m_fLightDiffuse0);
	glLightfv(GL_LIGHT0,GL_SPECULAR,m_fLightSpecular0);
	glLightfv(GL_LIGHT0,GL_POSITION,m_fLightPosition0);

	// Enable the spotlight
	glLightfv(GL_LIGHT1,GL_AMBIENT,m_fLightAmbient1);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,m_fLightDiffuse1);
	glLightfv(GL_LIGHT1,GL_SPECULAR,m_fLightSpecular1);
	glLightfv(GL_LIGHT1,GL_POSITION,m_fLightPosition1);
	glLightf(GL_LIGHT1,GL_SPOT_CUTOFF,60.0);
	glLightf(GL_LIGHT1,GL_SPOT_EXPONENT,25.0);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	if ( m_bDoSpotlight ) {
		// Enable the spot
		glEnable(GL_LIGHT1);
	} // if
	else {
		// Disable the spot
		glDisable(GL_LIGHT1);
	} // else
	glPolygonMode(GL_FRONT,GL_FILL);
    glEnable(GL_DEPTH_TEST);
	glEnable(GL_POLYGON_SMOOTH);

	// Set up object material parameters
	glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,m_fMaterialAmbient);
	glMaterialfv(GL_FRONT,GL_EMISSION,m_fMaterialEmitted);
	glMaterialfv(GL_FRONT,GL_SPECULAR,m_fMaterialSpecular);
	glMaterialf(GL_FRONT,GL_SHININESS,m_fMaterialShininess);
}

/////////////////////////////////////////////////////////////////////////////
// CGLObjectView message handlers

void CGLObjectView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Activate mouse axis rotation
	SetCapture();
	m_bMouseActive = TRUE;
	m_ptActive = point;
	
	CGLView::OnLButtonDown(nFlags, point);
}

void CGLObjectView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// Deactivate mouse axis rotation
	ReleaseCapture();
	m_bMouseActive = FALSE;
	
	CGLView::OnLButtonUp(nFlags, point);
}

void CGLObjectView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// Rotate OpenGL view based on stored positional data.  Note
	// the mouse motion is scaled (the "5.0" division)...adjust
	// this value to increase or decrease the mouse sensitivity.
	if ( m_bMouseActive ) {
		m_fXRotate -= (GLfloat)(m_ptActive.y - point.y) / 7.0f;
		m_fYRotate -= (GLfloat)(m_ptActive.x - point.x) / 7.0f;
		m_ptActive = point;
		Invalidate();
	} // if
	
	CGLView::OnMouseMove(nFlags, point);
}

void CGLObjectView::OnChangeObject(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMOBJECTCHANGE pObjectInfo = reinterpret_cast<LPNMOBJECTCHANGE>(pNMHDR);

	// Accept our new object
	int iNewObject = pObjectInfo->iValue;
	ASSERT((iNewObject >= 0) && (iNewObject <= 3));
	if ((iNewObject >= 0) && (iNewObject <= 3)) {
		// Valid object number...
		m_iObject = iNewObject;
	} // if

	// Redraw
	Invalidate();

	*pResult = 0;
}

void CGLObjectView::OnChangeRender(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMRENDERCHANGE pRenderInfo = reinterpret_cast<LPNMRENDERCHANGE>(pNMHDR);

	// Pull the new render style
	m_iRenderStyle = pRenderInfo->iValue;

	// Redraw
	Invalidate();

	*pResult = 0;
}

void CGLObjectView::OnChangeScale(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMSCALECHANGE pScaleInfo = reinterpret_cast<LPNMSCALECHANGE>(pNMHDR);

	// Pull our scale information and redraw
	switch ( pScaleInfo->iAxis ) {
		case SCALEX:
			m_fScaleX = pScaleInfo->fValue;
			break;

		case SCALEY:
			m_fScaleY = pScaleInfo->fValue;
			break;

		case SCALEZ:
			m_fScaleZ = pScaleInfo->fValue;
			break;

		default:
			break;

	} //switch

	// Redraw
	Invalidate();

	*pResult = 0;
}

void CGLObjectView::OnChangeProps(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMPROPSCHANGE pPropInfo = reinterpret_cast<LPNMPROPSCHANGE>(pNMHDR);

	// Pull our property information and redraw
	switch ( pPropInfo->iProperty ) {
		case LIGHT_AMBIENT:
			switch ( pPropInfo->iComponent ) {
				case RED:
					m_fLightAmbient0[0] = pPropInfo->fValue;
					break;

				case GREEN:
					m_fLightAmbient0[1] = pPropInfo->fValue;
					break;

				case BLUE:
					m_fLightAmbient0[2] = pPropInfo->fValue;
					break;

				default:
					break;
			} // switch
			break;

		case LIGHT_DIFFUSE:
			switch ( pPropInfo->iComponent ) {
				case RED:
					m_fLightDiffuse0[0] = pPropInfo->fValue;
					break;

				case GREEN:
					m_fLightDiffuse0[1] = pPropInfo->fValue;
					break;

				case BLUE:
					m_fLightDiffuse0[2] = pPropInfo->fValue;
					break;

				default:
					break;
			} // switch
			break;

		case LIGHT_SPECULAR:
			switch ( pPropInfo->iComponent ) {
				case RED:
					m_fLightSpecular0[0] = pPropInfo->fValue;
					break;

				case GREEN:
					m_fLightSpecular0[1] = pPropInfo->fValue;
					break;

				case BLUE:
					m_fLightSpecular0[2] = pPropInfo->fValue;
					break;

				default:
					break;
			} // switch
			break;

		case MATERIAL_AMBIENT:
			switch ( pPropInfo->iComponent ) {
				case RED:
					m_fMaterialAmbient[0] = pPropInfo->fValue;
					break;

				case GREEN:
					m_fMaterialAmbient[1] = pPropInfo->fValue;
					break;

				case BLUE:
					m_fMaterialAmbient[2] = pPropInfo->fValue;
					break;

				default:
					break;
			} // switch
			break;

		case MATERIAL_EMITTED:
			switch ( pPropInfo->iComponent ) {
				case RED:
					m_fMaterialEmitted[0] = pPropInfo->fValue;
					break;

				case GREEN:
					m_fMaterialEmitted[1] = pPropInfo->fValue;
					break;

				case BLUE:
					m_fMaterialEmitted[2] = pPropInfo->fValue;
					break;

				default:
					break;
			} // switch
			break;

		case MATERIAL_SPECULAR:
			switch ( pPropInfo->iComponent ) {
				case RED:
					m_fMaterialSpecular[0] = pPropInfo->fValue;
					break;

				case GREEN:
					m_fMaterialSpecular[1] = pPropInfo->fValue;
					break;

				case BLUE:
					m_fMaterialSpecular[2] = pPropInfo->fValue;
					break;

				default:
					break;
			} // switch
			break;

		case MATERIAL_SHININESS:
			m_fMaterialShininess = pPropInfo->fValue;
			break;

		default:
			break;

	} //switch

	// Redraw
	Invalidate();

	*pResult = 0;
}

void CGLObjectView::OnChangeSpot(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMSPOTCHANGE pSpotInfo = reinterpret_cast<LPNMSPOTCHANGE>(pNMHDR);

	// Pull the axes enabled status
	m_bDoSpotlight = pSpotInfo->bEnable;

	// Redraw
	Invalidate();

	*pResult = 0;
}

void CGLObjectView::OnChangeAxes(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMAXESCHANGE pAxesInfo = reinterpret_cast<LPNMAXESCHANGE>(pNMHDR);

	// Pull the axes enabled status
	m_bDrawAxes = pAxesInfo->bEnable;

	// Redraw
	Invalidate();

	*pResult = 0;
}
