// GLSceneView.cpp : implementation of the CGLSceneView class
//

#include "stdafx.h"
#include "GLScene.h"

#include "GLSceneDoc.h"
#include "GLSceneView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGLSceneView

IMPLEMENT_DYNCREATE(CGLSceneView, CGLView)

BEGIN_MESSAGE_MAP(CGLSceneView, CGLView)
	//{{AFX_MSG_MAP(CGLSceneView)
	ON_WM_TIMER()
	ON_COMMAND(ID_VIEW_AXES, OnViewAxes)
	ON_UPDATE_COMMAND_UI(ID_VIEW_AXES, OnUpdateViewAxes)
	ON_COMMAND(ID_VIEW_ALPHA, OnViewAlpha)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ALPHA, OnUpdateViewAlpha)
	ON_COMMAND(ID_VIEW_FOG, OnViewFog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FOG, OnUpdateViewFog)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CGLView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGLSceneView construction/destruction

CGLSceneView::CGLSceneView() :
	m_fYRotate(0.0f),
   m_fAlpha(0.7f),
   m_bDoAlpha(FALSE),
   m_bDoFog(FALSE),
   m_bDoAxes(TRUE)
{
	// Establish our initial light parameters, starting with
	// the ambient light conditions.
	m_fLightAmbient0[0] = 0.2f; // red component
	m_fLightAmbient0[1] = 0.2f; // green component
	m_fLightAmbient0[2] = 0.2f; // blue component
	m_fLightAmbient0[3] = 1.0f; // alpha

	// Next the diffuse light parameter
	m_fLightDiffuse0[0] = 0.7f; // red component
	m_fLightDiffuse0[1] = 0.7f; // green component
	m_fLightDiffuse0[2] = 0.7f; // blue component
	m_fLightDiffuse0[3] = 1.0f; // alpha

	// Then, the specular light parameter
	m_fLightSpecular0[0] = 0.0f; // red component
	m_fLightSpecular0[1] = 0.0f; // green component
	m_fLightSpecular0[2] = 0.0f; // blue component
	m_fLightSpecular0[3] = 1.0f; // alpha

	// And finally, its position
	m_fLightPosition0[0] = 0.0f; // X
	m_fLightPosition0[1] = 7.0f; // Y
	m_fLightPosition0[2] = 4.5f; // Z
	m_fLightPosition0[3] = 1.0f; // W (positional light)

	// Now set up our fog color (white)
	m_fFogColor[0] = 1.0f; // red component
	m_fFogColor[1] = 1.0f; // green component
	m_fFogColor[2] = 1.0f; // blue component
	m_fFogColor[3] = 1.0f; // alpha
}

CGLSceneView::~CGLSceneView()
{
}

BOOL CGLSceneView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CGLView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGLSceneView drawing

void CGLSceneView::OnDraw(CDC* pDC)
{
	CGLSceneDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Manage our palette
	if ( m_hPalette ) {
	    SelectPalette(pDC->m_hDC,m_hPalette,FALSE);
	    RealizePalette(pDC->m_hDC);
	} // if

	// Draw the OpenGL scene
	wglMakeCurrent(pDC->m_hDC,m_hRC);
	RenderView();
	SwapBuffers(pDC->m_hDC);
	wglMakeCurrent(pDC->m_hDC,NULL);
}

/////////////////////////////////////////////////////////////////////
// CGLObjectView helpers

void CGLSceneView::RenderView()
{
	// Set up our lights
	SetupLight();

	// Add any fog
	if ( m_bDoFog ) {
		// Activate fog...
		glEnable(GL_FOG);
		glFogf(GL_FOG_DENSITY,0.05f);
		glFogi(GL_FOG_MODE,GL_EXP2);
		glFogfv(GL_FOG_COLOR,m_fFogColor);
		glClearColor(0.8f,0.8f,0.8f,0.0f); // gray
	} // if
	else {
		// Make sure fog is disabled...
		glDisable(GL_FOG);
		glClearColor(0.0f,0.0f,0.0f,0.0f); // black
	} // else

	// Clear the viewport
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
		// Rotate the coordinate axes a bit for visual clarity
		glRotatef(30.0f,1.0f,0.0f,0.0f);

		// Spin the scene as a whole
		glRotatef(m_fYRotate,0.0f,1.0f,0.0f);

		// Draw spheres, dodecahedrons, and cubes
		for (int x = -5; x <= 5; x += 5) {
			for (int y=-5; y <= 5; y += 5) {
				// Spheres, color first
				GLfloat glfMaterialColorSpheres[] =
							{ 0.0f, 0.0f, 1.0f, 1.0f }; // blue
				glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,
							glfMaterialColorSpheres);

				// Save our model view matrix
				glPushMatrix();
					// Translate this sphere into place
					glTranslatef((GLfloat)x,(GLfloat)y,-5.0f);

					// Draw the sphere
					auxSolidSphere(2.0);

				// Restore our matrix
				glPopMatrix();

				// Dodecahedrons, color first
				GLfloat glfMaterialColorDodecs[] =
							{ 0.0f, 1.0f, 0.0f, 1.0f }; // green
				glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,
							glfMaterialColorDodecs);

				// Save our model view matrix
				glPushMatrix();
					// Translate this dodec into place
					glTranslatef((GLfloat)x,(GLfloat)y,0.0f);

					// Spin the dodec for effect
					glRotatef(-m_fYRotate, 1.0f, 1.0f, 1.0f);

					// Draw the dodec
					auxSolidDodecahedron(2.0);

				// Restore our matrix
				glPopMatrix();

				// Cubes, color first
				GLfloat glfMaterialColorCubes[] =
							{ 1.0f, 0.0f, 0.0f, 1.0f };
				glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,
							glfMaterialColorCubes);

				// Save our model view matrix
				glPushMatrix();
					// Translate this cube into place
					glTranslatef((GLfloat)x,(GLfloat)y,5.0f);

					// Spin the cube for effect
					glRotatef(m_fYRotate, 0.0f, 1.0f, 0.0f);

					// Scale it (originates as a 1x1x1 cube, so
					// we scale it to fit this scene size context).
					glScalef(1.50f,1.50f,1.50f);

					// Draw the cube
					glCallList(SOLIDCUBE);

				// Restore our matrix
				glPopMatrix();
			} // for
		} // for

		// Draw alpha-blended cube
		if ( m_bDoAlpha ) {
			// Enable blending for this object
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

			// Save our model view matrix
			glPushMatrix();
				// Scale it (originates as a 1x1x1 cube, so
				// we scale it to fit this scene size context).
				glScalef(7.5f,7.5f,7.5f);

				// Establish its color (note the alpha setting...)
				glColor4f(1.0f, 1.0f, 1.0f, m_fAlpha); // white

				// Draw the cube
				glCallList(ALPHACUBE);

			// Restore our matrix
			glPopMatrix();
		} // if

		// Disable alpha blending for axes call
		glDisable(GL_BLEND);

		// Draw coordinate exes
		if ( m_bDoAxes ) {
			// Scale axes to they appear outside the alpha cube
			glScalef(10.0f,10.0f,10.0f);

			// Draw the axes
			glCallList(AXES);
		} // if

	// Restore our matrix
	glPopMatrix();
	// Flush this buffer to screen
	glFlush();
}

void CGLSceneView::CreateDisplayLists()
{
	// Begin a display list for a solid cube
	glNewList(SOLIDCUBE,GL_COMPILE);
		// You are free to place nearly any OpenGL command
		// in a display list.  You could, for example,
		// set the color here.  However, I elected to set
		// the color when I render, thus allowing me to
		// have the flexibility.
		glBegin(GL_POLYGON);
			// I could have calculated the normal vector, but
			// we know what it should be, so I simply set it
			// in this case.
			glNormal3f(0.0f,0.0f,1.0f);
			glVertex3f(1.0f,1.0f,1.0f);
			glVertex3f(-1.0f,1.0f,1.0f);            
			glVertex3f(-1.0f,-1.0f,1.0f);           
			glVertex3f(1.0f,-1.0f,1.0f);            
		glEnd();

		glBegin(GL_POLYGON);
			glNormal3f(0.0f,0.0f,-1.0f);
			glVertex3f(1.0f,1.0f,-1.0f);
			glVertex3f(1.0f,-1.0f,-1.0f);           
			glVertex3f(-1.0f,-1.0f,-1.0f);          
			glVertex3f(-1.0f,1.0f,-1.0f);           
		glEnd();

		glBegin(GL_POLYGON);
			glNormal3f(-1.0f,0.0f,0.0f);
			glVertex3f(-1.0f,1.0f,1.0f);
			glVertex3f(-1.0f,1.0f,-1.0f);           
			glVertex3f(-1.0f,-1.0f,-1.0f);          
			glVertex3f(-1.0f,-1.0f,1.0f);           
		glEnd();

		glBegin(GL_POLYGON);
			glNormal3f(1.0f,0.0f,0.0f);
			glVertex3f(1.0f,1.0f,1.0f);
			glVertex3f(1.0f,-1.0f,1.0f);            
			glVertex3f(1.0f,-1.0f,-1.0f);           
			glVertex3f(1.0f,1.0f,-1.0f);            
		glEnd();

		glBegin(GL_POLYGON);
			glNormal3f(0.0f,1.0f,0.0f);
			glVertex3f(-1.0f,1.0f,-1.0f);
			glVertex3f(-1.0f,1.0f,1.0f);            
			glVertex3f(1.0f,1.0f,1.0f);             
			glVertex3f(1.0f,1.0f,-1.0f);            
		glEnd();

		glBegin(GL_POLYGON);
			glNormal3f(0.0f,-1.0f,0.0f);
			glVertex3f(-1.0f,-1.0f,-1.0f);
			glVertex3f(1.0f,-1.0f,-1.0f);           
			glVertex3f(1.0f,-1.0f,1.0f);            
			glVertex3f(-1.0f,-1.0f,1.0f);           
		glEnd();
	glEndList(); // SOLIDCUBE

	// Next, build a display list for the alpha-blended cube
	glNewList(ALPHACUBE,GL_COMPILE);
		// No light for this one...
		glDisable(GL_LIGHTING);
		glDisable(GL_LIGHT0);

		glBegin(GL_POLYGON);
			// No light, no normal vector!
			glVertex3f(1.0f,1.0f,1.0f);
			glVertex3f(-1.0f,1.0f,1.0f);            
			glVertex3f(-1.0f,-1.0f,1.0f);           
			glVertex3f(1.0f,-1.0f,1.0f);            
		glEnd();

		glBegin(GL_POLYGON);
			glVertex3f(1.0f,1.0f,-1.0f);
			glVertex3f(1.0f,-1.0f,-1.0f);           
			glVertex3f(-1.0f,-1.0f,-1.0f);          
			glVertex3f(-1.0f,1.0f,-1.0f);           
		glEnd();

		glBegin(GL_POLYGON);
			glVertex3f(-1.0f,1.0f,1.0f);
			glVertex3f(-1.0f,1.0f,-1.0f);           
			glVertex3f(-1.0f,-1.0f,-1.0f);          
			glVertex3f(-1.0f,-1.0f,1.0f);           
		glEnd();

		glBegin(GL_POLYGON);
			glVertex3f(1.0f,1.0f,1.0f);
			glVertex3f(1.0f,-1.0f,1.0f);            
			glVertex3f(1.0f,-1.0f,-1.0f);           
			glVertex3f(1.0f,1.0f,-1.0f);            
		glEnd();

		glBegin(GL_POLYGON);
			glVertex3f(-1.0f,1.0f,-1.0f);
			glVertex3f(-1.0f,1.0f,1.0f);            
			glVertex3f(1.0f,1.0f,1.0f);             
			glVertex3f(1.0f,1.0f,-1.0f);            
		glEnd();

		glBegin(GL_POLYGON);
			glVertex3f(-1.0f,-1.0f,-1.0f);
			glVertex3f(1.0f,-1.0f,-1.0f);           
			glVertex3f(1.0f,-1.0f,1.0f);            
			glVertex3f(-1.0f,-1.0f,1.0f);           
		glEnd();
	glEndList(); // ALPHACUBE

	// Finally, the coordinate axes display list
	glNewList(AXES,GL_COMPILE);
		// No light for axes
		glDisable(GL_LIGHTING);
		glDisable(GL_LIGHT0);

		glColor3f(0.0f, 0.0f, 1.0f); // blue
		glBegin(GL_LINES); // X axis
			glVertex3f(1.0f,0.0f,0.0f);
			glVertex3f(-1.0f,0.0f,0.0f);
		glEnd();

		glColor3f(1.0f,1.0f,0.0f); // yellow
		glBegin(GL_LINES); // Y axis
			glVertex3f(0.0f,1.0f,0.0f);
			glVertex3f(0.0f,-1.0f,0.0f);
		glEnd();

		glColor3f(1.0f, 0.0f, 0.0f); // red
		glBegin(GL_LINES); // Z axis
			glVertex3f(0.0f, 0.0f, 1.0f);
			glVertex3f(0.0f, 0.0f, -1.0f);
		glEnd();
	glEndList(); // AXES
}

void CGLSceneView::SetupLight()
{
	// Assign our light parameters
	glLightfv(GL_LIGHT0,GL_AMBIENT,m_fLightAmbient0);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,m_fLightDiffuse0);
	glLightfv(GL_LIGHT0,GL_SPECULAR,m_fLightSpecular0);
	glLightfv(GL_LIGHT0,GL_POSITION,m_fLightPosition0);

	// Enable lighting
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// Enable depth cueing and polygon culling
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
}

/////////////////////////////////////////////////////////////////////////////
// CGLSceneView diagnostics

#ifdef _DEBUG
void CGLSceneView::AssertValid() const
{
	CGLView::AssertValid();
}

void CGLSceneView::Dump(CDumpContext& dc) const
{
	CGLView::Dump(dc);
}

CGLSceneDoc* CGLSceneView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGLSceneDoc)));
	return (CGLSceneDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////
// CGLSceneView message handlers

void CGLSceneView::OnTimer(UINT nIDEvent) 
{
	// Bump our current rotation angle
	m_fYRotate += 5.0f;
	if ( m_fYRotate >= 359.0f ) {
		// We've about to exceed 360 degrees, so
		// reset to zero.
		m_fYRotate = 0.0f;
	} // if
        
	// Manage our alpha value
	static BOOL bDecrement = TRUE;
	if ( bDecrement ) {
		// Subtract alpha value
		m_fAlpha -= 0.05f;
		if ( m_fAlpha <= 0.29 ) {
			// It's light enough, so begin increasing
			// alpha value
			bDecrement = FALSE;
		} // if
	} // if
	else {
		if ( m_fAlpha >= 0.69f ) {
			// It's dark enough, so begin decreasing
			// alpha value
			bDecrement = TRUE;
		} // if

		// Subtract alpha value
		m_fAlpha += 0.05f;
	} // else

	// Redraw
	Invalidate();    
	
	CGLView::OnTimer(nIDEvent);
}

void CGLSceneView::OnActivateView(BOOL bActivate,
								  CView* pActivateView,
								  CView* pDeactiveView) 
{
	// Initialize our display lists
	static BOOL bInit = FALSE;
	if ( !bInit ) {
		// Obtain a device context
		CDC* pDC = GetDC();

		// Make our rendering context current
		wglMakeCurrent(pDC->m_hDC,m_hRC);

		// Create the lists
		CreateDisplayLists();

		// Make our rendering context not current
		wglMakeCurrent(pDC->m_hDC,NULL);

		// Release the DC
		ReleaseDC(pDC);
		bInit = TRUE;
	} // if

	// Set up a timer
	m_iTimerID = SetTimer(1,100,NULL);
	
	CGLView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

BOOL CGLSceneView::DestroyWindow() 
{
	// Kill the time we created
	KillTimer(m_iTimerID);

	// Delete our display lists
	glDeleteLists(SOLIDCUBE,3); // shaded cube, alpha cube, and axes

	return CGLView::DestroyWindow();
}

void CGLSceneView::OnViewAxes() 
{
	// Toggle axes status
	m_bDoAxes = !m_bDoAxes;
}

void CGLSceneView::OnUpdateViewAxes(CCmdUI* pCmdUI) 
{
	// Check/uncheck the option
	pCmdUI->SetCheck(m_bDoAxes);
}

void CGLSceneView::OnViewAlpha() 
{
	// Toggle alpha cube display status
	m_bDoAlpha = !m_bDoAlpha;
}

void CGLSceneView::OnUpdateViewAlpha(CCmdUI* pCmdUI) 
{
	// Check/uncheck the option
	pCmdUI->SetCheck(m_bDoAlpha);
}

void CGLSceneView::OnViewFog() 
{
	// Toggle fog status
	m_bDoFog = !m_bDoFog;
}

void CGLSceneView::OnUpdateViewFog(CCmdUI* pCmdUI) 
{
	// Check/uncheck the option
	pCmdUI->SetCheck(m_bDoFog);
}
