// GLTexCubeView.cpp : implementation of the CGLTexCubeView class
//

#include "stdafx.h"
#include "GLTexCube.h"

#include "GLTexCubeDoc.h"
#include "GLTexCubeView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView

IMPLEMENT_DYNCREATE(CGLTexCubeView, CGLView)

BEGIN_MESSAGE_MAP(CGLTexCubeView, CGLView)
	//{{AFX_MSG_MAP(CGLTexCubeView)
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CGLView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView construction/destruction

CGLTexCubeView::CGLTexCubeView() :
	m_yRotate(0.0f)
{
	// TODO: add construction code here

}

CGLTexCubeView::~CGLTexCubeView()
{
}

BOOL CGLTexCubeView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CGLView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView drawing

void CGLTexCubeView::OnDraw(CDC* pDC)
{
	CGLTexCubeDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Draw the OpenGL scene
	wglMakeCurrent(pDC->m_hDC,m_hRC);
	RenderView();
	SwapBuffers(pDC->m_hDC);
	wglMakeCurrent(pDC->m_hDC,NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView printing

BOOL CGLTexCubeView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGLTexCubeView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGLTexCubeView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView diagnostics

#ifdef _DEBUG
void CGLTexCubeView::AssertValid() const
{
	CGLView::AssertValid();
}

void CGLTexCubeView::Dump(CDumpContext& dc) const
{
	CGLView::Dump(dc);
}

CGLTexCubeDoc* CGLTexCubeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGLTexCubeDoc)));
	return (CGLTexCubeDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView helpers

void CGLTexCubeView::InitColorTables()
{
	LPRGBQUAD pColorTable = m_CDIBHolder.GetDIBRGBTable();
    
	// Copy the colors used by the DIB to class local
	// storage.
	for( int i = 0; i <= 255; ++i ) {
		m_rgbRed[i] = (GLfloat)pColorTable[i].rgbRed / 255;
		m_rgbGreen[i] = (GLfloat)pColorTable[i].rgbGreen / 255;
		m_rgbBlue[i] = (GLfloat)pColorTable[i].rgbBlue / 255;
	} // for

	// Map the colors for OpenGL to use
	CClientDC dcClient(this);
	wglMakeCurrent(dcClient.m_hDC, m_hRC);

	glPixelMapfv(GL_PIXEL_MAP_I_TO_R,256,m_rgbRed);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_G,256,m_rgbGreen);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_B,256,m_rgbBlue);
	glPixelTransferi(GL_MAP_COLOR,TRUE);

	wglMakeCurrent(dcClient.m_hDC, m_hRC);
}

void CGLTexCubeView::RenderView()
{
	GLvoid* pTextureBits = (GLvoid*) m_CDIBHolder.GetDIBBits();
	GLint iDIBWidth = m_CDIBHolder.Width();
	GLint iDIBHeight = m_CDIBHolder.Height();

	glTexImage2D(GL_TEXTURE_2D,0,3,iDIBWidth,iDIBHeight,
		0,GL_COLOR_INDEX,GL_UNSIGNED_BYTE,pTextureBits);
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
		// Scale it a bit
		glScalef(10.0f, 10.0f, 10.0f);

		// Rotate it
		glRotatef(m_yRotate,0.0f,1.0f,1.0f);

		// Render it
		glCallList(TEXCUBE);
	glPopMatrix();

	glFlush();
}

void CGLTexCubeView::CreateDisplayList()
{
	// Establish our texture mapped cube
	glNewList(TEXCUBE,GL_COMPILE);
		glBegin(GL_POLYGON); // "front"
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(-1.0f, 1.0f, 1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f, 1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(1.0f, -1.0f, 1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(1.0f, 1.0f, 1.0f);
		glEnd();
 
		glBegin(GL_POLYGON); // "right"
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(1.0f, 1.0f, 1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(1.0f, -1.0f, 1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(1.0f, 1.0f, -1.0f);
		glEnd();

		glBegin(GL_POLYGON); // "back"
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(-1.0f, 1.0f, -1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(1.0f, 1.0f, -1.0f);
		glEnd();

		glBegin(GL_POLYGON); // "left"
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(-1.0f, 1.0f, 1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f, 1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(-1.0f, 1.0f, -1.0f);
		glEnd();

		glBegin(GL_POLYGON); // "top"
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(1.0f, 1.0f, 1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(1.0f, 1.0f, -1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(-1.0f, 1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(-1.0f, 1.0f, 1.0f);
		glEnd();

		glBegin(GL_POLYGON); // "bottom"
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(1.0f, -1.0f, 1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(-1.0f, -1.0f, 1.0f);
		glEnd();
	glEndList();
}

/////////////////////////////////////////////////////////////////////////////
// CGLTexCubeView message handlers

int CGLTexCubeView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CGLView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// We'll need to add some things to get texture mapping
	// working in addition to the standard CGLView OpenGL
	// initialization.
	CClientDC dcClient(this);
	wglMakeCurrent(dcClient.m_hDC, m_hRC);

	// Set up texture mapping modes
	glTexParameteri(GL_TEXTURE_2D,
		GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,
		GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,
		GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,
		GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexEnvi(GL_TEXTURE_ENV,
		GL_TEXTURE_ENV_MODE,GL_DECAL);

	// Enable texture mapping
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);

	// Create our display list
	CreateDisplayList();

	// Release the rendering context
	wglMakeCurrent(dcClient.m_hDC, NULL);

	// Load our texture
	m_CDIBHolder.LoadDIBFile("Texture.bmp");
    InitColorTables();
	
	return 0;
}

void CGLTexCubeView::OnTimer(UINT nIDEvent) 
{
	// Bump rotation angle (12 degrees)
	m_yRotate += 12.0f;

	// Adjust for > 360 degrees
	if ( m_yRotate > 359.0f ) {
		m_yRotate = 0.0f;
	} // if

	// Redraw window
	Invalidate(FALSE);

	CGLView::OnTimer(nIDEvent);
}

BOOL CGLTexCubeView::DestroyWindow() 
{
	// Delete our display list
	glDeleteLists(TEXCUBE,1);
	
	return CGLView::DestroyWindow();
}

void CGLTexCubeView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	// If we're being activated, start the timer
	if ( bActivate && pActivateView == this ) {
		// Start the rotation
		SetTimer(m_iTimerID,100,NULL);
	} // if
	
	CGLView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

void CGLTexCubeView::OnSize(UINT nType, int cx, int cy) 
{
	CGLView::OnSize(nType, cx, cy);
	
	// Square up our aspect ratio...
	CClientDC dcClient(this);
	wglMakeCurrent(dcClient.m_hDC,m_hRC);
	glViewport(0,0,min(cx,cy),min(cx,cy));
	wglMakeCurrent(NULL,NULL);
}
