// GLView.cpp : implementation of the CGLView class
//

#include "stdafx.h"
#include "GLView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////
// CGLView

IMPLEMENT_DYNCREATE(CGLView, CView)

BEGIN_MESSAGE_MAP(CGLView, CView)
	//{{AFX_MSG_MAP(CGLView)
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	// Palette handlers
	ON_WM_PALETTECHANGED()
	ON_WM_QUERYNEWPALETTE()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////
// CGLView construction/destruction

CGLView::CGLView() :
	m_hPalette(NULL)
{
	// TODO: add construction code here
}

CGLView::~CGLView()
{
}

BOOL CGLView::PreCreateWindow(CREATESTRUCT& cs)
{
	// Set up style to suit OpenGL
    cs.style |= WS_CLIPCHILDREN | WS_CLIPSIBLINGS;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////
// CGLView printing

BOOL CGLView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGLView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGLView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////
// CGLView drawing

void CGLView::OnDraw(CDC* pDC)
{
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////
// CGLView diagnostics

#ifdef _DEBUG
void CGLView::AssertValid() const
{
	CView::AssertValid();
}

void CGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////
// CGLView helpers

void CGLView::SetupPalette()
{
	LOGICALPALETTE pal = { 0x300, 256 };
	BYTE baRed[] = {0, 36, 72, 109, 145, 182, 218, 255};
	BYTE baGreen[] = {0, 36, 72, 109, 145, 182, 218, 255};
	BYTE baBlue[] = {0, 85, 170, 255};

	// Fill our logical palette structure with color data
	for ( int iColor = 0; iColor < 256; ++iColor ) {
		pal.peaEntries[iColor].peRed =
								baRed[iColor & 0x07];
		pal.peaEntries[iColor].peGreen =
								baGreen[(iColor >> 0x03) & 0x07];
		pal.peaEntries[iColor].peBlue =
								baBlue[(iColor >> 0x06) & 0x03];
		pal.peaEntries[iColor].peFlags = 0;
	} // for

	// Build our palette
	m_hPalette = CreatePalette((LOGPALETTE*)&pal);
}

/////////////////////////////////////////////////////////////////////
// CGLView message handlers required by OpenGL

int CGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Initialize OpenGL parameters
	PIXELFORMATDESCRIPTOR pfd = {
        sizeof(PIXELFORMATDESCRIPTOR), // Structure size
        1,                             // Version number
        PFD_DRAW_TO_WINDOW |           // Property flags
            PFD_SUPPORT_OPENGL |
            PFD_DOUBLEBUFFER,          // (remove if no double buf)
        PFD_TYPE_RGBA,                 // PixelType
        24,                            // 24-bit color
        0, 0, 0, 0, 0, 0,              // Color bits and shift
        0, 0, 0, 0, 0, 0, 0,           // Alpha and accum buffer bits
        32,                            // 32-bit depth buffer
        0, 0,                          // No stencil or aux buffer
        PFD_MAIN_PLANE,                // Layer type
        0,                             // Reserved
        0, 0, 0                        // Unsupported
    };

	// Tell GDI to convert device context from Win32 to
	// OpenGL.
    CClientDC dcClient(this);
    int iPixelFormat = ChoosePixelFormat(dcClient.m_hDC,&pfd);
    if ( !iPixelFormat ) {
		// This system cannot run OpenGL
		TRACE("Error retrieving pixel format index...\n");
		ASSERT(FALSE);
		AfxMessageBox("Cannot initialize OpenGL...quitting.",
						MB_OK | MB_ICONERROR);
		return -1; // will fail new document creation...
	} // if

    if ( !SetPixelFormat(dcClient.m_hDC,iPixelFormat,&pfd) ) {
		// This system cannot run OpenGL
		TRACE("Error setting new pixel format...\n");
		ASSERT(FALSE);
		AfxMessageBox("Cannot initialize OpenGL...quitting.",
						MB_OK | MB_ICONERROR);
		return -1; // will fail new document creation...
	} // if

	// Update the PIXELFORMATDESCRIPTOR structure once
	// the device context has been modified.
    DescribePixelFormat(dcClient.m_hDC,iPixelFormat,
						sizeof(pfd),&pfd);

	// The PIXELFORMATDESCRIPTOR has been updated, so we now
	// determine whether to create and manage a custom
	// palette.
    if ( pfd.dwFlags & PFD_NEED_PALETTE ) {
		// We do, so build a new palette...
        SetupPalette();
	} // if
	
	// Create the OpenGL rendering context
    m_hRC = wglCreateContext(dcClient.m_hDC);
    if ( m_hRC == NULL ) {
		// This system cannot run OpenGL
		TRACE("Error creating OpenGL rendering context...\n");
		ASSERT(FALSE);
		AfxMessageBox("Cannot initialize OpenGL...quitting.",
						MB_OK | MB_ICONERROR);
		return -1; // will fail new document creation...
	} // if

	// We now make it the current rendering context so
	// we might set our clear color
    wglMakeCurrent(dcClient.m_hDC,m_hRC);
	glClearColor(0.0f,0.0f,0.0f,0.0f); // black
    wglMakeCurrent(dcClient.m_hDC,NULL);

	return 0;
}

BOOL CGLView::DestroyWindow() 
{
	// Shut down OpenGL by deleting the rendering context
    wglDeleteContext(m_hRC);

	// Delete our custom palette, if any was created
	if ( m_hPalette != NULL ) {
	    DeleteObject(m_hPalette);
	} // if
	
	return CView::DestroyWindow();
}

BOOL CGLView::OnEraseBkgnd(CDC* pDC) 
{
	// Don't erase view...let OpenGL handle
	// that for us...
	return TRUE;
}

void CGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// Change OpenGL's rendering context size and
	// viewing frustrum.
	CClientDC dcClient(this);
	wglMakeCurrent(dcClient.m_hDC,m_hRC);
	glViewport(0,0,cx,cy);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-10.0,10.0,-10.0,10.0,18.0,70.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-35.0f);
	wglMakeCurrent(NULL,NULL);
}


void CGLView::OnPaletteChanged(CWnd* pFocusWnd) 
{
	CView::OnPaletteChanged(pFocusWnd);
	
	// Manage our palette
	if ( m_hPalette ) {
		CDC* pDC = GetDC();
	    SelectPalette(pDC->m_hDC,m_hPalette,FALSE);
	    RealizePalette(pDC->m_hDC);
		ReleaseDC(pDC);

		// Redraw
		Invalidate();
	} // if
}

BOOL CGLView::OnQueryNewPalette() 
{
	// Manage our palette
	if ( m_hPalette ) {
		CDC* pDC = GetDC();
	    SelectPalette(pDC->m_hDC,m_hPalette,FALSE);
	    RealizePalette(pDC->m_hDC);
		ReleaseDC(pDC);

		// Redraw
		Invalidate();
	} // if

	return CView::OnQueryNewPalette();
}

/////////////////////////////////////////////////////////////////////
// Non-OpenGL CGLView message handlers
