// CTableView Game Logic Auxiliary Functions.
//


#include "stdafx.h"
#include "BlackJack.h"
#include "TableView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

HRESULT CTableView::ShowCards( BOOL bShowDealerHand /*=FALSE*/)
{
	// Copy table surface to secondary surface
    HRESULT hr = m_lpDDSSecondary->Blt(CRect(0,0,350,300),m_lpDDSTable.p,NULL,
                    DDBLT_WAIT,NULL);
    if ( FAILED(hr) ) {
		AfxMessageBox("Error bltting secondary surface",
					MB_OK | MB_ICONERROR);
		return hr;
	} // if

	// Copy dealer cards to secondary surface
	int iCard, iSuit;
	for ( int i = 0; i < m_iDealerNumCards; i++ ) {
		// Pull this card
		iCard = m_iDealerHand[i]%NUMCARDSINSUIT;
		iSuit = m_iDealerHand[i]/NUMCARDSINSUIT;

		int x = CARDXPOS + (i*CARDOFFSET);
		if ( !i && !bShowDealerHand ) {
			// First dealer card is face down...
			hr = m_lpDDSSecondary->Blt(CRect(x,DEALERYPOS,x+CARDWIDTH,DEALERYPOS+CARDHEIGHT),m_lpDDSCardBack.p,CRect(0,0,CARDWIDTH,CARDHEIGHT),
							DDBLT_WAIT,NULL);
			if ( FAILED(hr) ) {
				AfxMessageBox("Error bltting secondary surface",
							MB_OK | MB_ICONERROR);
				return hr;
			} // if
		} // if
		else {
			// Remaining dealer cards are face up...
			if ( iCard <= 7 ) {
				// Pull image from first surface
				hr = m_lpDDSSecondary->Blt(CRect(x,DEALERYPOS,x+CARDWIDTH,DEALERYPOS+CARDHEIGHT),m_lpDDSCards1.p,CRect(iCard*CARDWIDTH,iSuit*CARDHEIGHT,(iCard*CARDWIDTH)+(CARDWIDTH-1),(iSuit*CARDHEIGHT)+(CARDHEIGHT-1)),
								DDBLT_WAIT,NULL);
				if ( FAILED(hr) ) {
					AfxMessageBox("Error bltting secondary surface",
								MB_OK | MB_ICONERROR);
					return hr;
				} // if
			} // if
			else {
				// Pull image from second surface
				hr = m_lpDDSSecondary->Blt(CRect(x,DEALERYPOS,x+CARDWIDTH,DEALERYPOS+CARDHEIGHT),m_lpDDSCards2.p,CRect((iCard-8)*CARDWIDTH,iSuit*CARDHEIGHT,((iCard-8)*CARDWIDTH)+(CARDWIDTH-1),(iSuit*CARDHEIGHT)+(CARDHEIGHT-1)),
								DDBLT_WAIT,NULL);
				if ( FAILED(hr) ) {
					AfxMessageBox("Error bltting secondary surface",
								MB_OK | MB_ICONERROR);
					return hr;
				} // if
			} // else
		} // else
	} // for

	// Copy player cards to secondary surface
	for ( i = 0; i < m_iPlayerNumCards; i++ ) {
		// Pull this card
		iCard = m_iPlayerHand[i]%NUMCARDSINSUIT;
		iSuit = m_iPlayerHand[i]/NUMCARDSINSUIT;

		int x = CARDXPOS + (i*CARDOFFSET);
		if ( iCard <= 7 ) {
			// Pull image from first surface
			hr = m_lpDDSSecondary->Blt(CRect(x,PLAYERYPOS,x+CARDWIDTH,PLAYERYPOS+CARDHEIGHT),m_lpDDSCards1.p,CRect(iCard*CARDWIDTH,iSuit*CARDHEIGHT,(iCard*CARDWIDTH)+(CARDWIDTH-1),(iSuit*CARDHEIGHT)+(CARDHEIGHT-1)),
							DDBLT_WAIT,NULL);
			if ( FAILED(hr) ) {
				AfxMessageBox("Error bltting secondary surface",
							MB_OK | MB_ICONERROR);
				return hr;
			} // if
		} // if
		else {
			// Pull image from second surface
			hr = m_lpDDSSecondary->Blt(CRect(x,PLAYERYPOS,x+CARDWIDTH,PLAYERYPOS+CARDHEIGHT),m_lpDDSCards2.p,CRect((iCard-8)*CARDWIDTH,iSuit*CARDHEIGHT,((iCard-8)*CARDWIDTH)+(CARDWIDTH-1),(iSuit*CARDHEIGHT)+(CARDHEIGHT-1)),
							DDBLT_WAIT,NULL);
			if ( FAILED(hr) ) {
				AfxMessageBox("Error bltting secondary surface",
							MB_OK | MB_ICONERROR);
				return hr;
			} // if
		} // else
	} // for

	return hr;
}

void CTableView::DealPlayer()
{
	// Pull one from the deck and place into
	// the player's hand
	m_iPlayerHand[m_iPlayerNumCards++] = m_iCardDeck[m_iCurrentCard++];
	ShowCards(FALSE);
}

void CTableView::DealDealer()
{
	// Pull one from the deck and place into
	// the dealer's hand
	m_iDealerHand[m_iDealerNumCards++] = m_iCardDeck[m_iCurrentCard++];
	ShowCards(FALSE);
}

BOOL CTableView::Reset()
{
	TRACE("Game reset.\n");

	// Reset deck and hand statistics
	m_iCurrentCard = 0;
	m_iDealerNumCards = 0;
	m_bDealerHolds = FALSE;
	m_iPlayerNumCards = 0;
	m_bPlayerHolds = FALSE;

	// Initialize our card deck and player hands
	for ( int iCard = 0; iCard < NUMCARDS; iCard++ ) {
		// Fill the array with the count
		m_iCardDeck[iCard] = iCard;

		// Fill each hand with -1 (no card)
		m_iDealerHand[iCard] = -1;
		m_iPlayerHand[iCard] = -1;
	} // for

	// Reset the player decision
	m_mvLastDecision = NoDecision;

	// Deal new game!
	Shuffle();
	DealPlayer();
	DealDealer();
	DealPlayer();
	DealDealer();

	// Potential winner already?
	return IsWinner();
}

void CTableView::Shuffle()
{
	// Shuffle the deck ten times
	int iIndex, iTemp;
	for ( int iShuffle = 0; iShuffle < 10; iShuffle++ ) {
		for ( int iCard = 0; iCard < NUMCARDS; iCard ++ ) {
			// Swap this card with a random card
			iIndex = rand()%NUMCARDS;
			iTemp = m_iCardDeck[iCard];
			m_iCardDeck[iCard] = m_iCardDeck[iIndex];
			m_iCardDeck[iIndex] = iTemp;
		} // for
	} // for
}

BOOL CTableView::Move()
{
	// Check for player decision
	switch ( m_mvLastDecision ) {
		case NoDecision:
		case StandPat:
		default:
			// Do nothing...check for win momentarily
			m_bPlayerHolds = TRUE;
			break;

		case HitMe:
			// Deal player one card
			DealPlayer();
			m_bPlayerHolds = FALSE;
			break;

	} // switch

	// Make our decision
	if ( HandValue(TRUE) < 16 ) {
		// House stands 16 or better
		DealDealer();
		m_bDealerHolds = FALSE;
	} // if
	else {
		// Stand pat
		m_bDealerHolds = TRUE;
	} // else

	// Show cards
	Blt();

	// Check win conditions
	BOOL bIsWinner = IsWinner();
	if ( bIsWinner ) {
		// We had a winner...
		while ( bIsWinner ) {
			if ( AfxMessageBox("Try your luck again?",MB_YESNO | MB_ICONQUESTION) == IDYES ) {
				// Deal another game...note there
				// could be a winner here, too...
				bIsWinner = Reset();
				if ( !bIsWinner ) {
					// If no winner, display cards
					Blt();
				} // if
			} // if
			else {
				// Quit...
				break;
			} // else
		} // while
	} // if

	return bIsWinner;
}

BOOL CTableView::IsWinner()
{
	// Pull player hand value.
	int iPlayerHandValue = HandValue(FALSE);

	// Pull our hand value
	int iDealerHandValue = HandValue(TRUE);

	// Check for an outright win...
	BOOL bIsWinner = FALSE;
	if ( iDealerHandValue == 21 ) {
		// Dealer is cheating...  :)
		ProcessWinner(Dealer);
		bIsWinner = TRUE;
	} // if
	else if ((iDealerHandValue > 21) &&
		(iPlayerHandValue <= 21)) {
		// Dealer busted
		ProcessWinner(Player);
		bIsWinner = TRUE;
	} // if
	else if ((iPlayerHandValue > 21) &&
		(iDealerHandValue <= 21)) {
		// Player busted
		ProcessWinner(Dealer);
		bIsWinner = TRUE;
	} // else if
	else if ((iPlayerHandValue > 21) &&
			 (iDealerHandValue > 21)) {
		// Player busted
		ProcessWinner(Bust);
		bIsWinner = TRUE;
	} // else if
	else if ((m_iDealerNumCards >= 5) &&
			 (iDealerHandValue < 21)) {
		// Dealer five card charley.  We check dealer
		// first as house always wins in a tie.
		ProcessWinner(Dealer);
		bIsWinner = TRUE;
	} // else if
	else if ((m_iPlayerNumCards >= 5) &&
			 (iPlayerHandValue < 21)) {
		// Player five card charley
		ProcessWinner(Player);
		bIsWinner = TRUE;
	} // else if
	else {
		if ( m_bPlayerHolds && m_bDealerHolds ) {
			// Check win conditions
			if ((iPlayerHandValue > iDealerHandValue) &&
				(iPlayerHandValue <= 21)) {
				// Player hand is greater than ours and
				// they are less than or equal to 21 in
				// value, so player wins
				ProcessWinner(Player);
				bIsWinner = TRUE;
			} // if
			else if ((iPlayerHandValue <= iDealerHandValue) &&
				(iDealerHandValue <= 21)) {
				// Our hand is greater than or equal to the
				// player hand, and we're less than or equal
				// to 21, so we win.  House always wins in a
				// tie.  :)
				ProcessWinner(Dealer);
				bIsWinner = TRUE;
			} // else if
			else {
				// No other win condition was met, so it's a bust
				ProcessWinner(Bust);
				bIsWinner = TRUE;
			} // else
		} // if
	} // else

	return bIsWinner;
}

int CTableView::HandValue( BOOL bIsDealer )
{
	// Determine if we're the dealer or not...
	int* piHand = NULL;
	int iNumCards = 0;
	if ( bIsDealer ) {
		// Calculate dealer hand value
		piHand = m_iDealerHand;
		iNumCards = m_iDealerNumCards;
		TRACE("Dealer hand value: ");
	} // if
	else {
		// Calculate player hand value
		piHand = m_iPlayerHand;
		iNumCards = m_iPlayerNumCards;
		TRACE("Player hand value: ");
	} // else

	// Calculate actual value, aces high and low
	int iTotalAceLow = 0, iTotalAceHigh = 0, iThisCard;
	for ( int iCard = 0; iCard < iNumCards; iCard++ ) {
		// Determine the card first, then decide what
		// the card is worth:
		// Ace : 11
		// Deuce - 10 : face value
		// Face : 10
		iThisCard = (piHand[iCard]%13)+1;
		switch ( iThisCard ) {
			case 1: // Ace
				iTotalAceHigh += 11;
				iTotalAceLow += 1;
				break;

			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
			case 10:
				iTotalAceHigh += iThisCard;
				iTotalAceLow += iThisCard;
				break;

			case 11: // Jack
			case 12: // Queen
			case 13: // King
				iTotalAceHigh += 10;
				iTotalAceLow += 10;
				break;

		} // switch
	} // for

	// Now we decide which value to return.  If the
	// ace high value is greater than 21, we return the
	// low value, othewise we return the high value.
	if ( iTotalAceHigh <= 21 ) {
		// Ace high value is less than 21, so return that
		// value.
		TRACE1("%d (ace high).\n",iTotalAceHigh);
		return iTotalAceHigh;
	} // if

	TRACE1("%d (ace low).\n",iTotalAceLow);
	return iTotalAceLow;
}

void CTableView::ProcessWinner( Winner wnWinner /* =Bust */ )
{
	// Show the hands outright
	ShowCards(TRUE);
	Blt();

	// Display a mesage box...
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	switch ( wnWinner ) {
		case Bust:
			TRACE("It\'s a bust.\n");
			AfxMessageBox("It\'s a bust!",MB_OK | MB_ICONEXCLAMATION);
			break;

		case Dealer:
			TRACE("Dealer wins.\n");
			AfxMessageBox("Dealer wins!",MB_OK | MB_ICONEXCLAMATION);
			pFrame->GetScoreBoard().BumpDealer();
			break;

		case Player:
			TRACE("Player wins.\n");
			AfxMessageBox("Player wins!",MB_OK | MB_ICONEXCLAMATION);
			pFrame->GetScoreBoard().BumpPlayer();
			break;

		default:
			// There must be some logical winner or a bust
			// condition.
			ASSERT(FALSE);
			break;

	} // switch
}