// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "ScriptApp.h"

#include "MainFrm.h"
#include "ScriptView.h"
#include "NewMacroDlg.h"
#include "FileMacroDlg.h"
#include "EditMacroDlg.h"
#include "RunMacroDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_MACRO_NEW, OnMacroNew)
	ON_COMMAND(ID_MACRO_RUN, OnMacroRun)
	ON_COMMAND(ID_MACRO_EDIT, OnMacroEdit)
	ON_COMMAND(ID_MACRO_RESET, OnMacroReset)
	//}}AFX_MSG_MAP
	ON_CBN_SELCHANGE(AFX_IDW_TOOLBAR + 1,OnSelchangeMacro)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	// Create the macro combo box
	if (!m_wndMacro.Create(CBS_DROPDOWNLIST | WS_CHILD | CBS_AUTOHSCROLL | WS_VSCROLL | CBS_SORT,CRect(0,0,200,120),this,AFX_IDW_TOOLBAR + 1))
	{
		TRACE0("Failed to create combobox\n");
		return -1;      // fail to create
	}

	if (!m_wndReBar.Create(this) ||
		!m_wndReBar.AddBar(&m_wndToolBar) ||
		!m_wndReBar.AddBar(&m_wndMacro,_T("Macros:"),NULL,RBBS_FIXEDBMP | RBBS_BREAK))
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY);

	// Set an initial caption
	SetWindowText(_T("MFC Scripted Application"));

	// Finally, create the scripting object...
	if ( FAILED(m_pScriptObject.CreateInstance(CLSID_ScriptObject)) ) {
		// Unable to create the scripting object...
		AfxMessageBox("Unable to create the scripting object",MB_OK|MB_ICONERROR);
		return -1;
	} // if

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// No auto-title...
	cs.style &= ~FWS_ADDTOTITLE;

	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame helpers

void CMainFrame::FillMacroBox() 
{
	// Clear any existing data
	m_wndMacro.ResetContent();

	// Find all files in the current directory with the
	// macro file extension, ".mac".
	TCHAR szCurrentDir[_MAX_PATH+1];
	GetCurrentDirectory(_MAX_PATH,szCurrentDir);
	CString strPath(szCurrentDir);
	if ( strPath[strPath.GetLength()-1] != '\\' ) {
		// The directory doesn't end with a backslash,
		// so we add one now.
		strPath += '\\';
	} // if
	CString strFile = strPath + "*.mac";
	WIN32_FIND_DATA wfd;
	HANDLE hSearchHandle = FindFirstFile(strFile,&wfd);
	if ( hSearchHandle != INVALID_HANDLE_VALUE ) {
		// Pull first filename
		CString strFileTitle(wfd.cFileName);
		strFileTitle = strFileTitle.Left(strFileTitle.Find("."));
		int iItem = m_wndMacro.AddString(strFileTitle);

		while ( FindNextFile(hSearchHandle,&wfd) ) {
			// While we have more files...
			strFileTitle = wfd.cFileName;
			strFileTitle = strFileTitle.Left(strFileTitle.Find("."));
			m_wndMacro.AddString(strFileTitle);
		} // while

		// Close our search handle
		FindClose(hSearchHandle);
	} // if

	// Make sure no macro files are selected...
	m_wndMacro.SetCurSel(-1);
}

HRESULT CMainFrame::AddDocObject(BSTR bstrName, IDispatch* pIDispatch)
{
	HRESULT hr = S_OK;
	try {
		hr = m_pAppObject->SetDocument(pIDispatch);
		if ( SUCCEEDED(hr) ) {
			hr = m_pScriptObject->AddObject(bstrName,pIDispatch);
		} // if
	} // try
	catch (...) {
		AfxMessageBox("Unable to add the document object",MB_OK|MB_ICONERROR);
		hr = E_FAIL;
	} // catch

	return hr;
}

HRESULT CMainFrame::AddTextSelObject(BSTR bstrName, IDispatch* pIDispatch)
{
	HRESULT hr = S_OK;
	try {
		hr = m_pScriptObject->AddObject(bstrName,pIDispatch);
	} // try
	catch (...) {
		AfxMessageBox("Unable to add the text selection object",MB_OK|MB_ICONERROR);
		hr = E_FAIL;
	} // catch

	return hr;
}

void CMainFrame::RunMacro(LPCTSTR strText)
{
	try {
		// The script is passed in, which we convert to a 
		// BSTR, then feed to the parser.
		bstr_t bstrScript(strText);
		if ( FAILED(m_pScriptObject->ParseScript(bstrScript)) ) {
			// Throw some trash to hit the catch block
			throw "trash";
		} // if
	} // try
	catch (...) {
		// Some error...could be invalid script text, or
		// could be something else.  In any case, it didn't
		// run...
		AfxMessageBox("Error running script",MB_OK|MB_ICONERROR);
	} // catch
}

void CMainFrame::LoadMacro(const CString& strMacroName, CString& strMacroText)
{
	// Rebuild the macro filename
	TCHAR szCurrentDir[_MAX_PATH+1];
	GetCurrentDirectory(_MAX_PATH,szCurrentDir);
	CString strPath(szCurrentDir);
	if ( strPath[strPath.GetLength()-1] != '\\' ) {
		// The directory doesn't end with a backslash,
		// so we add one now.
		strPath += '\\';
	} // if
	ASSERT(strMacroName.GetLength());
	CString strFile;
	strFile = strPath + strMacroName + ".mac";

	// Load the file
	try {
		CFile fileMacro(strFile,CFile::modeRead|CFile::shareDenyWrite);

		// Determine the file's length
		DWORD dwSize = fileMacro.GetLength();

		// Read the file
		fileMacro.Read(strMacroText.GetBuffer(dwSize),dwSize);
		strMacroText.ReleaseBuffer();

		// Close it
		fileMacro.Close();
	} // try
	catch (CFileException* e) {
		// File I/O error
		CString strMsg;
		::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM |
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						e->m_cause,
						MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
						strMsg.GetBuffer(_MAX_PATH),
						0,
						NULL );
		strMsg.ReleaseBuffer();
		strMsg = _T("File error reading macro file, '") + strMsg + _T("'");

		// Display the string.
		AfxMessageBox(strMsg,MB_OK|MB_ICONERROR);
	} // catch
	catch (...) {
		// Some other error...
		CString strMsg;
		::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM |
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
						strMsg.GetBuffer(_MAX_PATH),
						0,
						NULL );
		strMsg.ReleaseBuffer();
		strMsg = _T("Error reading macro file, '") + strMsg + _T("'");

		// Display the string.
		AfxMessageBox(strMsg,MB_OK|MB_ICONERROR);
	} // catch
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::ActivateFrame(int nCmdShow) 
{
	CFrameWnd::ActivateFrame(nCmdShow);

	try {
		// Create the application object
		HRESULT hr = m_pAppObject.CreateInstance(CLSID_AppObject);
		if ( SUCCEEDED(hr) ) {
			// Initialize the application object
			m_pScriptObject->AddObject(bstr_t("application"),m_pAppObject);
			m_pAppObject->InitHwnd(reinterpret_cast<long>(m_hWnd));

			CScriptView* pView = (CScriptView*)GetActiveView();
			ASSERT(pView);
			pView->InitializeObjects();
		} // if
		else {
			// Some error...
			AfxMessageBox("Unable to create the application object",MB_OK|MB_ICONERROR);
		} // else
	} // try
	catch (...) {
		// Some error...
		AfxMessageBox("Unable to create the application object",MB_OK|MB_ICONERROR);
	} // catch

	// Fill the combobox with macro files
	FillMacroBox();
}

void CMainFrame::OnMacroNew() 
{
	// Bring up the new macro dialog
	CNewMacroDlg dlgNewMacro;

	if ( dlgNewMacro.DoModal() == IDOK ) {
		// Refill the combo box
		FillMacroBox();

		// Ask if they want to run the macro now
		if ( AfxMessageBox("Run macro now?",MB_YESNO|MB_ICONQUESTION) == IDYES ) {
			// Run the macro
			RunMacro(dlgNewMacro.m_strText);
		} // if
	} // if
}

void CMainFrame::OnMacroEdit() 
{
	// Determine which macro to edit
	CFileMacroDlg dlgFileMacro;

	if ( dlgFileMacro.DoModal() == IDCANCEL ) {
		// User canceled
		return;
	} // if

	// Edit that macro
	CEditMacroDlg dlgEditMacro;
	dlgEditMacro.m_strMacroName = dlgFileMacro.m_strMacroName;
	
	if ( dlgEditMacro.DoModal() == IDOK ) {
		// Ask if they want to run the macro now
		if ( AfxMessageBox("Run macro now?",MB_YESNO|MB_ICONQUESTION) == IDYES ) {
			// Run the macro
			RunMacro(dlgEditMacro.m_strText);
		} // if
	} // if
}

void CMainFrame::OnMacroRun() 
{
	// Determine which macro to run
	CRunMacroDlg dlgRunMacro;

	if ( dlgRunMacro.DoModal() == IDOK ) {
		// Run their macro
		RunMacro(dlgRunMacro.m_strText);
	} // if
}

void CMainFrame::OnMacroReset() 
{
	try {
		// Tell the script object to reset
		m_pScriptObject->Reset();
	} // try
	catch (...) {
		// Some error...
		AfxMessageBox("Unable to reset the script engine object",MB_OK|MB_ICONERROR);
	} // catch
}

void CMainFrame::OnSelchangeMacro()
{
	// A macro title was selected, so execute it
	int iIndex = m_wndMacro.GetCurSel();
	if ( iIndex != LB_ERR ) {
		// If we have a selction, load it
		CString strMacroName;
		m_wndMacro.GetLBText(iIndex,strMacroName);
		CString strMacroText;
		LoadMacro(strMacroName,strMacroText);
		RunMacro(strMacroText);
	} // if
}
