// ObjectPage.cpp : implementation file
//

#include "stdafx.h"
#include "GLObject.h"
#include "ObjectPage.h"

#include "MainFrm.h"
#include "GLObjectDoc.h"
#include "GLObjectView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CObjectPage property page

IMPLEMENT_DYNCREATE(CObjectPage, CPropertyPage)

CObjectPage::CObjectPage() : CPropertyPage(CObjectPage::IDD)
{
	//{{AFX_DATA_INIT(CObjectPage)
	m_iRenderStyle = 0;
	m_bDrawAxes = FALSE;
	//}}AFX_DATA_INIT
}

CObjectPage::~CObjectPage()
{
}

void CObjectPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CObjectPage)
	DDX_Control(pDX, IDC_SCALEZ, m_CScaleZ);
	DDX_Control(pDX, IDC_SCALEY, m_CScaleY);
	DDX_Control(pDX, IDC_SCALEX, m_CScaleX);
	DDX_Control(pDX, IDC_OBJECT, m_CObjectBox);
	DDX_Radio(pDX, IDC_WIREFRAME, m_iRenderStyle);
	DDX_Check(pDX, IDC_DOAXES, m_bDrawAxes);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CObjectPage, CPropertyPage)
	//{{AFX_MSG_MAP(CObjectPage)
	ON_BN_CLICKED(IDC_WIREFRAME, OnRenderChange)
	ON_CBN_SELCHANGE(IDC_OBJECT, OnSelchangeObject)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_SOLID, OnRenderChange)
	ON_BN_CLICKED(IDC_SHADED, OnRenderChange)
	ON_BN_CLICKED(IDC_DOAXES, OnDoAxes)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CObjectPage message handlers

BOOL CObjectPage::OnSetActive() 
{
	// Establish slider ranges
	m_CScaleX.SetRange(0,10);
	m_CScaleX.SetPos(5);
	m_CScaleY.SetRange(0,10);
	m_CScaleY.SetPos(5);
	m_CScaleZ.SetRange(0,10);
	m_CScaleZ.SetPos(5);
	UpdateData(FALSE);

	// Establish object (first time)
	static BOOL bInit = FALSE;
	if ( !bInit ) {
		// Set the initial object to be the cube
		m_CObjectBox.SetCurSel(0);
		bInit = TRUE;
	} // if

	return CPropertyPage::OnSetActive();
}

void CObjectPage::OnRenderChange() 
{
	// Pull control value
	UpdateData(TRUE);

	// Obtain our view
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	CGLObjectView* pView = (CGLObjectView*)pFrame->GetActiveView();
	ASSERT(pView != NULL);

	// Notify the view
	NMRENDERCHANGE msg;
	msg.hdr.hwndFrom = m_hWnd;
	msg.hdr.idFrom = IDD_OBJECT;
	msg.hdr.code = OBJECT_NOTIFY_RENDERCHANGE;

	// Notify render change
	msg.iValue = m_iRenderStyle;
	pView->SendMessage(WM_NOTIFY,(WPARAM)IDD_OBJECT,(LPARAM)(&msg));
}

void CObjectPage::OnSelchangeObject() 
{
	// Obtain our view
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	CGLObjectView* pView = (CGLObjectView*)pFrame->GetActiveView();
	ASSERT(pView != NULL);

	// Pull combo box value and notify the view
	NMOBJECTCHANGE msg;
	msg.hdr.hwndFrom = m_hWnd;
	msg.hdr.idFrom = IDD_OBJECT;
	msg.hdr.code = OBJECT_NOTIFY_OBJECTCHANGE;

	// Notify object change
	msg.iValue = m_CObjectBox.GetCurSel();
	pView->SendMessage(WM_NOTIFY,(WPARAM)IDD_OBJECT,(LPARAM)(&msg));
}

void CObjectPage::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// Obtain our view
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	CGLObjectView* pView = (CGLObjectView*)pFrame->GetActiveView();
	ASSERT(pView != NULL);

	// Pull slider values and notify the view
	NMSCALECHANGE msg;
	msg.hdr.hwndFrom = m_hWnd;
	msg.hdr.idFrom = IDD_OBJECT;
	msg.hdr.code = OBJECT_NOTIFY_SCALECHANGE;
	if ( pScrollBar == (CScrollBar*)&m_CScaleX ) {
		// Notify X axis scale change
		msg.iAxis = SCALEX;
		// The "raw" scale value will come in the range of
		// 0 - 10, which we need to convert to a float
		// value from 0.0 to 1.0.
		msg.fValue = (GLfloat)m_CScaleX.GetPos() / 10.0f;
		pView->SendMessage(WM_NOTIFY,(WPARAM)IDD_OBJECT,(LPARAM)(&msg));
	} // if
	else if ( pScrollBar == (CScrollBar*)&m_CScaleY ) {
		// Notify Y axis scale change
		msg.iAxis = SCALEY;
		msg.fValue = (GLfloat)m_CScaleY.GetPos() / 10.0f;
		pView->SendMessage(WM_NOTIFY,(WPARAM)IDD_OBJECT,(LPARAM)(&msg));
	} // else if
	else if ( pScrollBar == (CScrollBar*)&m_CScaleZ ) {
		// Notify Z axis scale change
		msg.iAxis = SCALEZ;
		msg.fValue = (GLfloat)m_CScaleZ.GetPos() / 10.0f;
		pView->SendMessage(WM_NOTIFY,(WPARAM)IDD_OBJECT,(LPARAM)(&msg));
	} // else if

	CPropertyPage::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CObjectPage::OnDoAxes() 
{
	// Pull control value
	UpdateData(TRUE);

	// Obtain our view
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	CGLObjectView* pView = (CGLObjectView*)pFrame->GetActiveView();
	ASSERT(pView != NULL);

	// Pull combo box value and notify the view
	NMAXESCHANGE msg;
	msg.hdr.hwndFrom = m_hWnd;
	msg.hdr.idFrom = IDD_OBJECT;
	msg.hdr.code = OBJECT_NOTIFY_AXESCHANGE;

	// Notify object change
	msg.bEnable = m_bDrawAxes;
	pView->SendMessage(WM_NOTIFY,(WPARAM)IDD_OBJECT,(LPARAM)(&msg));
}
