// ScriptImpl.h : Implementation of IActiveScript* interfaces

#ifndef __SCRIPTIMPL_H_
#define __SCRIPTIMPL_H_

// IActiveScript* definitions
#include <activscp.h>

// swprintf definition
#include <stdio.h>

class ATL_NO_VTABLE IActiveScriptSiteImpl : public IActiveScriptSite
{
	STDMETHOD(QueryInterface)(REFIID riid, LPVOID* ppvObject) = 0;
	_ATL_DEBUG_ADDREF_RELEASE_IMPL(IActiveScriptSiteImpl)

	// This method *must* be overridden in your implementation
	// (hence the E_FAIL).
	STDMETHOD(GetItemInfo)(LPCOLESTR pstrName, DWORD dwReturnMask,
		IUnknown** ppiunkItem, ITypeInfo** ppti)
	{
		// You must override this and provide your own implementation.
		return E_FAIL;
	}
 
	STDMETHOD(OnScriptError)(IActiveScriptError *pscripterror)
	{
		// Set up the exception record
		EXCEPINFO ei;
		::ZeroMemory(&ei,sizeof(ei));

		// Determine from where the error came
		DWORD dwCookie;
		LONG nChar;
		ULONG nLine;
		pscripterror->GetSourcePosition(&dwCookie, &nLine, &nChar);

		// Retrieve the source line
		BSTR bstrSourceLine = NULL;
		pscripterror->GetSourceLineText(&bstrSourceLine);

		// Fill the exception record
		pscripterror->GetExceptionInfo(&ei);
    
		// Create an output string
		OLECHAR wszMsg[2048];
		swprintf(wszMsg, OLESTR("%s\n[Line: %d] %s\n%s"),
				  ei.bstrSource, nLine, ei.bstrDescription,
				  bstrSourceLine ? bstrSourceLine : OLESTR(""));

		// Display the error string
		::MessageBoxW(GetDesktopWindow(),
						wszMsg,
						L"Script Error",
						MB_SETFOREGROUND);

		// Free our BSTRs
		::SysFreeString(bstrSourceLine);
		::SysFreeString(ei.bstrSource);
		::SysFreeString(ei.bstrDescription);
		::SysFreeString(ei.bstrHelpFile);

		return S_OK;
	}

	STDMETHOD(GetLCID)(LCID *plcid)
	{
		// Change the LCID value to meet your internationalization
		// requirements...
		*plcid = MAKELCID(LANG_USER_DEFAULT,SORT_DEFAULT);
		return S_OK;
	}

	STDMETHOD(GetDocVersionString)(BSTR *pbstrVersion) 
	{
		*pbstrVersion = SysAllocString(L"");
		return S_OK;
	}

	STDMETHOD(OnScriptTerminate)(const VARIANT *pvr, const EXCEPINFO *pei)
	{
		return S_OK;
	}

	STDMETHOD(OnStateChange)(SCRIPTSTATE ssScriptState)
	{
		return S_OK;
	}

	STDMETHOD(OnEnterScript)(void)
	{
		return S_OK;
	}

	STDMETHODIMP OnLeaveScript(void) 
	{
		return S_OK;
	}
};

class IActiveScriptSiteWindowImpl : public IActiveScriptSiteWindow
{
	STDMETHOD(QueryInterface)(REFIID riid, void ** ppvObject) = 0;
	_ATL_DEBUG_ADDREF_RELEASE_IMPL(IActiveScriptSiteImpl)

	STDMETHODIMP GetWindow(HWND *phwnd)
	{
		*phwnd = GetDesktopWindow();
		return S_OK;
	}

	STDMETHODIMP EnableModeless(BOOL)
	{
		return S_OK;
	}
};

#endif // __SCRIPTIMPL_H_
