// ScriptObjectCtl.cpp : Implementation of CScriptObject
#include "stdafx.h"
#include "ScriptObject.h"
#include "ScriptObjectCtl.h"

#include <initguid.h>
#include "msscript.h"

/////////////////////////////////////////////////////////////////////////////
// CScriptObject

bool operator<(const SCRIPTELEMENT& lhs, const SCRIPTELEMENT& rhs)
{
	return lhs.bstrName < rhs.bstrName;
}

bool operator==(const SCRIPTELEMENT& lhs, const SCRIPTELEMENT& rhs)
{
	return lhs.bstrName == rhs.bstrName;
}

HRESULT CScriptObject::FinalConstruct()
{
	// CoCreate the scripting engine
	HRESULT hr = CoCreateInstance(CLSID_VBScript,
							NULL,
							CLSCTX_ALL,
							IID_IActiveScriptParse,
							reinterpret_cast<LPVOID*>(&m_pScriptParser));
	if ( SUCCEEDED(hr) ) {
		// Query for the script interface
		CComQIPtr<IActiveScript,&IID_IActiveScript> pScript(m_pScriptParser);
		if ( pScript.p != NULL ) {
			// Assign our attribute
			m_pScript = pScript;

			// Initialize the script engine
			hr = m_pScriptParser->InitNew();
			if ( SUCCEEDED(hr) ) hr = m_pScript->SetScriptSite(this);
			if ( SUCCEEDED(hr) ) hr = m_pScript->SetScriptState(SCRIPTSTATE_STARTED);
		} // if
		else {
			// Failed...
			hr = E_FAIL;
		} // else
	} // if
		
	return hr;
}

void CScriptObject::FinalRelease()
{
	// TODO: more cleanup, if required.
}

STDMETHODIMP CScriptObject::AddObject(BSTR bstrObjName, IUnknown* pUnk)
{
	HRESULT hr = S_OK;
	try {
		m_vScriptElements.push_back(SCRIPTELEMENT(bstrObjName,pUnk));
		hr = m_pScript->AddNamedItem(bstrObjName,
				SCRIPTITEM_ISVISIBLE | SCRIPTITEM_ISSOURCE);
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CScriptObject::ParseScript(BSTR bstrScript)
{
	HRESULT hr = S_OK;
	try {
		// Set up the error information structure we'll
		// return if there is a parsing error.
		EXCEPINFO e;
		ZeroMemory(&e,sizeof(e));

		// We're connected...
		hr = m_pScript->SetScriptState(SCRIPTSTATE_CONNECTED);
		if ( SUCCEEDED(hr) ) {
			// ...so parse the script and execute.
			hr = m_pScriptParser->ParseScriptText(bstrScript,0,0,0,0,0,
						SCRIPTTEXT_ISPERSISTENT | SCRIPTTEXT_ISVISIBLE,0,&e);
		} // if
	} // try
	catch (...) {
		// Some error
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CScriptObject::GetItemInfo(LPCOLESTR pstrName, DWORD dwReturnMask, IUnknown **ppiunkItem, ITypeInfo **ppti)
{
	HRESULT hr = S_OK;
	try {
		if ( dwReturnMask & SCRIPTINFO_IUNKNOWN ) {
			// Requesting the IUnknown pointer
			if ( ppiunkItem != NULL ) {
			// Clear the return pointer
				*ppiunkItem = NULL;
			} // if
			else {
				// Bad pointer
				return E_POINTER;
			} // else
		} // if
		if ( dwReturnMask & SCRIPTINFO_ITYPEINFO ) {
			// Requesting the IUnknown pointer
			if ( ppti != NULL ) {
			// Clear the return pointer
				*ppti = NULL;
			} // if
			else {
				// Bad pointer
				return E_POINTER;
			} // else
		} // if

		ScriptElements::iterator itElement;
		for (itElement = m_vScriptElements.begin(); itElement != m_vScriptElements.end(); itElement++) {
			if ( itElement->bstrName == bstr_t(pstrName) ) {
				// Return the object's IUnknown pointer
				if ( dwReturnMask & SCRIPTINFO_IUNKNOWN ) {
					if (*ppiunkItem = itElement->pUnknown) {
						(*ppiunkItem)->AddRef();
					} // if
				} // if

				// Return the object's ITypeInfo pointer
				if ( dwReturnMask & SCRIPTINFO_ITYPEINFO ) {
					CComPtr<IProvideClassInfo> pClassInfo;
					hr = itElement->pUnknown->QueryInterface(IID_IProvideClassInfo,
						reinterpret_cast<LPVOID*>(&pClassInfo));
					if ( SUCCEEDED(hr) ) {
						hr = pClassInfo->GetClassInfo(ppti);
					} // if
				} // if
			} // if
		} // for
	} // try
	catch (...) {
		// Some error
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CScriptObject::Reset()
{
	// For this, we release our script objects and create new.
	HRESULT hr = S_OK;
	try {
		hr = m_pScript->SetScriptState(SCRIPTSTATE_INITIALIZED);
		if ( SUCCEEDED (hr) ) {
			// Now shut the thread down
			EXCEPINFO e;
			ZeroMemory(&e,sizeof(e));
			hr = m_pScript->InterruptScriptThread(SCRIPTTHREADID_ALL,
									&e,
									SCRIPTINTERRUPT_RAISEEXCEPTION);
			if ( hr == E_INVALIDARG ) {
				// For some reason, this method insists on sending
				// back this HRESULT when the arguments are correct.
				// Further more, the script will, in fact, be
				// reset.  So I filter this HRESULT and claim the
				// method worked...
				hr = S_OK;
			} // if
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch
	
	return hr;
}
