// ScriptObjectCtl.h : Declaration of the CScriptObject

#ifndef __SCRIPTOBJECT_H_
#define __SCRIPTOBJECT_H_

#include "resource.h"       // main symbols
#include "ScriptImpl.h"

#pragma warning(disable : 4786)
#include <vector>
using namespace std;

/////////////////////////////////////////////////////////////////////////////
// SCRIPTELEMENT
//
// This is our "storage unit" for scripted elements, such
// as our application and document objects.  Note I'm using
// the "built in" compiler support for BSTRs here (not ATL).
// I'm simply using the better tool.  In this case, we later
// need to make a string comparision, and CComBSTR doesn't
// have an operator== defined while bstr_t does.

typedef struct tagScriptElement {
	bstr_t bstrName;
	CComPtr<IUnknown> pUnknown;

	tagScriptElement(BSTR name = NULL, IUnknown* pUnk = NULL)
	{ bstrName = name; pUnknown = pUnk; }
} SCRIPTELEMENT, FAR * LPSCRIPTELEMENT;

bool operator<(const SCRIPTELEMENT& lhs, const SCRIPTELEMENT& rhs);
bool operator==(const SCRIPTELEMENT& lhs, const SCRIPTELEMENT& rhs);

typedef vector<SCRIPTELEMENT> ScriptElements;

/////////////////////////////////////////////////////////////////////////////
// CScriptObject

class ATL_NO_VTABLE CScriptObject : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CScriptObject, &CLSID_ScriptObject>,
	public IDispatchImpl<IScriptObject, &IID_IScriptObject, &LIBID_SCRIPTOBJECTLib>,
	public IActiveScriptSiteImpl,
	public IActiveScriptSiteWindowImpl
{
public:
	CScriptObject()
	{
	}

	HRESULT FinalConstruct();
	void FinalRelease();

DECLARE_REGISTRY_RESOURCEID(IDR_SCRIPTOBJECT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CScriptObject)
	COM_INTERFACE_ENTRY(IScriptObject)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IActiveScriptSite)
	COM_INTERFACE_ENTRY(IActiveScriptSiteWindow)
END_COM_MAP()

// IActiveScriptSite
public:
	STDMETHOD(GetItemInfo)(/*[in]*/ LPCOLESTR pstrName, /*[in]*/ DWORD dwReturnMask, /*[out]*/ IUnknown** ppiunkItem, /*[out]*/ ITypeInfo** ppti);

// IScriptObject
public:
	STDMETHOD(Reset)();
	STDMETHOD(ParseScript)(/*[in]*/ BSTR bstrScript);
	STDMETHOD(AddObject)(/*[in]*/ BSTR bstrObjName, /*[in]*/ IUnknown* lpUnk);

// Attributes
private:
	CComPtr<IActiveScriptParse> m_pScriptParser;
	CComPtr<IActiveScript> m_pScript;
	ScriptElements m_vScriptElements;
};

#endif //__SCRIPTOBJECT_H_
