// ScriptView.cpp : implementation of the CScriptView class
//

#include "stdafx.h"
#include "ScriptApp.h"

#include "ScriptDoc.h"
#include "ScriptView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScriptView

IMPLEMENT_DYNCREATE(CScriptView, CEditView)

BEGIN_MESSAGE_MAP(CScriptView, CEditView)
	//{{AFX_MSG_MAP(CScriptView)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScriptView construction/destruction

CScriptView::CScriptView()
{
	// TODO: add construction code here

}

CScriptView::~CScriptView()
{
}

BOOL CScriptView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	BOOL bPreCreated = CEditView::PreCreateWindow(cs);
	cs.style &= ~(ES_AUTOHSCROLL|WS_HSCROLL);	// Enable word-wrapping

	return bPreCreated;
}

/////////////////////////////////////////////////////////////////////////////
// CScriptView drawing

void CScriptView::OnDraw(CDC* pDC)
{
	CScriptDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CScriptView printing

BOOL CScriptView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default CEditView preparation
	return CEditView::OnPreparePrinting(pInfo);
}

void CScriptView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// Default CEditView begin printing.
	CEditView::OnBeginPrinting(pDC, pInfo);
}

void CScriptView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// Default CEditView end printing
	CEditView::OnEndPrinting(pDC, pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CScriptView diagnostics

#ifdef _DEBUG
void CScriptView::AssertValid() const
{
	CEditView::AssertValid();
}

void CScriptView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

CScriptDoc* CScriptView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CScriptDoc)));
	return (CScriptDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CScriptView helpers

void CScriptView::InitializeObjects() 
{
	try {
		// Create our scripting objects
		HRESULT hr = m_pDocObject.CreateInstance(CLSID_DocObject);
		if ( SUCCEEDED(hr) ) {
			// Initialize the document object
			CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
			ASSERT(pFrame != NULL);
			pFrame->AddDocObject(bstr_t("document"),m_pDocObject);
			m_pDocObject->InitViewHwnd(reinterpret_cast<long>(m_hWnd));
			m_pDocObject->InitEditHwnd(reinterpret_cast<long>(GetEditCtrl().m_hWnd));

			// Text selection object...	
			hr = m_pTextSelObject.CreateInstance(CLSID_TextSelObject);
			if ( SUCCEEDED(hr) ) {
				// Initialize the document object
				pFrame->AddTextSelObject(bstr_t("textselection"),m_pTextSelObject);
				m_pTextSelObject->InitHwnd(reinterpret_cast<long>(GetEditCtrl().m_hWnd));

				// Wire up the objects
				m_pDocObject->SetApplication(pFrame->GetAppObject());
				m_pDocObject->SetTextSelection(m_pTextSelObject);
				m_pTextSelObject->SetApplication(pFrame->GetAppObject());
				m_pTextSelObject->SetDocument(m_pDocObject);
			} // if
			else {
				// Some error...
				AfxMessageBox("Unable to create the text selection object",MB_OK|MB_ICONERROR);
			} // else
		} // if
		else {
			// Some error...
			AfxMessageBox("Unable to create the document object",MB_OK|MB_ICONERROR);
		} // else
	} // try
	catch (...) {
		// Some error...
		AfxMessageBox("Unable to create scripted objects",MB_OK|MB_ICONERROR);
	} // catch
}

/////////////////////////////////////////////////////////////////////////////
// CScriptView message handlers

void CScriptView::OnFileSave() 
{
	// Refer the save to the document
	CScriptDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->SaveFile();
}

void CScriptView::OnFileSaveAs() 
{
	// Refer the saveas to the document
	CScriptDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->SaveAsFile();
}

void CScriptView::OnFileOpen() 
{
	// Refer the open to the application
	CScriptApp* pApp = (CScriptApp*)AfxGetApp();
	ASSERT(pApp != NULL);
	pApp->OpenFile();
}

void CScriptView::OnFileNew() 
{
	// Refer the new to the application
	CScriptApp* pApp = (CScriptApp*)AfxGetApp();
	ASSERT(pApp != NULL);
	pApp->NewFile();
}
