// ToggleSplitWnd.cpp : implementation of the CToggleSplitWnd class
//

#include "stdafx.h"
#include "DocNavigator.h"

#include "SplitFrame.h"
#include "DocNavigatorDoc.h"
#include "SplitTreeView.h"
#include "DocNavigatorView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSplitFrame

IMPLEMENT_DYNAMIC(CSplitFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CSplitFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CSplitFrame)
	ON_COMMAND(ID_VIEW_TOGGLESPLITTER, OnToggleSplitter)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOGGLESPLITTER, OnUpdateToggleSplitter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSplitFrame construction/destruction

CSplitFrame::CSplitFrame(CRuntimeClass* pView1, CRuntimeClass* pView2) :
	m_pView1(pView1),
	m_pView2(pView2),
	m_bInitialOrientation(FALSE) // vertical
{
	// Initialize our sizes to zippo...
	m_splitSizes[0] = CSize(0,0);
	m_splitSizes[1] = CSize(0,0);
}

CSplitFrame::~CSplitFrame()
{
}

/////////////////////////////////////////////////////////////////////////////
// CSplitFrame diagnostics

#ifdef _DEBUG
void CSplitFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CSplitFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSplitFrame message handlers

BOOL CSplitFrame::OnCreateClient(LPCREATESTRUCT /* lpcs */, CCreateContext* pContext)
{
	// Size initially...
	CSize sPane1, sPane2;
	MakeSplitSizes(m_bInitialOrientation,sPane1,sPane2);

	// Establish initial orientation (rows/cols)
	int nRows = 1, nCols = 2; // vertical
	if ( m_bInitialOrientation ) {
		// Horizontal
		nCols = 1;
		nRows = 2;
	} // if

	// Create the views...
	BOOL bSuccess;
	bSuccess  = m_wndSplitter.CreateStatic(this,nRows,nCols);
	bSuccess &= m_wndSplitter.CreateView(0,0,m_pView1,sPane1,pContext);
	bSuccess &= m_wndSplitter.CreateView(nRows-1,nCols-1,m_pView2,sPane2,pContext);
	ASSERT(bSuccess);

	return bSuccess;
}

void CSplitFrame::OnToggleSplitter()
{
	// Toggle orientation
	if ( m_wndSplitter.GetSafeHwnd() ) {
		// Save current size
		CacheSize();

		// Do the toggle
		m_wndSplitter.Toggle();

		CSize sPane1, sPane2;
		MakeSplitSizes(m_wndSplitter.BarIsHorizontal(),sPane1,sPane2);
		m_wndSplitter.UpdateSizes(sPane1,sPane2);
	} // if
}

void CSplitFrame::OnUpdateToggleSplitter(CCmdUI* pCmdUI)
{
	// Update menu
	CString strMenuItem;

	if ( IsHorizontal() ) {
		// We are horizontal, so we want to show the menu
		// item for vertical.
		strMenuItem.LoadString(IDS_VERTICAL);
	} // if
	else {
		// We are vertical, so we want to show the menu
		// item for horizontal.
		strMenuItem.LoadString(IDS_HORIZONTAL);
	} // else

	// Change the menu item text
	pCmdUI->SetText(strMenuItem);
}

void CSplitFrame::MakeSplitSizes(BOOL bHorizontal, CSize& sPane1, CSize& sPane2)
{
	CRect rcClient;
	GetClientRect(&rcClient);
	CSize sClient = rcClient.Size();
	CSize sCached = ReadCachedSize(bHorizontal);

	// If we've been properly sized...
	if ((sCached.cx > 0) && (sCached.cy > 0)) {
		// Adjust sizings using cached values
		sPane2.cx = sClient.cx - sCached.cx;
		sPane2.cy = sClient.cy - sCached.cy;
		sPane1.cx = sCached.cx;
		sPane1.cy = sCached.cy;
	} // if
	else {
		// Create new sizings (arbitrary values...)
		sPane1 = sClient;
		sPane1.cx /= 3;
		sPane1.cy /= 2;

		sPane2.cx = sPane1.cx * 2;
		sPane2.cy = sPane1.cy * 1;
	} // else
}

CWnd* CSplitFrame::RetrieveView(int nView)
{
	// Display one of the two window views...
	ASSERT((nView == 1) || (nView == 2));

	// Get appropriate view...
	if ( nView == 1 ) {
		return m_wndSplitter.GetPane(0,0);
	} // if

	return m_wndSplitter.GetPane(0,1);
}

void CSplitFrame::OnViewBoth() 
{
	// Retrieve cached split sizes and adjust views accordingly...
	CSize sPane1, sPane2;
	MakeSplitSizes(IsHorizontal(),sPane1,sPane2);
	m_wndSplitter.UpdateSizes(sPane1,sPane2);
}

void CSplitFrame::OnViewFirstView() 
{
	// Show first view only
	ShowView(1);
}

void CSplitFrame::OnViewSecondView() 
{
	// Show second view only
	ShowView(2);
}

void CSplitFrame::ShowView(int nView) 
{
	// Display one of the two window views...
	ASSERT((nView == 1) || (nView == 2));

	// Save our current size for later use...
	CacheSize();

	// Determine our current sizings...
	CRect rcClient;
	GetClientRect(&rcClient);

	// Set appropriate view size to zippo...
	if ( nView == 1 ) {
		m_wndSplitter.UpdateSizes(rcClient.Size(),CSize(0,0));
	} // if
	else {
		m_wndSplitter.UpdateSizes(CSize(0,0),rcClient.Size());
	} // else
}

void CSplitFrame::CacheSize()
{
	BOOL bOkToWrite = FALSE;
	int nMin, nTemp1, nTemp2;

	// Retrieve pane info based on orientation...
	if ( IsHorizontal() ) {
		m_wndSplitter.GetRowInfo(0,nTemp1,nMin);
		m_wndSplitter.GetRowInfo(1,nTemp2,nMin);
	} // if
	else {
		m_wndSplitter.GetColumnInfo(0,nTemp1,nMin);
		m_wndSplitter.GetColumnInfo(1,nTemp2,nMin);
	} // else

	// Establish our "cachability"
	bOkToWrite = ((nTemp1 > 0) && (nTemp2 > 0));

	// We want to discount the maximized views...
	if ( bOkToWrite ) {
		// Retrieve current sizing information.  Note we don't care
		// about the minimum sizing data.
		int x, y;
		m_wndSplitter.GetColumnInfo(0,x,nMin);
		m_wndSplitter.GetRowInfo(0,y,nMin);

		// Save size to cache
		m_splitSizes[IsHorizontal() ? 1 : 0] = CSize(x,y);
	} // if
}

CSize CSplitFrame::ReadCachedSize(BOOL bHorizontal)
{
	// Retrieve our cached size, based on orientation
	return m_splitSizes[bHorizontal ? 1 : 0];
}
