// SplitTreeView.cpp : implementation of the CSplitTreeView class
//

#include "stdafx.h"
#include "DocNavigator.h"

#include "DocNavigatorDoc.h"
#include "MainFrm.h"
#include "SplitTreeView.h"
#include "DocNavigatorView.h"

// DHTML definitions
#include "MSHTML.H"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSplitTreeView

IMPLEMENT_DYNCREATE(CSplitTreeView, CTreeView)

BEGIN_MESSAGE_MAP(CSplitTreeView, CTreeView)
	//{{AFX_MSG_MAP(CSplitTreeView)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelchanged)
	//}}AFX_MSG_MAP
	ON_WM_DESTROY()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSplitTreeView construction/destruction

CSplitTreeView::CSplitTreeView()
{
}

CSplitTreeView::~CSplitTreeView()
{
}

BOOL CSplitTreeView::PreCreateWindow(CREATESTRUCT& cs)
{
	// Add styles for tree control
    cs.style |= TVS_HASLINES | TVS_LINESATROOT;

	return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSplitTreeView drawing

void CSplitTreeView::OnDraw(CDC* pDC)
{
	CDocNavigatorDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Let tree control draw itself...
	CTreeView::OnDraw(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CSplitTreeView diagnostics

#ifdef _DEBUG
void CSplitTreeView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CSplitTreeView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}

CDocNavigatorDoc* CSplitTreeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDocNavigatorDoc)));
	return (CDocNavigatorDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSplitTreeView helpers

void CSplitTreeView::ClearTreeControl()
{
	// We've loaded a new HTML file...we now clear any
	// tree data we may already have.
	int iNumItems = GetTreeCtrl().GetCount();
	if ( iNumItems ) {
		HTREEITEM hCurrentNode = GetTreeCtrl().GetNextItem(reinterpret_cast<HTREEITEM>(TVI_ROOT),TVGN_ROOT);
		for ( int iItem = 0; iItem < iNumItems; iItem++ ) {
			// Run through tree and delete the bookmarks
			TVITEM tvItem;
			tvItem.mask = TVIF_PARAM | TVIF_STATE;
			tvItem.hItem = hCurrentNode;
			if ( GetTreeCtrl().GetItem(&tvItem) ) {
				CString* pstrBookmark = reinterpret_cast<CString*>(tvItem.lParam);
				delete pstrBookmark;
			} // if

			// Check to see if this item is selected...
			if ( tvItem.state & TVIS_SELECTED ) {
				// If this item is selected, reset it
				tvItem.lParam = NULL;
				tvItem.state &= TVIS_SELECTED;
				tvItem.stateMask |= TVIS_SELECTED;
				GetTreeCtrl().SetItem(&tvItem);
			} // if

			// Get next item...
			hCurrentNode = GetTreeCtrl().GetNextItem(hCurrentNode,TVGN_NEXT);
		} // for
	} // if

	// Clear the tree control
	GetTreeCtrl().DeleteAllItems();
}

void CSplitTreeView::UpdateTree(LPDISPATCH pDocDispatch)
{
	// Clear an existing data
	ClearTreeControl();

	// Now build a new tree... Check to see that
	// we do have a document.  The HTML document
	// pointer will be NULL if the browser was unable
	// to load the HTML file.
	if ( pDocDispatch != NULL ) {
		// We have a document, so now query for the
		// IHTMLDocument2 interface
		CComQIPtr<IHTMLDocument2,&IID_IHTMLDocument2> pHTMLDocument2(pDocDispatch);
		if ( pHTMLDocument2.p != NULL ) {
			// We have an active document, so retrieve the
			// "all" element collection.
			CComPtr<IHTMLElementCollection> pElements;
			HRESULT hr = pHTMLDocument2->get_anchors(&pElements);
			if ( SUCCEEDED(hr) ) {
				// Pull the elements
				long iNumElements;
				hr = pElements->get_length(&iNumElements);
				if ( SUCCEEDED(hr) && iNumElements ) {
					// Loop through the number of items
					// and pull its IHTMLElement interface
					// pointer.
					for ( long i = 0; i < iNumElements; i++ ) {
						// Retrieve item in collection
						CComVariant varName(i);
						varName.ChangeType(VT_UINT);
						CComVariant varIndex;
						CComPtr<IDispatch> pDisp; 
						hr = pElements->item(varName,varIndex,&pDisp);
						if ( SUCCEEDED(hr) ) {
							// We have the element, so retrieve its
							// tag name.
							CComQIPtr<IHTMLElement,
								&IID_IHTMLElement> pElement(pDisp);
							if ( pElement != NULL ) {
								CComBSTR bstrAnchor;
								CComQIPtr<IHTMLAnchorElement,
									&IID_IHTMLAnchorElement> pAnchor(pDisp);
								if ( pAnchor ) {
									// Was a link...
									pAnchor->get_name(&bstrAnchor);
									if ( !bstrAnchor ) {
										// Not a bookmark...
										continue;
									} // if

									// Retrieve anchor text
									CString strAnchor;
									strAnchor = bstrAnchor.m_str;
									CString* pstrBookmark = new CString(strAnchor);

									// Retrieve anchor text name
									pElement->get_innerText(&bstrAnchor);
									if ( !bstrAnchor ) {
										// No text...
										strAnchor = _T("???");
									} // if
									else {
										strAnchor = bstrAnchor.m_str;
									} // else

									// Add this anchor to the tree
									GetTreeCtrl().InsertItem(TVIF_TEXT | TVIF_PARAM,
										strAnchor,
										0,0,0,0,
										(LPARAM)pstrBookmark,
										reinterpret_cast<HTREEITEM>(TVI_ROOT),
										reinterpret_cast<HTREEITEM>(TVI_LAST));

									// Continue to next anchor
									continue;
								} // if
						 	} // if
						} // if
					} // for
				} // if
			} // if
		} // if
		else {
			// We couldn't retrieve the IHTMLDocument2
			// interface.
			AfxMessageBox("Unable to retrieve the active document",
							MB_OK | MB_ICONERROR);
		} // else
	} // if
}

/////////////////////////////////////////////////////////////////////////////
// CSplitTreeView message handlers

void CSplitTreeView::OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	// Examine notification structure to determine new
	// selection
	if ((pNMTreeView->itemNew.state & TVIS_SELECTED) &&
			(pNMTreeView->action)) {
		CString* pstrBookmark = reinterpret_cast<CString*>(pNMTreeView->itemNew.lParam);
		ASSERT(!pstrBookmark->IsEmpty());
		if ( !pstrBookmark->IsEmpty() ) {
			CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
			pFrame->Navigate((LPCTSTR)pstrBookmark->GetBuffer(0));
		} // if
	} // if

	*pResult = 0;
}

void CSplitTreeView::OnDestroy() 
{
	// Clear any bookmark data we may have created
	ClearTreeControl();
	
	CTreeView::OnDestroy();
}

void CSplitTreeView::OnFileOpen()
{
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	pFrame->BrowseURL();
}

void CSplitTreeView::OnFilePrint()
{
	CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
	pFrame->PrintURL();
}
