// StringEncrypt.cpp : Implementation of CStringEncrypt
#include "stdafx.h"
#include "ATLEncrypt.h"
#include "StringEncrypt.h"

/////////////////////////////////////////////////////////////////////////////
// CStringEncrypt


// Define an arbitrary bit mask.  Pick anything, but sprinkle
// in a fairly even mix of 1's and 0's.
#define MASK 0x9A;

// Define a shift value.  We'll use this to rotate the bits of
// the nearly-encrypted character byte in memory this number
// of bits.  Pick any value from 1 to 7...I picked 3 bits for
// no particular reason.
#define SHIFT 0x03

// Define a buffer filled with trash values (it's 34
// characters long).  We could generate random values
// using rand(), but this is just as easy and faster
// to access.
char trash[] = "a54%Jg*b(+/sA}`Wnc#j_(84NrLk3$7^Z\0";

// A gentle reminder...  The assembly instructions below are for X86
// processors only.  Alphas and PowerPCs are out of luck, as I don't
// have their assembly manuals...  Compilation on other than an Intel
// processor will cause an error.
#ifndef _M_IX86
#pragma message("***NOTE: You *must* add assembly code to rotate encrypted/decrypted bits for this processor type!")
#endif

STDMETHODIMP CStringEncrypt::Encrypt(BSTR bstrClearText, BSTR * pbstrMunged)
{
	// Check out pointer
	_ASSERTE(pbstrMunged != NULL);
	if ( pbstrMunged == NULL ) {
		// Sorry...
		return E_POINTER;
	} // if

	// Always wrap COM code in try/catch!
	HRESULT hr = E_FAIL;
	try {
		// Convert the input string from BSTR to TCHAR
		USES_CONVERSION;
		LPTSTR strClearText = OLE2T(bstrClearText);
		TCHAR strMunged[MaxPasswordBuffer] = {0};

		// Check the string length...if too long, return an error
		if ( lstrlen(strClearText) > MaxPasswordLength ) {
			// Too long...too bad
			return E_INVALIDARG;
		} // if

		// We'll apply a simple encryption techique, following this
		// basic recipe.  Decryption will reverse this process.
		//
		// Step 1: Determine the length of the "clear text"
		// (unencrypted) password string.  It will be from zero
		// to some maximum number of characters long (32 in this
		// case, which is a purely arbitrary number).
		//
		// Step 2: In a new buffer, copy the single-byte length
		// value into the first byte of the buffer.  This stores
		// our original password length in the buffer we're about
		// to encrypt.
		//
		// Step 3: Copy the characters from the clear text password
		// into the encryption buffer, starting at the second byte
		// in the buffer (the length is stored in the first byte).
		//
		// Step 4: If our password is *less than* the maximum number
		// of password characters long (32 in this case), we'll "pad"
		// to 32 characters with garbage.  We do this so all encrypted
		// password strings are always the same length. That way,
		// you can't just look at it and deduce the password based on
		// length.  If your password was your dog's name, something
		// like "Muffy", and I know you well and saw the encrypted
		// password was five bytes long, I might guess "Muffy".  Sounds
		// corny, but it *really does* happen that way sometimes.  Now
		// you see why the MaxPasswordBuffer is 34 bytes...one for length,
		// 32 for password and  padding trash, and one for null termination.
		//
		// Step 5: For each of the characters in the encryption buffer,
		// we want to shift their individual bit masks over in memory by some
		// value.  I chose their character position mod'd with 8 (8 bits in
		// a byte).  This encrypts adjacent characters differently.  Back to
		// "Muffy".  If I didn't do something to seemingly randomize
		// the encrypted values, both of the 'f's in "Muffy" would be
		// encrypted to the *same* value.  This tells me, the hacker,
		// a lot.  I only need to guess FOUR characters, not FIVE, as I
		// know two of them are the same.  Sure it's a guess on my part,
		// but an educated one that is correct more times than not.
		//
		// Step 6: We exclusive-or the characters in the encryption
		// buffer with our arbitrary bit mask (which has been shifted
		// in memory).  This changes the bits in the encryption buffer,
		// further hiding their true value.  Note we can get the character
		// back by exclusive-or'ing with the mask value again (shifed
		// properly, of course).
		//
		// Step 7: We perform a final bit shift on each byte of the encryption
		// buffer, just for grins.  It doesn't cost that much and it
		// adds an extra measure of security.
		//
		// Step 8: Allocate a new BSTR based on the encryption buffer.  This
		// is our new, encrypted string, ready for persistence or whatever.
		//
		// Do this, and the resulting password string you'll receive will be
		// completely unrecognizable.
		TCHAR cEncryptBuffer[MaxPasswordBuffer];
		cEncryptBuffer[0] = (TCHAR)lstrlen((LPTSTR)strClearText); // Step 1/2

		for ( int i = 0; i < cEncryptBuffer[0]; i++ ) {
			// Step 3...
			cEncryptBuffer[i+1] = strClearText[i]; // "i+1" skips the length byte...
		} // for

		for ( /* nothing */; i < MaxPasswordBuffer; i++ ) {
			// Step 4...
			cEncryptBuffer[i+1] = trash[i]; // automatically adds null terminator...
		} // for

		TCHAR cByte, cMask;
		for ( i = 0; i <= MaxPasswordBuffer-1; i++ ) {
			// Assign the temporary character value
			cMask = (TCHAR)MASK;
			cByte = cEncryptBuffer[i];

			// Apply the mask to the bits.  Don't let the assembly
			// instructions scare you.  All they do is "rotate"
			// the bits iMaskShift bits to the left.  The most significant
			// bit becomes the least significant bit, and so on,
			// for "iMaskShift" bits.  I did it this way as I wanted a
			// true bit rotation, *not* a shift, as the << operator
			// would provide.  When using the << operator, the bits
			// falling off the end are destroyed, whereas I want them
			// to rotate back through the byte...
			TCHAR iMaskShift = i % 8; // Step 5
			__asm {
				MOV cl,iMaskShift
				ROL cMask,cl
			} // __asm
			cByte ^= cMask; // Step 6

			// Shift the bits...Step 7
			__asm ROL cByte,SHIFT

			// Re-assign the password with encrypted values, Step 8
			strMunged[i] = cByte;
		} // for

		// Terminate the buffer
		strMunged[MaxPasswordBuffer-1] = '\0';

		// Pass the buffer back...here we create a new BSTR and assign
		// it to the pointer we were given.
		CComBSTR bstrMungeBuf(strMunged);
		*pbstrMunged = ::SysAllocString(bstrMungeBuf.m_str);

		hr = S_OK;
	} // try
	catch (...) {
		// Return a failure HRESULT
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CStringEncrypt::Decrypt(BSTR bstrMunged, BSTR * pbstrClearText)
{
	// Check out pointer
	_ASSERTE(pbstrClearText != NULL);
	if ( pbstrClearText == NULL ) {
		// Sorry...
		return E_POINTER;
	} // if

	// Always wrap COM code in try/catch!
	HRESULT hr = E_FAIL;
	try {
		// Convert the input string from BSTR to TCHAR
		USES_CONVERSION;
		LPTSTR strMunged = OLE2T(bstrMunged);
		TCHAR strClearText[MaxPasswordLength+1] = {0};

		// Check the string length...if too long, return an error
		if ( lstrlen(strMunged) > MaxPasswordBuffer ) {
			// Too long...too bad
			return E_INVALIDARG;
		} // if

		// Decryption is a simple reversal of the process we
		// applied previously.  Note the order of events changes
		// slightly:
		//
		// Step 1: Rotate the individual characters "SHIFT" bits
		// TO THE RIGHT (they shifted left when encrypted).
		//
		// Step 2: Rotate our bit mask based upon character
		// position (as before).
		//
		// Step 3: Apply the mask using the exclusive-or operator
		// (as before).
		//
		// Step 4: Assign the cleared byte to the decryption buffer.
		// This "holds" the string temporarily while we rotate and 
		// mask all of the bytes in the stored password buffer.
		//
		// Step 5: Pull the length byte from the decryption buffer.
		//
		// Step 6: Assign the "good" characters to a temporary password
		// buffer.  The password is now fully clear text again.
		//
		// Step 7: Add the null terminator to the temporary password
		// buffer.
		//
		TCHAR cDecryptBuffer[MaxPasswordBuffer];
		TCHAR cByte, cMask;
		for ( int i = 0; i <= MaxPasswordBuffer-1; i++ ) {
			// Assign the temporary character value
			cMask = (TCHAR)MASK;
			cByte = strMunged[i];

			// Shift the bits back again, Step 1
			__asm ROR cByte,SHIFT

			// Apply the mask to the bits
			TCHAR iMaskShift = i % 8; // Step 2
			__asm {
				MOV cl,iMaskShift
				ROL cMask,cl
			} // __asm
			cByte ^= cMask; // Step 3

			// Re-assign the password with encrypted values, Step 4
			cDecryptBuffer[i] = cByte;
		} // for

		// Pull the nearly-decrypted string from the buffer, Step 5
		for ( i = 0; i < cDecryptBuffer[0]; i++ ) {
			// Step 6...
			 strClearText[i] = cDecryptBuffer[i+1];
		} // for
		strClearText[i] = '\0'; // add appropriate null termination, Step 7

		// Pass the buffer back...here we create a new BSTR and assign
		// it to the pointer we were given.
		CComBSTR bstrDecryptBuf(strClearText);
		*pbstrClearText = ::SysAllocString(bstrDecryptBuf.m_str);

		hr = S_OK;
	} // try
	catch (...) {
		// Return a failure HRESULT
		hr = E_FAIL;
	} // catch

	return hr;
}
