// TabSheet.cpp : implementation file
//

#include "stdafx.h"
#include "GLObject.h"
#include "TabSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTabSheet

CTabSheet::CTabSheet() :
	m_nCurSel(0)
{
}

CTabSheet::~CTabSheet()
{
}


BEGIN_MESSAGE_MAP(CTabSheet, CTabCtrl)
	//{{AFX_MSG_MAP(CTabSheet)
	ON_NOTIFY_REFLECT(TCN_SELCHANGE, OnSelchange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTabSheet message handlers

void CTabSheet::OnSelchange(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Determine new page and activate it
	int nOldPage = m_nCurSel;
	m_nCurSel = GetCurSel();
	ActivatePage(nOldPage,m_nCurSel);

	*pResult = 0;
}

void CTabSheet::AddTab(CPropertyPage* pdlgPage, LPTSTR strText, UINT nCtrlID /*=IDC_STATIC*/)
{
	// Some checks
	ASSERT_VALID(this);
	ASSERT(pdlgPage != NULL);
	ASSERT(strText != NULL);
	ASSERT_KINDOF(CPropertyPage, pdlgPage);
	ASSERT_VALID(pdlgPage);

	// Add page to our list of pages
	int nIndex = m_pages.Add(pdlgPage);

	// Set up the tab
	TC_ITEM item;
	item.mask = TCIF_TEXT;
	item.pszText = strText;
	item.cchTextMax = lstrlen(strText);
	InsertItem(nIndex,&item);

	// Create the page and hide
	if ( pdlgPage->m_hWnd == NULL ) {
		// Not been seen yet...
		pdlgPage->Create(pdlgPage->m_psp.pszTemplate,this);
		pdlgPage->SetDlgCtrlID(nCtrlID); // must be non-zero to send notifications
		CRect rcTabClient;
		GetClientRect(rcTabClient);
		ClientToScreen(rcTabClient);
		AdjustRect(FALSE,rcTabClient);
		ScreenToClient(rcTabClient);
		pdlgPage->MoveWindow(rcTabClient.left,
							rcTabClient.top,
							rcTabClient.Width(),
							rcTabClient.Height());
	} // if
	pdlgPage->ShowWindow(SW_HIDE);

	if ( !nIndex ) {
		// Activate first page...
		ActivatePage(-1,0);
	} // if
}

void CTabSheet::ActivatePage(int nOldPage, int nNewPage)
{
	// Bring down any previous the property page...
	if ( nOldPage != -1 ) {
		CPropertyPage* pOldPage = (CPropertyPage*)m_pages[nOldPage];
		pOldPage->ShowWindow(SW_HIDE);
		pOldPage->OnKillActive();
	} // if

	// Bring up the new property page...
	CPropertyPage* pNewPage = (CPropertyPage*)m_pages[nNewPage];
	pNewPage->OnSetActive();
	pNewPage->ShowWindow(SW_SHOWNORMAL);
}

void CTabSheet::OnDestroy() 
{
	// Kill our dialog pages
	for ( int i = 0; i < m_pages.GetSize(); i++ ) {
		// Destroy the window
		CPropertyPage* pPage = (CPropertyPage*)m_pages[i];
		pPage->DestroyWindow();
	} // for

	// Remove our pointer list items
	m_pages.RemoveAll();

	CTabCtrl::OnDestroy();
}
