// TableView.cpp : implementation of the CTableView class
//

#include "stdafx.h"
#include "BlackJack.h"
#include "TableView.h"
#include "MainFrm.h"
#include "DDExcept.h"
#include <time.h> // for srand() initialization

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTableView

CTableView::CTableView() :
	m_pIDirectDraw(NULL),
	m_lpDDPalette(NULL)
{
	// Seed random number generator
	srand((unsigned)time(NULL));
}

CTableView::~CTableView()
{
}


BEGIN_MESSAGE_MAP(CTableView,CWnd )
	//{{AFX_MSG_MAP(CTableView)
	ON_WM_PAINT()
	ON_WM_CREATE()
	ON_COMMAND(ID_MOVE_STANDPAT, OnMoveStandpat)
	ON_COMMAND(ID_MOVE_HITME, OnMoveHitme)
	ON_COMMAND(ID_MOVE_NEWGAME, OnMoveNewGame)
	//}}AFX_MSG_MAP
	// Palette handlers
	ON_WM_PALETTECHANGED()
	ON_WM_QUERYNEWPALETTE()
	// Destruction
	ON_WM_DESTROY()
	// Dialog bar handlers
	ON_BN_CLICKED(IDC_HITME, OnMoveHitme)
	ON_BN_CLICKED(IDC_STANDPAT, OnMoveStandpat)
	ON_BN_CLICKED(IDC_NEWGAME, OnMoveNewGame)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTableView helpers

BOOL CTableView::RePalettize() 
{
	// Reload our bitmaps...
	BOOL bReturn = TRUE;
	HRESULT hr = S_OK;
	try {
		hr = ReLoadDDBitmap(IDB_TABLE,m_lpDDSTable);
		if FAILED( hr ) {
			// Some error
			throw new CDDException(TRUE,"Error reloading table bitmap");
		} // if

		hr = ReLoadDDBitmap(IDB_CARDS1,m_lpDDSCards1);
		if FAILED( hr ) {
			// Some error
			throw new CDDException(TRUE,"Error reloading first card bitmap");
		} // if

		hr = ReLoadDDBitmap(IDB_CARDS2,m_lpDDSCards2);
		if FAILED( hr ) {
			// Some error
			throw new CDDException(TRUE,"Error reloading second card bitmap");
		} // if

		hr = ReLoadDDBitmap(IDB_CARDBACK,m_lpDDSCardBack);
		if FAILED( hr ) {
			// Some error
			throw new CDDException(TRUE,"Error reloading card back bitmap");
		} // if
	} // try
	catch (CDDException* dde) {
		// Intercept our special exception and
		// tell user of error
		dde->ReportError(MB_OK | MB_ICONERROR,hr);

		// Delete the exception
		dde->Delete();

		// Return failure
		bReturn = FALSE;
	} // catch
	catch (...) {
		// Return failure
		bReturn = FALSE;
	} // catch

	return bReturn;
}

/////////////////////////////////////////////////////////////////////////////
// CTableView message handlers

BOOL CTableView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW+1), NULL);

	return TRUE;
}

void CTableView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// Copy our secondary buffer to primary
	Blt();
	
	// Do not call CWnd::OnPaint() for painting messages
}

int CTableView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	int iReturn = 0;
	HRESULT hr = S_OK;
	try {
		// Create the DirectDraw object.  We won't need the
		// capabilities of IDirectDraw2, so there isn't
		// a need to QI.
		hr = DirectDrawCreate(NULL,&m_pIDirectDraw,NULL);
		if FAILED( hr ) {
			// Some error
			throw new CDDException(TRUE,"Error creating DirectDraw object");
		} // if

		// Set our cooperative level to be windowed...
		hr = m_pIDirectDraw->SetCooperativeLevel(m_hWnd,
						DDSCL_NORMAL);
		if FAILED( hr ) {
			// Some error
			throw new CDDException(TRUE,"Error setting cooperative level");
		} // if

		// Create our primary surface
		DDSURFACEDESC ddsd;
		ZeroMemory(&ddsd,sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
    
		hr = m_pIDirectDraw->CreateSurface(&ddsd,&m_lpDDSPrimary,NULL);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error creating primary surface");
		} // if

		// Since we're windowed, we need to create a clipper
		// and attach it to the primary surface.
		CComPtr<IDirectDrawClipper> lpDDClipper;
		hr = m_pIDirectDraw->CreateClipper(0,&lpDDClipper,NULL);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error creating the clipper");
		} // if

		// We now associate the clipper with our frame
		// so Windows will update the clipping region for
		// us.
		CMainFrame* pFrame = (CMainFrame*)AfxGetMainWnd();
		ASSERT(pFrame);
		hr = lpDDClipper->SetHWnd(0,pFrame->GetSafeHwnd());
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error setting the clipper\'s HWND");
		} // if

		// Associate our clipper with the primary surface
		hr = m_lpDDSPrimary->SetClipper(lpDDClipper);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error associating the clipper with the primary surface");
		} // if

		// Check the surface format...will we need a palette?
		DDPIXELFORMAT ddpf;
		ZeroMemory(&ddpf,sizeof(ddpf));
		ddpf.dwSize = sizeof(ddpf);
		hr = m_lpDDSPrimary->GetPixelFormat(&ddpf);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error retrieving pixel format");
		} // if

		if ( ddpf.dwFlags & DDPF_PALETTEINDEXED8 ) {
			// We are palettized, so create a palette and attach it to
			// the primary surface.
			hr = LoadDDPalette(IDB_CARDS2,&m_lpDDPalette);
			if ( FAILED( hr ) ) {
				// Some error
				throw new CDDException(TRUE,"Error establishing a new palette");
			} // if

			m_lpDDSPrimary->SetPalette(m_lpDDPalette);
			if ( FAILED( hr ) ) {
				// Some error
				throw new CDDException(TRUE,"Error assigning the new palette to the primary surface");
			} // if
		} // if

		// Load our table bitmap
		hr = LoadDDBitmap(IDB_TABLE,&m_lpDDSTable);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error loading table bitmap");
		} // if

		// Load our card bitmaps
		hr = LoadDDBitmap(IDB_CARDS1,&m_lpDDSCards1);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error loading cards 1 bitmap");
		} // if

		hr = LoadDDBitmap(IDB_CARDS2,&m_lpDDSCards2);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error loading cards 2 bitmap");
		} // if

		hr = LoadDDBitmap(IDB_CARDBACK,&m_lpDDSCardBack);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error loading card back bitmap");
		} // if
		
		// Create our back buffer
		hr = CreateSecondarySurface(350,300,&m_lpDDSSecondary);
		if ( FAILED( hr ) ) {
			// Some error
			throw new CDDException(TRUE,"Error creating secondary back buffer");
		} // if

		// Deal first hand
		Reset();
	} // try
	catch (CDDException* dde) {
		// Intercept our special exception and
		// tell user of error
		dde->ReportError(MB_OK | MB_ICONERROR,hr);

		// Delete the exception
		dde->Delete();

		// Return failure
		iReturn = -1;
	} // catch
	catch (...) {
		// Return failure
		iReturn = -1;
	} // catch

	// NOTE:  At this point the window has been
	// created.  If we threw an exception and are
	// returning -1 (to indicate failure during
	// creation), a WM_DESTROY message will still
	// be issued.  The net result is if we created
	// and DirectDraw objects prior to the
	// exception, they will automatically be
	// released at that time.  No special action is
	// required here.
	return iReturn;
}

void CTableView::OnPaletteChanged(CWnd* pFocusWnd) 
{
	// Someone changed the palette...was it us?
	if ( pFocusWnd->GetSafeHwnd() == this->m_hWnd ) {
		// It was us, so let base class handle the
		// message
		CWnd::OnPaletteChanged(pFocusWnd);
	} // if
	else {
		// It wasn't us, so repalettize
		RePalettize();
	} // if
}

BOOL CTableView::OnQueryNewPalette() 
{
	// We now control the palette, so repalettize
	return RePalettize();
}

void CTableView::OnMoveStandpat() 
{
	// Hold these cards...
	m_mvLastDecision = StandPat;
	Move();
}

void CTableView::OnMoveHitme() 
{
	// Deal player another...
	m_mvLastDecision = HitMe;
	Move();
}

void CTableView::OnMoveNewGame() 
{
	// Reset the game
	Reset();
	Blt();
}

void CTableView::OnDestroy() 
{
	// Release our surfaces first.  The order in which
	// DirectDraw objects are released is important.
	// Surfaces first, then the palette, then finally
	// the DirectDraw object.
	m_lpDDSPrimary = NULL;
	m_lpDDSSecondary = NULL;
	m_lpDDSCards1 = NULL;
	m_lpDDSCards2 = NULL;
	m_lpDDSCardBack = NULL;
	m_lpDDSTable = NULL;

	// Now the palette.  If we never created one,
	// this call will still succeed (thanks to
	// CComPtr).
	m_lpDDPalette = NULL;

	// Finally release our DirectDraw object
	m_pIDirectDraw = NULL;

	CWnd::OnDestroy();
}
