// TextSelObjectCtl.cpp : Implementation of CTextSelObject
#include "stdafx.h"
#include "TextObject.h"
#include "TextSelObjectCtl.h"

/////////////////////////////////////////////////////////////////////////////
// CTextSelObject

STDMETHODIMP CTextSelObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ITextSelObject
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CTextSelObject::get_Application(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Copy in our application's IDispatch pointer
			VARIANT var;
			var.vt = VT_DISPATCH;
			var.pdispVal = m_pAppObject.p;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::get_Document(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Copy in our document's IDispatch pointer
			VARIANT var;
			var.vt = VT_DISPATCH;
			var.pdispVal = m_pDocObject.p;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::get_UpperSelBound(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Determine our current selection bounds
			DWORD dwUpper = 0;
			::SendMessage(m_hWndEdit,EM_GETSEL,(WPARAM)NULL,(LPARAM)&dwUpper);

			// Copy in the requested value
			VARIANT var;
			var.vt = VT_I4;
			var.lVal = dwUpper;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::put_UpperSelBound(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current selection bounds
				DWORD dwLower = 0, dwUpper = 0;
				::SendMessage(m_hWndEdit,EM_GETSEL,(WPARAM)&dwLower,(LPARAM)&dwUpper);

				// Select the given range
				dwUpper = var.lVal;
				::SendMessage(m_hWndEdit,EM_SETSEL,(WPARAM)dwLower,(LPARAM)dwUpper);
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::get_LowerSelBound(VARIANT *pVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointer
		if ( pVal != NULL ) {
			// Clear their variant
			::VariantClear(pVal);

			// Determine our current selection bounds
			DWORD dwLower = 0;
			::SendMessage(m_hWndEdit,EM_GETSEL,(WPARAM)&dwLower,(LPARAM)NULL);

			// Copy in the requested value
			VARIANT var;
			var.vt = VT_I4;
			var.lVal = dwLower;
			::VariantCopy(pVal,&var);
		} // if
		else {
			// Wasn't a valid pointer
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::put_LowerSelBound(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_I4,&newVal)) ) {
			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine our current selection bounds
				DWORD dwLower = 0, dwUpper = 0;
				::SendMessage(m_hWndEdit,EM_GETSEL,(WPARAM)&dwLower,(LPARAM)&dwUpper);

				// Select the given range
				dwLower = var.lVal;
				::SendMessage(m_hWndEdit,EM_SETSEL,(WPARAM)dwLower,(LPARAM)dwUpper);
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::SetApplication(LPDISPATCH newVal)
{
	HRESULT hr = S_OK;
	try {
		// Accept new value
		if ( newVal == NULL ) {
			// Invalid pointer
			ATLASSERT(newVal != NULL);
			hr = E_POINTER;
		} // if
		else {
			// Set the pointer
			m_pAppObject = newVal;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::SetDocument(LPDISPATCH newVal)
{
	HRESULT hr = S_OK;
	try {
		// Accept new value
		if ( newVal == NULL ) {
			// Invalid pointer
			ATLASSERT(newVal != NULL);
			hr = E_POINTER;
		} // if
		else {
			// Set the pointer
			m_pDocObject = newVal;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::GetCursorPos(VARIANT *pLineVal, VARIANT *pIndexVal)
{
	HRESULT hr = S_OK;
	try {
		// Check their pointers
		if ((pLineVal != NULL) && (pIndexVal != NULL)) {
			// Clear their variants
			::VariantClear(pLineVal);
			::VariantClear(pIndexVal);

			// Determine our current character position
			POINT pt;
			::GetCaretPos(&pt);
			int iCharIndex = ::SendMessage(m_hWndEdit,EM_CHARFROMPOS,(WPARAM)0,(LPARAM)MAKELPARAM(pt.x,pt.y));
			int iCharsToLine = ::SendMessage(m_hWndEdit,EM_LINEINDEX,(WPARAM)HIWORD(iCharIndex),(LPARAM)0);

			// Copy in the line
			VARIANT varLine;
			varLine.vt = VT_I4;
			varLine.lVal = (HIWORD(iCharIndex)) + 1; // VBScript is 1-based
			::VariantCopy(pLineVal,&varLine);

			// Copy in the index
			VARIANT varIndex;
			varIndex.vt = VT_I4;
			varIndex.lVal = LOWORD(iCharIndex) - iCharsToLine;
			::VariantCopy(pIndexVal,&varIndex);
		} // if
		else {
			// Weren't valid pointers
			hr = E_POINTER;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::SetCursorPos(VARIANT lineVal, VARIANT indexVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant varLine;
		CComVariant varIndex;
		if ( SUCCEEDED(varLine.ChangeType(VT_I4,&lineVal)) &&
				SUCCEEDED(varIndex.ChangeType(VT_I4,&indexVal)) ) {
			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Determine where they want it
				int iCharsToLine = ::SendMessage(m_hWndEdit,EM_LINEINDEX,(WPARAM)(LOWORD(lineVal.lVal)-1),(LPARAM)0); // VBScript is 1-based

				// Place it there
				::SendMessage(m_hWndEdit,EM_SETSEL,(WPARAM)(LOWORD(indexVal.lVal)+iCharsToLine),(LPARAM)(LOWORD(indexVal.lVal)+iCharsToLine));
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return S_OK;
}

STDMETHODIMP CTextSelObject::Cut()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndEdit,WM_CUT,(WPARAM)0,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::Copy()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndEdit,WM_COPY,(WPARAM)0,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::Paste()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Pull the new value
			::SendMessage(m_hWndEdit,WM_PASTE,(WPARAM)0,(LPARAM)0);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::SelectAll()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Select all of the text
			::SendMessage(m_hWndEdit,EM_SETSEL,(WPARAM)0,(LPARAM)-1);
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::SelectRange(VARIANT lowerVal, VARIANT upperVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant varLower;
		CComVariant varUpper;
		if (SUCCEEDED(varLower.ChangeType(VT_I4,&lowerVal)) && 
			SUCCEEDED(varUpper.ChangeType(VT_I4,&upperVal))) {
			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Select the given range
				DWORD dwLower = varLower.lVal;
				DWORD dwUpper = varUpper.lVal;
				::SendMessage(m_hWndEdit,EM_SETSEL,(WPARAM)dwLower,(LPARAM)dwUpper);
			} // else
		} // if
		else {
			// Wasn't some sort of integer we could
			// coerce.
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::InsertText(VARIANT newVal)
{
	HRESULT hr = S_OK;
	try {
		// Coerce the variant
		CComVariant var;
		if ( SUCCEEDED(var.ChangeType(VT_BSTR,&newVal)) ) {
			if ( !::IsWindow(m_hWndEdit) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Pull the new caption
				CComBSTR bstrCaption(newVal.bstrVal);
#ifndef _UNICODE
				// For non-Unicode versions, we must convert the
				// BSTR value to ANSI.
				USES_CONVERSION;
				LPTSTR szCaption = OLE2T(newVal.bstrVal);
				::SendMessage(m_hWndEdit,EM_REPLACESEL,(WPARAM)FALSE,(LPARAM)(LPCTSTR)szCaption);
#else
				// For Unicode versions, we can simply use the
				// BSTR.
				::SendMessage(m_hWndEdit,EM_REPLACESEL,(WPARAM)FALSE,(LPARAM)newVal.bstrVal);
#endif
			} // else
		} // if
		else {
			// Wasn't some sort of variant we could
			// coerce
			hr = E_FAIL;
		} // if
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::ClearAll()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Replace the text with nothing
#ifndef _UNICODE
			TCHAR szReplace[] = {_T("")};
			::SendMessage(m_hWndEdit,WM_SETTEXT,(WPARAM)0,(LPARAM)szReplace);
#else
			WCHAR wszReplace[] = L"";
			::SendMessage(m_hWndEdit,WM_SETTEXT,(WPARAM)0,(LPARAM)wszReplace);
#endif
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::ClearSelection()
{
	HRESULT hr = S_OK;
	try {
		// Clear the existing text
		if ( !::IsWindow(m_hWndEdit) ) {
			// Not a valid HWND...
			ATLASSERT(FALSE);
			hr = E_POINTER;
		} // if
		else {
			// Replace the text with nothing
#ifndef _UNICODE
			TCHAR szReplace[] = {_T("")};
			::SendMessage(m_hWndEdit,EM_REPLACESEL,(WPARAM)0,(LPARAM)szReplace);
#else
			WCHAR wszReplace[] = L"";
			::SendMessage(m_hWndEdit,EM_REPLACESEL,(WPARAM)0,(LPARAM)wszReplace);
#endif
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}

STDMETHODIMP CTextSelObject::InitHwnd(OLE_HANDLE hWnd)
{
	HRESULT hr = S_OK;
	try {
		if ( hWnd ) {
			// Initialize the window handle attribute
			if ( !::IsWindow(reinterpret_cast<HWND>(hWnd)) ) {
				// Not a valid HWND...
				ATLASSERT(FALSE);
				hr = E_POINTER;
			} // if
			else {
				// Assign our attribute
				m_hWndEdit = reinterpret_cast<HWND>(hWnd);

				// Subclass the window
				if ( !m_CEditCtrl.SubclassWindow(m_hWndEdit) ) {
					// We failed to subclass the window!
					ATLASSERT(FALSE);
					hr = E_UNEXPECTED;
				} // if
			} // else
		} // if
		else {
			// Invalid HWND
			hr = E_INVALIDARG;
		} // else
	} // try
	catch (...) {
		// Some error...
		hr = E_FAIL;
	} // catch

	return hr;
}
