//-----------------------------------------------------------------
// Shape Object
// C++ Source - Shape.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Inclusions
//-----------------------------------------------------------------
#include "StdAfx.h"
//-----------------------------------------------------------------
#include "Shape.h"
#include "Resource.h"

//-----------------------------------------------------------------
// MFC Debugging Support
//-----------------------------------------------------------------
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------
// Serialization Support
//-----------------------------------------------------------------
IMPLEMENT_SERIAL(CShape, CObject, 1)

//-----------------------------------------------------------------
// Public Constructor(s)/Destructor
//-----------------------------------------------------------------
CShape::CShape() {
}

CShape::CShape(int nType, CRect& rcPosition, COLORREF crColor) {
  m_nType = nType;
  m_rcPosition = rcPosition;
  m_crColor = crColor;
}

//-----------------------------------------------------------------
// Member Functions
//-----------------------------------------------------------------
void CShape::Draw(CDC* pDC) {
  ASSERT_VALID(pDC);

  // Select the pen and brush
  CPen  pen(PS_SOLID, 2, m_crColor);
  CPen* ppenOld;
  CBrush* pbrOld;
  VERIFY(ppenOld = pDC->SelectObject(&pen));
  VERIFY(pbrOld = pDC->SelectObject(
    CBrush::FromHandle((HBRUSH)::GetStockObject(NULL_BRUSH))));

  // Calculate shape bounds
  CPoint ptTopCenter = m_rcPosition.TopLeft();
  ptTopCenter.Offset(m_rcPosition.Width() / 2, 0);
  CPoint ptBottomLeft = m_rcPosition.BottomRight();
  ptBottomLeft.Offset(-m_rcPosition.Width(), 0);

  // Draw shape based on the type
  switch (m_nType) {
  case LINE:
    // Draw a line
    pDC->MoveTo(m_rcPosition.TopLeft());
    pDC->LineTo(m_rcPosition.BottomRight());
    break;
  
  case RECTANGLE:
    // Draw a rectangle
    pDC->Rectangle(m_rcPosition);
    break;

  case TRIANGLE:
    // Draw a triangle
    pDC->MoveTo(ptTopCenter);
    pDC->LineTo(ptBottomLeft);
    pDC->LineTo(m_rcPosition.BottomRight());
    pDC->LineTo(ptTopCenter);
    break;

  case ELLIPSE:
    // Draw an ellipse
    pDC->Ellipse(m_rcPosition);
    break;
  }

  // Cleanup the DC
  VERIFY(pDC->SelectObject(ppenOld));
  VERIFY(pDC->SelectObject(pbrOld));
}

void CShape::Serialize(CArchive& ar) {
  if (ar.IsStoring()) {
    ar << (WORD)m_nType;
    ar << m_rcPosition;
    ar << m_crColor;
  }
  else {
    WORD w;
    ar >> w;
    m_nType = w;
    ar >> m_rcPosition;
    ar >> m_crColor;
  }
}

CString CShape::GetTypeAsString() {
  switch (m_nType) {
  case LINE:
    return CString("Line");

  case RECTANGLE:
    return CString("Rectangle");

  case TRIANGLE:
    return CString("Triangle");

  case ELLIPSE:
    return CString("Ellipse");

  default:
    return CString("Unknown");
  }
}
