//-----------------------------------------------------------------
// Shaper Document Object
// C++ Source - ShaperDoc.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Inclusions
//-----------------------------------------------------------------
#include "StdAfx.h"
//-----------------------------------------------------------------
#include "ShaperDoc.h"
#include "Resource.h"

//-----------------------------------------------------------------
// MFC Debugging Support
//-----------------------------------------------------------------
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------
// Message Map & Runtime Support
//-----------------------------------------------------------------
BEGIN_MESSAGE_MAP(CShaperDoc, CDocument)
  ON_COMMAND(ID_EDIT_CLEAR_ALL, OnEditClearAll)
  ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR_ALL, OnUpdateEditClearAll)
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CShaperDoc, CDocument)

//-----------------------------------------------------------------
// Public Constructor(s)/Destructor
//-----------------------------------------------------------------
CShaperDoc::CShaperDoc() : m_sizDoc(800, 600) {
}

CShaperDoc::~CShaperDoc() {
}

//-----------------------------------------------------------------
// Public Member Functions
//-----------------------------------------------------------------
BOOL CShaperDoc::OnNewDocument() {
  if (!CDocument::OnNewDocument())
    return FALSE;
  UpdateAllViews(NULL);
  return TRUE;
}

BOOL CShaperDoc::OnOpenDocument(LPCTSTR lpszPathName) {
  if (!CDocument::OnOpenDocument(lpszPathName))
    return FALSE;
  UpdateAllViews(NULL, 0L, &m_shapeList);
  return TRUE;
}

void CShaperDoc::DeleteContents() {
  // Cleanup shape list
  while (!m_shapeList.IsEmpty())
    delete m_shapeList.RemoveHead();
}

void CShaperDoc::Serialize(CArchive& ar) {
  // Store/retrieve the document size
  if (ar.IsStoring()) {
    ar << m_sizDoc;
  }
  else {
    ar >> m_sizDoc;
  }
  
  // Store/retrieve shape list
  m_shapeList.Serialize(ar);
}

void CShaperDoc::AddShape(CShape* pShape) {
  // Add a new shape to end of shape list
  ASSERT_VALID(pShape);
  m_shapeList.AddTail(pShape);

  // Enlarge the document size if necessary
  CSize sizShape = pShape->GetPosition().BottomRight();
  sizShape.cy = -sizShape.cy;
  if ((sizShape.cx > m_sizDoc.cx) || (sizShape.cy > m_sizDoc.cy)) {
    m_sizDoc.cx = max(m_sizDoc.cx, sizShape.cx);
    m_sizDoc.cy = max(m_sizDoc.cy, sizShape.cy);
  }

  // Flag document as modified and update view
  SetModifiedFlag();
  UpdateAllViews(NULL, 0L, pShape);
}

//-----------------------------------------------------------------
// Message Handlers
//-----------------------------------------------------------------
void CShaperDoc::OnEditClearAll() {
  // Clear the document
  DeleteContents();
  SetModifiedFlag();
  UpdateAllViews(NULL);
}

void CShaperDoc::OnUpdateEditClearAll(CCmdUI* pCmdUI) {
  // Update the Edit|Clear All menu item
  pCmdUI->Enable(!m_shapeList.IsEmpty());
}

//-----------------------------------------------------------------
// Diagnostic Functions
//-----------------------------------------------------------------
#ifdef _DEBUG
void CShaperDoc::AssertValid() const {
  CDocument::AssertValid();
}

void CShaperDoc::Dump(CDumpContext& dc) const {
  CDocument::Dump(dc);
}
#endif //_DEBUG
