// TryODBCRecordset.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "BooksRecordset.h"
#include "TryODBCRecordset.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

static void getRecords(CDatabase *db)
{
   CBooksRecordset rst(db);

   rst.Open();

   while (!rst.IsEOF()) {
      cout << (const TCHAR *) rst.m_ISBN  << _T("\t") 
           << (const TCHAR *) rst.m_Title << endl;
      rst.MoveNext();
   }

   rst.Close();
}

static void addRecord(CDatabase *db)
{
   CBooksRecordset rst(db);

   rst.Open(CRecordset::dynaset);

   rst.AddNew();

   rst.m_BookId      = 123;
   rst.m_ISBN        = "1234567890";
   rst.m_Title       = "MFC Unleashed";
   rst.m_RetailPrice = "49.99";

   rst.Update();

   cout << _T("added record") << endl;

   rst.Close();
}

static void editRecord(CDatabase *db)
{
   CBooksRecordset rst(db);

   rst.Open(CRecordset::dynaset);

   while (!rst.IsEOF()) {
      if (_tcscmp(rst.m_ISBN, _T("1234567890")) == 0) {
         rst.Edit();
         rst.m_ISBN = "0987654321";
         rst.Update();

         cout << _T("edited record") << endl;
      }
      rst.MoveNext();
   }

   rst.Close();
}

static void deleteRecord(CDatabase *db)
{
   CBooksRecordset rst(db);

   rst.Open(CRecordset::dynaset);

   while (!rst.IsEOF()) {
      if ((_tcscmp(rst.m_ISBN, _T("1234567890")) == 0) 
          || (_tcscmp(rst.m_ISBN, _T("0987654321")) == 0)) {
         rst.Delete();

         cout << _T("deleted record") << endl;
      }
      rst.MoveNext();
   }

   rst.Close();
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{
      enum Action {
         None,
         Get,
         Add,
         Edit,
         Delete
      };

      Action action = None;

      if (argc == 3) {
         if (_tcscmp(argv[2], _T("get")) == 0) {
            action = Get;
         }
         else if (_tcscmp(argv[2], _T("add")) == 0) {
            action = Add;
         }
         else if (_tcscmp(argv[2], _T("edit")) == 0) {
            action = Edit;
         }
         else if (_tcscmp(argv[2], _T("delete")) == 0) {
            action = Delete;
         }
      }

      if (action == None) {
         cerr << _T("usage: TryODBCRecordset <mdb name> <get|add|edit|delete>") << endl;
         return 0;
      }

      try {
         CDatabase db;

         CString connect;

         connect += "UID=Admin;";
         connect += "DRIVER={Microsoft Access Driver (*.mdb)};";
         connect += "DBQ=";
         connect += argv[1];

         db.OpenEx(connect);

         switch (action) {
         case Get:
            getRecords(&db);
            break;
         case Add:
            addRecord(&db);
            break;
         case Edit:
            editRecord(&db);
            break;
         case Delete:
            deleteRecord(&db);
            break;
         }

         db.Close();
      }
      catch (CDBException *xcp) {
         cerr << _T("Database exception: ") << (const TCHAR *) xcp->m_strError << endl;
         xcp->Delete();
      }
	}

	return nRetCode;
}


