// TryADO.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Books.h"
#include "TryADO.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

static void getRecords(ADORecordset *rst, CBooks& books)
{
   VARIANT_BOOL isEOF;

   while (true) {
      rst->get_EOF(&isEOF);

      if (isEOF == VARIANT_TRUE) {
         break;
      }

      cout << (const TCHAR *) books.m_ISBN  << _T("\t") 
           << (const TCHAR *) books.m_Title << endl;
      
      rst->MoveNext();
   }
}

static void addRecord(ADORecordset *rst, 
                      IADORecordBinding *rstADOBind,
                      CBooks& books)
{
   books.ClearRecord();

   books.m_BookId = 123;
   books.m_CategoryId = 1;
   _tcscpy(books.m_ISBN, _T("1234567890"));
   books.m_PublicationDate = 39000.0;
   books.m_PublisherId = 1;
   books.m_RetailPrice.int64 = (49.99) * 10000;
   _tcscpy(books.m_Title, _T("MFC Unleashed"));
   books.m_TopicId = 1;

   rstADOBind->AddNew(&books);

   cout << _T("added record") << endl;
}

static void editRecord(ADORecordset *rst, 
                       IADORecordBinding *rstADOBind,
                       CBooks& books)
{
   VARIANT_BOOL isEOF;

   while (true) {
      rst->get_EOF(&isEOF);

      if (isEOF == VARIANT_TRUE) {
         break;
      }

      if (_tcscmp(books.m_ISBN, _T("1234567890")) == 0) {
         _tcscpy(books.m_ISBN, _T("0987654321"));

         rstADOBind->Update(&books);

         cout << _T("edited record") << endl;
      }

      rst->MoveNext();
   }
}

static void deleteRecord(ADORecordset *rst, CBooks& books)
{
   VARIANT_BOOL isEOF;

   while (true) {
      rst->get_EOF(&isEOF);

      if (isEOF == VARIANT_TRUE) {
         break;
      }

      if ((_tcscmp(books.m_ISBN, _T("1234567890")) == 0) 
          || (_tcscmp(books.m_ISBN, _T("0987654321")) == 0)) {
         rst->Delete(adAffectCurrent);

         cout << _T("deleted record") << endl;
      }

      rst->MoveNext();
   }
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
   int nRetCode = 0;

   CoInitialize(NULL);

   // initialize MFC and print and error on failure
   if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
   {
      // TODO: change error code to suit your needs
      cerr << _T("Fatal Error: MFC initialization failed") << endl;
      nRetCode = 1;
   }
   else
   {
      enum Action {
         None,
         Get,
         Add,
         Edit,
         Delete
      };

      Action action = None;

      if (argc == 3) {
         if (_tcscmp(argv[2], _T("get")) == 0) {
            action = Get;
         }
         else if (_tcscmp(argv[2], _T("add")) == 0) {
            action = Add;
         }
         else if (_tcscmp(argv[2], _T("edit")) == 0) {
            action = Edit;
         }
         else if (_tcscmp(argv[2], _T("delete")) == 0) {
            action = Delete;
         }
      }

      if (action == None) {
         cerr << _T("usage: TryADO <mdb name> <get|add|edit|delete>") << endl;
         CoUninitialize();
         return 1;
      }

      HRESULT hr;
      ADORecordset *rstADO;

      hr = CoCreateInstance(
                           //"{00000535-0000-0010-8000-00AA006D2EA4}",
                            CLSID_CADORecordset,
                            NULL,
                            CLSCTX_INPROC_SERVER,
                            IID_IADORecordset,
                            (LPVOID *) &rstADO);

      if (FAILED(hr)) {
         cerr << "Unable to create ADORecordset object." << endl;
         CoUninitialize();
         return 1;
      }

      CString table("Books");
      CString connect;

      connect += "DRIVER={Microsoft Access Driver (*.mdb)};";
      connect += "UID=Admin;";
      connect += "DBQ=";
      connect += argv[1];

      hr = rstADO->Open(COleVariant(table), 
                        COleVariant(connect),
                        adOpenKeyset, adLockOptimistic,
                        adCmdTable);

      if (FAILED(hr)) {
         cerr << "Unable to open ADORecordset object." << endl;
         CoUninitialize();
         return 1;
      }

      IADORecordBinding *rstADOBind;

      hr = rstADO->QueryInterface(__uuidof(IADORecordBinding), (LPVOID *) &rstADOBind);

      if (FAILED(hr)) {
         cerr << "Unable to obtain IADORecordBinding interface." << endl;
         CoUninitialize();
         return 1;
      }

      CBooks books;

      rstADOBind->BindToRecordset(&books);

      switch (action) {
      case Get:
         getRecords(rstADO, books);
         break;
      case Add:
         addRecord(rstADO, rstADOBind, books);
         break;
      case Edit:
         editRecord(rstADO, rstADOBind, books);
         break;
      case Delete:
         deleteRecord(rstADO, books);
         break;
      }

      rstADO->Close();
      rstADOBind->Release();
   }

   CoUninitialize();

   return nRetCode;
}
