// TryOLEDB.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Books.h"
#include "TryOLEDB.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

wchar_t *g_mdbLocation = L"TechBooks.mdb";

static void getRecords()
{
   CBooks books;

   if (FAILED(books.Open())) {
      cerr << "Error in books.Open()" << endl;
      return;
   }

   while (books.MoveNext() == S_OK) {
      cout << (const TCHAR *) books.m_ISBN  << _T("\t") 
           << (const TCHAR *) books.m_Title << endl;
   }

   books.Close();
}

static void addRecord()
{
   CBooks books;

   if (FAILED(books.Open())) {
      cerr << "Error in books.Open()" << endl;
      return;
   }

   books.ClearRecord();

   books.m_BookId = 123;
   _tcscpy(books.m_ISBN,  "1234567890");
   _tcscpy(books.m_Title, "MFC Unleashed");
   books.m_RetailPrice.int64 = (49.99) * 10000;

   if (FAILED(books.Insert())) {
      cerr << "Error in books.Insert()" << endl;
   }
   else {
      cout << _T("added record") << endl;
   }

   books.Close();
}

static void editRecord()
{
   CBooks books;

   if (FAILED(books.Open())) {
      cerr << "Error in books.Open()" << endl;
      return;
   }

   while (books.MoveNext() == S_OK) {
      if (_tcscmp(books.m_ISBN, _T("1234567890")) == 0) {
         _tcscpy(books.m_ISBN, _T("0987654321"));
     
         // set to use accessor #1, which only references
         // the ISBN column--since that's the only column
         // that we're updating...
         books.SetData(1);

         books.Update();
         cout << _T("edited record") << endl;
      }
   }

   books.Close();
}

static void deleteRecord()
{
   CBooks books;

   if (FAILED(books.Open())) {
      cerr << "Error in books.Open()" << endl;
      return;
   }

   while (books.MoveNext() == S_OK) {
      if ((_tcscmp(books.m_ISBN, _T("1234567890")) == 0) 
          || (_tcscmp(books.m_ISBN, _T("0987654321")) == 0)) {

         if (FAILED(books.Delete())) {
            cerr << "Error in books.Delete()" << endl;
            return;
         }
         else {
            cout << _T("deleted record") << endl;
         }
      }
   }

   books.Close();
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
   CoInitialize(NULL);

   int nRetCode = 0;

   // initialize MFC and print and error on failure
   if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
   {
      // TODO: change error code to suit your needs
      cerr << _T("Fatal Error: MFC initialization failed") << endl;
      nRetCode = 1;
   }
   else
   {
      enum Action {
         None,
         Get,
         Add,
         Edit,
         Delete
      };

      Action action = None;

      if (argc == 2) {
         if (_tcscmp(argv[1], _T("get")) == 0) {
            action = Get;
         }
         else if (_tcscmp(argv[1], _T("add")) == 0) {
            action = Add;
         }
         else if (_tcscmp(argv[1], _T("edit")) == 0) {
            action = Edit;
         }
         else if (_tcscmp(argv[1], _T("delete")) == 0) {
            action = Delete;
         }
      }

      if (action == None) {
         cerr << _T("usage: TryOLEDB <get|add|edit|delete>") << endl;
         CoUninitialize();
         return 0;
      }

      switch (action) {
      case Get:
         getRecords();
         break;
      case Add:
         addRecord();
         break;
      case Edit:
         editRecord();
         break;
      case Delete:
         deleteRecord();
         break;
      }
   }

   CoUninitialize();

   return nRetCode;
}


