/*************************************************************************
**      CX_EventRecorder.cpp:                                           **
**      ============================================================    **
**      This is the implementation file for the CXParticipant,			**
**		CXEvent, and CXEventRecorder classes...						    **
*************************************************************************/

#include "cx_eventrecorder.h"

#include "UNL_MultiEd.h"

const CString sEMPTY_STRING  = "";

/*************************************************************************
**  This utility function will take a delimited string and split it     **
**  into parsed strings based on the delimiter.  Useful for parsing     **
**  COMMA delimited strings...                                          **
**                                                                      **
**      NOTE:   This is strictly the MFC version..                      **
*************************************************************************/
void 
SplitPartsString( const CString&   cr_sLine, 
                  CStringList&     r_lParts,
                  const CString&   cr_sDelimiter)
{
    CString  sPart          = "";
    CString  sTemp;

    int      iLineLength = 0;
    int      iCurrentPos = 0;
    int      iTmpPos     = 0;

    r_lParts.RemoveAll();

    sTemp =  cr_sLine;
    iLineLength = sTemp.GetLength();

    /*********************************
    **      Main parsing loop..     **
    *********************************/
    while (0 != iLineLength)
    {

        /*************************************************
        **  We should have a delimiter in this string   **
        **  ..if not, then jump to else loop comment    **
        *************************************************/
        if (-1 != (iTmpPos = sTemp.Find(cr_sDelimiter)))
        {
            iCurrentPos = iTmpPos;
            
            sPart = sTemp.Left(iCurrentPos);

            if (CString("\t") == sTemp.Right(iLineLength - iCurrentPos))
            {
                sTemp += " ";iLineLength++;
            }
            sTemp = sTemp.Right(iLineLength - iCurrentPos - 1);
            
            //  We have a clean partial string.. add it to the StringList to
            //  pass back.
            r_lParts.AddTail(sPart);
        }
        else
        {
            /*********************************************************
            **      OK.. We are out of delimiters in the main       **
            **      string.  Make the rest of the string the        **
            **      last part                                       **
            *********************************************************/
            sPart = sTemp; // Set our part to the remainder of the string..
                
            sTemp = sEMPTY_STRING;

            if (sEMPTY_STRING != sPart)
            {
		        // This section of code will strip off our \255 character that 
		        // the MFC_CSTR_KLUDGE macro put in to fix the infamous 
		        //	MFC StdioFile::ReadString buffer bug...
                if (-1 != (iTmpPos = sPart.Find('\255')))
			        sPart = sPart.Left(iTmpPos-1);

                r_lParts.AddTail(sPart);
            }
        }
    }
}



CXParticipant::CXParticipant() 
{
    SetFirstName("");
    SetLastName("");
    SetTeam("");
}

CXParticipant::CXParticipant(CString sLastName)
			  : m_sLastName(sLastName)
{
}

CXParticipant::CXParticipant(CString sLastName, CString sFirstName)
              : m_sLastName(sLastName),
			    m_sFirstName(sFirstName)
{
}

CXParticipant::CXParticipant(CString sLastName, CString sFirstName, CString sTeam)
              : m_sLastName(sLastName),
			    m_sFirstName(sFirstName),
                m_sTeam(sTeam)
{
}

CXParticipant::~CXParticipant()
{
}

void 
CXParticipant::SetFirstName(CString sFirstName)
{
    m_sFirstName = sFirstName;
}

void
CXParticipant::SetLastName(CString sLastName)
{
    m_sLastName = sLastName;
}

void
CXParticipant::SetTeam(CString sTeam)
{
    m_sTeam = sTeam;
}
    
CString 
CXParticipant::GetFirstName()
{
    return m_sFirstName;
}

CString
CXParticipant::GetLastName()
{
    return m_sLastName;
}

CString 
CXParticipant::GetTeam()
{
    return m_sTeam;
}

void
CXParticipant::AddPoints(int nPoints)
{
    m_nRunningPoints += nPoints;
}

void
CXParticipant::BumpEvent()
{
    m_nEventsEntered++;
}

int
CXParticipant::GetRunningTotal()
{
    return m_nRunningPoints;
}

int
CXParticipant::GetEventsEntered()
{
    return m_nEventsEntered;
}



/*************************************************************
**      The Event class.  Defines the event ran..           **
*************************************************************/

CXEvent::CXEvent()
{
    SetEventName("");
    SetEventRan(FALSE);
	m_slEventParticipants = new CList <CXParticipant *, CXParticipant *> ();
}

CXEvent::~CXEvent()
{
}


CXEvent& 
CXEvent::operator = (const CXEvent& pOther)
{
    return *this;
}

int
CXEvent::operator == (CXEvent *pOther)
{
	return m_sName == (pOther->m_sName);
}


void 
CXEvent::SetEventName (CString sName)
{
    m_sName = sName;
}

CString
CXEvent::GetEventName()
{
    return m_sName;
}

void 
CXEvent::SetEventRan (BOOL bEventRan)
{
    m_bEventRan = bEventRan;
}
        
BOOL
CXEvent::GetEventRan()
{
    return m_bEventRan;
}

void
CXEvent::AddParticipantToEvent (CXParticipant * eventParticipant)
{
	POSITION pos = m_slEventParticipants->GetHeadPosition( );
	while( pos )
	{
        //Already in the list.. Don't do anything...
        if (m_slEventParticipants->GetNext(pos) == eventParticipant)
            return;
	}
    //  Ok.. We aren't a duplicate.. add to our list..
    m_slEventParticipants->AddTail(eventParticipant);
}

void
CXEvent::RemoveParticipant (CXParticipant * eventParticipant)
{
	POSITION pos = m_slEventParticipants->GetHeadPosition( );
	while( pos )
	{
        if (m_slEventParticipants->GetNext(pos) == eventParticipant)
            m_slEventParticipants->RemoveAt(pos);
	}
}

void 
CXEvent::PlaceParticipantInEvent (int nPos, CXParticipant * ourParticipant)
{
    m_mEventPlaces->SetAt(0,ourParticipant);
}

        
        
/*****************************************************************
**      Main global class for the UNL_MultiEd application.      **
**      ==================================================      **
**      Contains the maps and list for maintaining the          **
**      event placing... also contains the calculations         **
**      for determining overall winner/athlete..                **
*****************************************************************/

CXEventRecorder::CXEventRecorder()
{
  	m_plParticipants = new CList<CXParticipant *,CXParticipant *> ();
    m_elEvents = new CList<CXEvent *, CXEvent *> ();

}

CXEventRecorder::~CXEventRecorder()
{
}

/*****************************************************************
**      This function will take an ERX file and parse it in     **
**      placing the events in their respective list, and the    **
**      participants in their list.                             **
*****************************************************************/
void
CXEventRecorder::LoadEvents(CString sFileName)
{
    BOOL    bDO_EVENT           = FALSE;
    BOOL    bDO_PARTICIPANT     = FALSE;

	CStdioFile file;
	CString line;

    if (file.Open(sFileName,CFile::modeRead))
	{
		while (file.ReadString(line))
		{
            if (line == "EVENT:")
            {
                bDO_EVENT       = TRUE;
                bDO_PARTICIPANT = FALSE;
            }
            else if (line == "PARTICIPANT:")
            {
                bDO_EVENT       = FALSE;
                bDO_PARTICIPANT = TRUE;
            }
			else
            {
    			if (bDO_EVENT)
	            {
	                CXEvent *ourEvent = new CXEvent();
	                ourEvent->SetEventName(line);

                    AddEvent(ourEvent);
                }

                if (bDO_PARTICIPANT)
                {
                    CXParticipant *ourParticipant = new CXParticipant();
                    CString sLastName,sFirstName,sTeam;

                    CStringList slParts;
                    SplitPartsString(line, slParts, ",");

                    // The first token should be the last name
                    POSITION pPartPos = slParts.FindIndex(0);
                    sLastName = slParts.GetAt(pPartPos);

                    // Next we have the first name..
                    pPartPos = slParts.FindIndex(1);
                    sFirstName = slParts.GetAt(pPartPos);

                    //Next we have the Team name
                    pPartPos = slParts.FindIndex(1);
                    sTeam = slParts.GetAt(pPartPos);

                    ourParticipant->SetLastName(sLastName);
                    ourParticipant->SetFirstName(sFirstName);
                    ourParticipant->SetTeam(sTeam);

                    AddParticipant(ourParticipant);
                }
            }
		}
		file.Close();
	}


}
/*********************************************************************
**      Called from LoadEvents or from the Participant view to      **
**      create a new CXParticipant and add him to the list...       **
*********************************************************************/
void 
CXEventRecorder::LoadParticipants (CString sLastName,
                                   CString sFirstName,
                                   CString sTeam)
{
    CXParticipant *ourParticipant = new CXParticipant ( sLastName,
                                                        sFirstName,
                                                        sTeam );
    m_plParticipants->AddTail(ourParticipant);
}


void 
CXEventRecorder::SetEvent(int nEventNum, CString sEventName)
{
    m_sEventName    = sEventName;
    m_nActiveEvent  = nEventNum;
}


CString 
CXEventRecorder::GetEventName ()
{
    return m_sEventName;
}

int
CXEventRecorder::GetEventNumber()
{
    return m_nActiveEvent;
}

void 
CXEventRecorder::CalculatePoints ()
{
}

void
CXEventRecorder::AddParticipant(CXParticipant *ourParticipant)
{
    m_plParticipants->AddTail(ourParticipant);
}

void 
CXEventRecorder::RemoveParticipant(CXParticipant *ourParticipant)
{
	POSITION pos = m_plParticipants->GetHeadPosition( );
	while( pos )
    {
        if (m_plParticipants->GetNext(pos) == ourParticipant)
            m_plParticipants->RemoveAt(pos);
        break;
    }

}

void
CXEventRecorder::AddEvent(CXEvent *ourEvent)
{
    m_elEvents->AddTail(ourEvent);
}

void 
CXEventRecorder::RemoveEvent(CXEvent *ourEvent)
{
	POSITION pos = m_elEvents->GetHeadPosition( );
	while( pos )
    {
        if (m_elEvents->GetNext(pos) == ourEvent)
            m_elEvents->RemoveAt(pos);
        break;
    }

}

CList <CXEvent *, CXEvent *>
*CXEventRecorder::GetEvents ()
{
    return m_elEvents;
}

CList <CXParticipant *, CXParticipant *>
*CXEventRecorder::GetParticipants ()
{
    return m_plParticipants;
}


