// xeventrecorder.h

#ifndef cxeventrecorder_h
#define cxeventrecorder_h

#include "stdafx.h"
#include <stdio.h>
#include <afxtempl.h>


/*************************************************************************
**      CXParticipant:                                                  **
**      ============================================================    **
**      This class stores information about the participants in the     **
**      Games..    The constructors will insert the information...      **
**      This class is added in the map for the EventRecorder class..    **
*************************************************************************/

class CXParticipant
{
    public:

        CXParticipant();
        CXParticipant(CString sLastName);
        CXParticipant(CString sLastName, CString sFirstName);
        CXParticipant(CString sLastName, CString sFirstName, CString sTeam);

        virtual ~CXParticipant();

        void SetFirstName(CString sFirstName);
        void SetLastName(CString sLastName);
        void SetTeam(CString sTeam);
    
        CString GetFirstName();
        CString GetLastName();
        CString GetTeam();

        void AddPoints(int nPoints);
        void BumpEvent();

        int GetRunningTotal();
        int GetEventsEntered();

    private:

        CString     m_sLastName;
        CString     m_sFirstName;
        CString     m_sTeam;

        int         m_nEventsEntered;
        int         m_nRunningPoints;
};

/*************************************************************************
**      CXEvent:                                                        **
**      ============================================================    **
**      This class stores information about the event..  However, it    **
**      only stores whether it was competed..                           **
*************************************************************************/

class CXEvent
{
    public:

        CXEvent();
        virtual ~CXEvent();

		CXEvent&	operator = (const CXEvent& pOther);
		int			operator == (CXEvent *pOther);

        void SetEventName (CString sName);
        CString GetEventName ();

        void SetEventRan (BOOL bEventRan);
        BOOL GetEventRan ();

        void AddParticipantToEvent(CXParticipant * eventParticipant);
        void RemoveParticipant(CXParticipant * eventParticipant);

        /*********************************************
        **  This method will place the participant  **
        **  in the event (First, Second, etc..)     **
        *********************************************/
        void PlaceParticipantInEvent (int nPos, CXParticipant * ourParticipant);

    private:

        CString     m_sName;
        BOOL        m_bEventRan;

        CList <CXParticipant *, CXParticipant *>            *m_slEventParticipants;
        CMap <int, int, CXParticipant *, CXParticipant *>   *m_mEventPlaces;
};


/*****************************************************************************
**      CXEventRecorder Class..                                             **
**      ===============================================================     **
**      This class is a global store for the games...                       **
**      Maps are kept of events and participants and event totals           **
**      It also contains information regarding the reporting of the events  **
*****************************************************************************/
class CXEventRecorder
{
    public:
        
		CXEventRecorder();
        virtual ~CXEventRecorder();

        /*********************************************
        **  This will take an ASCII delimited file  **
        **  containing the events to take place     **
        **  and load them into the Event Map..      **
        *********************************************/
        void LoadEvents(CString sFileName);

        void LoadParticipants ( CString sLastName,
                                CString sFirstName,
                                CString sTeam);

        void SetEvent(int nEventNum, CString sEventName);
        CString GetEventName ();
        int GetEventNumber();

        CList <CXEvent *, CXEvent *>                *GetEvents ();
        CList <CXParticipant *, CXParticipant *>    *GetParticipants ();

        void AddParticipant (CXParticipant * ourParticipant);
        void AddEvent(CXEvent * ourEvent);

        void RemoveParticipant (CXParticipant *ourParticipant);
        void RemoveEvent (CXEvent *ourEvent);

        /*********************************************
        **  Using the Running Totals structure..    **
        **  this routine will clear and then recalc **
        **  the m_TotalPlaceMap map                 **
        *********************************************/
        void CalculatePoints ();

    private:

        /*****************************************************************
        **      We need to keep a list of Events and a list of          **
        **      Participants.  The Event class will place the           **
        **      Participants in that event.  The m_plParticipants       **
        **      map will be used to calculate overall winning           **
        **      participant.                                            **
        *****************************************************************/
        CList <CXParticipant*, CXParticipant*>    *m_plParticipants;
        CList <CXEvent *, CXEvent *>              *m_elEvents;

        CArray <CXParticipant*, CXParticipant*>   *m_paTotalPoints;

        int         m_nActiveEvent;
        CString     m_sEventName;
};

#endif // cxeventrecorder_h
