// contextmgr.h

#ifndef contextmgr_h
#define contextmgr_h

#include "stdtypes.h"
#include "stdafx.h"
#include <afxtempl.h>

#include "cxarticle.h"
#include "cxpagespec.h"

class CXTPageSpecDoc;
class CXTArticleDoc;
class CXCPageSpecDoc;
class CXCArticleDoc;

struct CXCATArticlePubListData
{
    CString           m_sPubName;
    BOOL              m_bTemplateExist;
};


// Enumerated identifiers for each type of window (view) in JMCATS.
static enum EXCATWindowType 
{
    XWT_TEMP_PAGE,		// DocTemplate for Page Spec Template Generator
	XWT_TEMP_ARTICLE,	// DocTemplate for Article Template Generator	
	XWT_CONT_PAGE,		// DocTemplate for Page Spec Content Editor
	XWT_CONT_ARTICLE,	// DocTemplate for 
    
	// if another type is added, code in the implementation file must 
    // be updated.  Search for "UPDATE_ME" to find the appropriate code.

    XWT_NUMBER
};

///////////////////////////////////////////////////////////////////////////////
// CXCATContextManager
//
// The purpose of the context manager is to handle all operations that
// result from a change in state for the XCAT application. 
//  
// These responisbilities include:
//
//     Creation of new Documents or Views, when appropriate.
//     Maintaining the Documents, and notifing them (and through them, 
//         the views) of changes in context.
///////////////////////////////////////////////////////////////////////////////
class CXCATContextManager
{
    public:
        
		CXCATContextManager();
        virtual ~CXCATContextManager();

		CMultiDocTemplate*	GetDocTemplate(int xwtTemp);

        // Used for initial creation of views when no change in context occurs.
        void		OpenNewDocument(EXCATWindowType xwtDocType);
        
		void		OpenView(CXTPageSpecDoc* pnewDoc, EXCATWindowType xwtViewType);
        void        OpenView(CXTArticleDoc*  pnewDoc, EXCATWindowType xwtViewType);
        void        OpenView(CXCPageSpecDoc* pnewDoc, EXCATWindowType xwtViewType);
        void        OpenView(CXCArticleDoc*  pnewDoc, EXCATWindowType xwtViewType);
        
		void		CloseAllFrameWnd();

        // Called by any of the XCAT documents upon their removal.  
        void        SignalDocumentDeath(CXTPageSpecDoc* pDeceasedDoc, 
										EXCATWindowType xwtDeceased);
        void		SignalDocumentDeath(CXTArticleDoc* pDeceasedDoc, 
										EXCATWindowType xwtDeceased);
        void		SignalDocumentDeath(CXCPageSpecDoc* pDeceasedDoc, 
										EXCATWindowType xwtDeceased);
        void		SignalDocumentDeath(CXCArticleDoc* pDeceasedDoc, 
										EXCATWindowType xwtDeceased);

        BOOL        IsEditExisting();
        void        SetEditExisting(BOOL bEdit);

        BOOL        IsIsLike();
        void        SetIsLike(BOOL bIsLike);

        BOOL        IsArticleOnly();
        void        SetArticleOnly(BOOL bAOnly);

        CXArticle   *GetArticle();
        void        SetArticle(CXArticle *pArticle);

        CXPageSpec  *GetPageSpec();
        void        SetPageSpec(CXPageSpec *pPageSpec);

    private:

        //////////////////////////////////////////////////////////////
        //  CX created by the Template Generator wizard that will   //
        //  be used to create the document stuff.  Normally, the    //
        //  Document will create the CX and Xchg classes for the    //
        //  editor.  However, coming out of the Wizard, there are   //
        //  no document classes..  so this is used as a temporary   //
        //  holding place for these classes.                        //
        //////////////////////////////////////////////////////////////

        BOOL                    m_bEditExisting;
        BOOL                    m_bIsLike;
        BOOL                    m_bArticleOnly;

        CXArticle*              p_cxaTheArticle;
        CXPageSpec*             p_cxpThePageSpec;

		// Document Templates..
        CMultiDocTemplate*		map_mdtDocumentTemplates[XWT_NUMBER];

};

#endif // contextmgr_h
