//-----------------------------------------------------------------
// Projector Dialog Box Object
// C++ Source - ProjectorDlg.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Inclusions
//-----------------------------------------------------------------
#include "StdAfx.h"
//-----------------------------------------------------------------
#include <CommDlg.h>
#include "ProjectorDlg.h"
#include "AboutDlg.h"

//-----------------------------------------------------------------
// MFC Debugging Support
//-----------------------------------------------------------------
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------
// Message Map & Runtime Support
//-----------------------------------------------------------------
BEGIN_MESSAGE_MAP(CProjectorDlg, CDialog)
  ON_WM_QUERYDRAGICON()
  ON_WM_PAINT()
  ON_WM_SYSCOMMAND()
  ON_BN_CLICKED(ID_PB_LOAD, OnPBLoad)
  ON_BN_CLICKED(ID_PB_PLAY, OnPBPlay)
  ON_BN_CLICKED(ID_PB_PAUSE, OnPBPause)
  ON_BN_CLICKED(ID_PB_STOP, OnPBStop)
END_MESSAGE_MAP()

//-----------------------------------------------------------------
// Public Constructor(s)/Destructor
//-----------------------------------------------------------------
CProjectorDlg::CProjectorDlg(CWnd* pParent) :
  CDialog(CProjectorDlg::IDD, pParent) {
  // Load the application icon
  m_hIcon = AfxGetApp()->LoadIcon(IDR_PROJECTOR);
}

//-----------------------------------------------------------------
// Public Member Functions
//-----------------------------------------------------------------
BOOL CProjectorDlg::OnInitDialog() {
  CDialog::OnInitDialog();

  // Add the About menu item to the system menu
  CMenu* pSysMenu = GetSystemMenu(FALSE);
  if (pSysMenu != NULL) {
    pSysMenu->AppendMenu(MF_SEPARATOR);
    pSysMenu->AppendMenu(MF_STRING, ID_ABOUT, "&About Projector...");
  }

  // Explicitly set the icon since the main window is a dialog
  SetIcon(m_hIcon, TRUE);

  // Get a pointer to the media player control
  m_pMediaPlayer = (CMediaPlayer *)GetDlgItem(ID_MP_PLAYER);

  return TRUE;
}

//-----------------------------------------------------------------
// Message Handlers
//-----------------------------------------------------------------
HCURSOR CProjectorDlg::OnQueryDragIcon() {
  // Return the cursor for the minimized application
  return (HCURSOR)m_hIcon;
}

void CProjectorDlg::OnPaint() {
  if (IsIconic()) {
    // Manually draw the application icon
    CPaintDC dc(this);
    SendMessage(WM_ICONERASEBKGND, (WPARAM)dc.GetSafeHdc(), 0);

    // Draw the icon centered in the client rectangle
    CRect rc;
    GetClientRect(&rc);
    int x = (rc.Width() - ::GetSystemMetrics(SM_CXICON) + 1) / 2;
    int y = (rc.Height() - ::GetSystemMetrics(SM_CYICON) + 1) / 2;
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
    // Perform default painting
    CDialog::OnPaint();
}

void CProjectorDlg::OnSysCommand(UINT nID, LPARAM lParam) {
  if ((nID & 0xFFF0) == ID_ABOUT) {
    // Display the About dialog box
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  }
  else
    // Perform default processing of system menu command
    CDialog::OnSysCommand(nID, lParam);
}

void CProjectorDlg::OnPBLoad() {
  // Get the filename of the movie
  CFileDialog dlg(TRUE, "asf", "", OFN_FILEMUSTEXIST,
    "Active Streaming Format (*.asf)|*.asf|" \
    "Active Streaming Redirector (*.asx)|*.asx|" \
    "Audio Video Interleave Format (*.avi)|*.avi|" \
    "RealAudio/RealVideo (*.rm)|*.rm|" \
    "Wave Audio (*.wav)|*.wav|" \
    "All Files (*.*)|*.*||");

  // Set the movie filename in the media player
  if (dlg.DoModal() == IDOK)
    m_pMediaPlayer->SetFileName(dlg.GetPathName());
}

void CProjectorDlg::OnPBPlay() {
  // Play the movie
  m_pMediaPlayer->Play();
}

void CProjectorDlg::OnPBPause() {
  // Pause the movie
  m_pMediaPlayer->Pause();
}

void CProjectorDlg::OnPBStop() {
  // Stop playing the movie and rewind
  m_pMediaPlayer->Stop();
  m_pMediaPlayer->SetCurrentPosition(0);
}
